

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
					
					
#define float double				//this enforces double precision for all calculations (and avoids type conversions)
#define MAX_LINE_LENGTH 5000	//length, in chars, of max line in input file
#define true 1
#define false 0




/*
*******************************************************************************************************
*******************************************************************************************************
READ ME FIRST:

This source code was written by J.W. Kirchner, who holds copyright to it.  It is being made available
to the public under the Creative Commons Attribution-NonCommercial-ShareAlike License (CC BY-NC-SA).
That means you are free:
to Share  to copy, distribute and transmit the work
to Remix  to adapt the work
Under the following conditions:

Attribution  You must attribute the work in the manner specified by the author or licensor (but not in any way that suggests that they endorse you or your use of the work).
Noncommercial  You may not use this work for commercial purposes.
Share Alike  If you alter, transform, or build upon this work, you may distribute the resulting work only under the same or similar license to this one.
Any of the above conditions can be waived if you get permission from the copyright holder.

For details see:
http://creativecommons.org/licenses/by-nc-sa/3.0/
http://creativecommons.org/licenses/by-nc-sa/3.0/legalcode
*******************************************************************************************************
*******************************************************************************************************
*/









// global constants



//this routine reads time series and calculates their averages over longer and longer intervals of time,
//then calculates the RMS mean of the differences between successive averages, and its standard error
//and the variance of the successive averages, and its standard error (currently disabled / commented out)
//and the MAD (median absolute difference) between successive averages, and its confidence interval (currently disabled / commented out)
//and the variogram (currently disabled / commented out)







//	this is for RMS differences as a function of averaging scales for 7-hourly time series:

	const char infilename[100]="7hour UHF for scales.txt";
	const char filespec[100]="7hr_UHF";

//	const char infilename[100]="7hour LHF for scales.txt";
//	const char filespec[100]="7hr_LHF";

//	const char infilename[100]="7hour CR for scales.txt";
//	const char filespec[100]="7hr_CR";



	const double sampling_interval=0.2916666666667;  //should be 7.0 (days) for weekly sampling or 0.2916666666667 (=7/24 days) for 7-hour sampling

#define n_columns 45   // 45 for UHF, UHFw, LHF, LHFw, or CRw, 41 for CR		//number of data columns to read
const int first_column=48;	// 48 for UHF, UHFw, LHF, LHFw, or CRw, 44 for CR  //first data column (first data column comes after dayno)

	#define n_scales 12		// maximum number of different scales (note that the highest scale that is useful is log2(n_data) for log scales, or n_data/2 for linear scales)
//const int linear_scales=false;	//if true, averaging scales are 1, 2, 3, 4, 5, 6... times the sampling interval (and we need about 1200 scales for 7-hourly data)
							//if false, averaging scales are 1, 2, 4, 8... times the sampling interval (and we need about 12 scales for 7-hourly data)

	const float scale[n_scales] = { 0.292, 0.584, 1.167, 2.334, 4.667, 9.333, 18.667, 37.334, 74.667, 149.334, 298.667, 597.334 }; //n_scales = 12







/*

//	this is for RMS differences as a function of averaging scales for weekly time series:

	const char infilename[100]="weekly UHFw for scales.txt";
	const char filespec[100]="wkly_UHFw";

//	const char infilename[100]="weekly LHFw for scales.txt";
//	const char filespec[100]="wkly_LHFw";

//	const char infilename[100]="weekly CRw for scales.txt";
//	const char filespec[100]="wkly_CRw";

	const double sampling_interval=7.0;  //should be 7.0 (days) for weekly sampling or 0.2916666666667 (=7/24 days) for 7-hour sampling

#define n_columns 45  // 45 for UHF, UHFw, LHF, LHFw, or CRw, 41 for CR		//number of data columns to read
const int first_column=48;		// 48 for UHF, UHFw, LHF, LHFw, or CRw, 44 for CR  //first data column (first data column comes after dayno)

	#define n_scales 10		// maximum number of different scales (note that the highest scale that is useful is log2(n_data) for log scales, or n_data/2 for linear scales)
//const int linear_scales=false;	//if true, averaging scales are 1, 2, 3, 4, 5, 6... times the sampling interval (and we need about 1200 scales for 7-hourly data)
							//if false, averaging scales are 1, 2, 4, 8... times the sampling interval (and we need about 12 scales for 7-hourly data)

	const float scale[n_scales] = { 7.0, 14.0, 28.0, 56.0, 112.0, 224.0, 448.0, 896.0, 1792.0, 3584.0 };  //n_scales = 10

*/












	
	const int header_len=50;		//max length, in characters, of header
	const double pi=3.14159265358979324;
	const double twopi=6.28318530717958648;
	const float null=-999.0; //null value for errors etc.    Note: system value NULL (uppercase) is different!


	
// global typedefs
	typedef float *fptr;			//pointer to float-type
	typedef double *dptr;			//pointer to double-type


// global variables 
	char *header[n_columns];
	dptr t, x;			//pointers to arrays of data point times and values
	int n_data;			//number of data values (= length of t and x vectors)
	
	FILE *outfile;
	FILE *infile;
	FILE *locfile;
	FILE *pointfile;
	time_t start_time;	//start time

	float RMS[n_columns][n_scales];				//array to store results
	float RMS_se[n_columns][n_scales];
	float meandiff[n_columns][n_scales];				//array to store results
	float meandiff_se[n_columns][n_scales];
	float VG[n_columns][n_scales];				//array to store results
	float VG_se[n_columns][n_scales];
	float MAD[n_columns][n_scales];				//array to store results
	float MAD_lo[n_columns][n_scales];
	float MAD_hi[n_columns][n_scales];
	float var[n_columns][n_scales];				//array to store results
	float var_se[n_columns][n_scales];

	



#include <ctype.h>
int read_string(FILE *infile, char *string, int len) {	
	int i, x;	//reads an input file for a string.  reads until the first whitespace character
	i=0;
	do { 
		x=fgetc(infile);
		if ( (! isspace(x)) && (i<len-1)) {string[i]=x; i=i+1;}
	} while (! isspace(x));
	string[i]=0;	//null termination
return (0);
}//read_string


int read_string_from_string(char *source, char *string, int len) {	
	int i, x;	//reads an input string for a string.  Reads until the first whitespace character or end of string
	i=0;
	do { 
		x=source[i];
		if ( (! isspace(x)) && (i<len-1)) {string[i]=x; i=i+1;}
	} while (! isspace(x));
	string[i]=0;	//null termination
return (0);
}//read_string



	

int fcomp(x,y)
double *x,*y;
{ if(*x > *y) return 1;
  if(*x < *y) return -1;
  return(0);
}





/*
hsort

     Perform a heap sort with comparison function comp on the array v[].

     void hsort(void *v,int n,int (*comp)())
       v = array of pointers to sort input
            (pointers index sorted array at exit)
       n = dimension of input array
       comp = pointer to comparison function (see numcmp for example)
*/
/*  hsort.c    CCMATH mathematics library source code.
 *
 *  Copyright (C)  2000   Daniel A. Atkinson    All rights reserved.
 *  This code may be redistributed under the terms of the GNU library
 *  public license (LGPL). ( See the lgpl.license file for details.)
 * ------------------------------------------------------------------------
 */
#define Swap(x,y) (tmp=(x),(x)=(y),(y)=tmp)
static rheap();
hsort(v,n,comp)
	void *v[]; int n,(*comp)();
	{ int k; char *tmp;
	for(k=n/2-1; k>=0 ;) rheap(v,k--,n,comp);
	for(--n; n>0 ;){ Swap(v[0],v[n]); rheap(v,0,n--,comp);}
return (0);
}

static rheap(v,k,n,comp)
	void *v[]; int k,n,(*comp)();
	{ int m=n-1,j; char *tmp;
	for(j=2*k+1; j<n ;k=j,j=2*k+1){
	if(j<m && (*comp)(v[j+1],v[j])>0) ++j;
	if((*comp)(v[j],v[k])>0) Swap(v[k],v[j]);
	else break;
	}
return (0);
}










int read_in_data(int read_column)		//opens infile, allocates global arrays, and reads in data
{	int i, j;
	double tt, xx; //use these to transfer to possible float-sized t[i] and x[i]
	char junk[MAX_LINE_LENGTH+1]; 
	
	
	 //open input file and verify it exists
	if( (infile=fopen(infilename,"r")) == NULL ) {fprintf(stderr, "input file %s could not be opened!\n",infilename); exit(-1); }

	// run through file and count legal points
	n_data=0;
	fgets(junk, MAX_LINE_LENGTH, infile);	//discard first line
	do {	tt=null; xx=null;
		fscanf(infile, "%lg", &tt);		//get the time value
		for (j=1; j<read_column; j++)  fscanf(infile, "%lg", &xx);  //throw out columns before the one we need
		fgets(junk, MAX_LINE_LENGTH, infile);	//get the rest of the line
		sscanf(junk, "%lg ", &xx);	//read the one we need
		if ((tt != null) && (xx != null))  n_data=n_data+1;  //increment n_data if this is a valid data point
	} while (feof(infile)==0);

	// now allocate memory for t, x, means, and counts
	if ( ! (t = calloc(n_data+1 , sizeof(double)) )) {fprintf(stderr, "out of memory allocating t!\n"); exit(0);}	//note we have defined these as one element longer than we will actually use
	if ( ! (x = calloc(n_data+1 , sizeof(double)) )) {fprintf(stderr, "out of memory allocating x!\n"); exit(0);}	//so that we don't read past the end of the array when evaluating conditionals
	
	t[n_data]=9.99e99;  //give this point (which will never be used) an absurdly large value

	// reset file, get data values, and store them in t and x
	fclose(infile);
	infile=fopen(infilename,"r");
	read_string(infile, junk, header_len);	//discard first column heading
	for (j=1; j<read_column; j++)  read_string(infile, junk, header_len);	//throw out headers before the one we need
	fgets(junk, MAX_LINE_LENGTH, infile); //get the rest of the line
	read_string_from_string(junk, header[read_column-first_column], header_len); //read that string until the first whitespace character to find the header string we need

fprintf(stderr, "\nheader %i: %s\n", read_column-first_column+1, header[read_column-first_column]); 

	i=0;
	do {	tt=null; xx=null;
		fscanf(infile, "%lg", &tt);		//get the time value
		for (j=1; j<read_column; j++)  fscanf(infile, "%lg", &xx);  //throw out columns before the one we need
		fgets(junk, MAX_LINE_LENGTH, infile);	//get the rest of the line
		sscanf(junk, "%lg ", &xx) ;	//read the one we need
		if ((tt != null) && (xx != null)) {
			t[i]=tt; x[i]=xx; i=i+1;
		} //if
	} while (feof(infile)==0);

	//check for ordering of data
	for(i=1; i<n_data; i++) if (t[i]<t[i-1]) {  //if there's a problem, dump to error file and terminate
		outfile=fopen("error_file.txt", "w+");
		for (j=0; j<n_data; j++) fprintf(outfile, "%g\t%g\n", t[j], x[j]);
		fclose(outfile);
		fprintf(stderr, "times must be sorted in ascending order!\n");
		exit(0);
	}
	
	

	fclose(infile);
	return(0);
}  //end of read_in_data









int print_results( void ) {
	int j,k;
	double ln10, log10_se;
	char outfilename[100];  //output filename string
	char outfilespec[100];  //output filename string

	ln10=log(10.0);
	

	strcpy(outfilespec, filespec);
	

		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_RMS.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "scale");
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s", header[j]);
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s_se", header[j]);
			fprintf (outfile, "\n");
	
			for (k=0; k<n_scales; k++) {
				fprintf (outfile, "%g", scale[k]);
				for (j=0; j<n_columns; j++)  { if (RMS[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%g", RMS[j][k]); } ;
				for (j=0; j<n_columns; j++)  { if (RMS_se[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%g", RMS_se[j][k]); } ;
				fprintf (outfile, "\n");
			}//loop scales
	
			fclose(outfile);
		} //end else








		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_RMS_logstats.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "analyte\tscale\tRMS\tse\tlog_scale\tlog_RMS\tse_log\tinv_var_wt\n");
			for (j=0; j<n_columns; j++) {
				for (k=0; k<n_scales; k++) 
					if (RMS[j][k]!=null) {
						log10_se=(RMS_se[j][k]/RMS[j][k])/ln10; //standard error, in log10 units
						fprintf (outfile, "%s\t%g\t%g\t%g\t%g\t%g\t%g\t%g\n", header[j], scale[k], RMS[j][k], RMS_se[j][k], log10(scale[k]), log10(RMS[j][k]), log10_se, 1.0/(log10_se*log10_se) );
					}
				fprintf (outfile, "\n");
			} //next column
	
			fclose(outfile);
		} //end else





		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_logRMS.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "scale\tlog_scale");
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s_logRMS\t%s_se", header[j], header[j]);
			fprintf (outfile, "\n");
	
			for (k=0; k<n_scales; k++) {
				fprintf (outfile, "%g\t%g", scale[k], log10(scale[k]));
				for (j=0; j<n_columns; j++)  { if (RMS[j][k]==null) fprintf(outfile, "\t\t"); else {
						log10_se=(RMS_se[j][k]/RMS[j][k])/ln10; //standard error, in log10 units
						fprintf (outfile, "\t%g\t%g", log10(RMS[j][k]), log10_se); 
					} 
				}
				fprintf (outfile, "\n");
			}//loop scales
	
			fclose(outfile);
		} //end else




/*
		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_var.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "scale");
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s", header[j]);
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s_se", header[j]);
			fprintf (outfile, "\n");
	
			for (k=0; k<n_scales; k++) {
				fprintf (outfile, "%g", scale[k]);
				for (j=0; j<n_columns; j++)  { if (var[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%g", var[j][k]); } ;
				for (j=0; j<n_columns; j++)  { if (var_se[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%g", var_se[j][k]); } ;
				fprintf (outfile, "\n");
			}//loop scales
	
			fclose(outfile);
		} //end else





		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_var_logstats.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "analyte\tscale\tvar\tse\tlog_scale\tlog_var\tse_log\tinv_var_wt\n");
			for (j=0; j<n_columns; j++) {
				for (k=0; k<n_scales; k++) 
					if (var[j][k]!=null) {
						log10_se=(var_se[j][k]/var[j][k])/ln10; //standard error, in log10 units
						fprintf (outfile, "%s\t%g\t%g\t%g\t%g\t%g\t%g\t%g\n", header[j], scale[k], var[j][k], var_se[j][k], log10(scale[k]), log10(var[j][k]), log10_se, 1.0/(log10_se*log10_se) );
					}
				fprintf (outfile, "\n");
			} //next column
	
			fclose(outfile);
		} //end else

*/






/*
//un-comment this part if one calculates variograms and wants to print them
	strcpy(outfilename, outfilespec);
			strcat(outfilename, "_VG.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "scale");
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s", header[j]);
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s_se", header[j]);
			fprintf (outfile, "\n");
	
			for (k=0; k<n_scales; k++) {
				fprintf (outfile, "%g", scale[k]);
				for (j=0; j<n_columns; j++)  { if (VG[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%g", VG[j][k]); } ;
				for (j=0; j<n_columns; j++)  { if (VG_se[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%g", VG_se[j][k]); } ;
				fprintf (outfile, "\n");
			}//loop scales
	
			fclose(outfile);
		} //end else
*/



/*
//un-comment this part if one calculates MAD statistics and wants to print them
		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_MAD.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "scale");
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s", header[j]);
			fprintf (outfile, "\n");
	
			for (k=0; k<n_scales; k++) {
				fprintf (outfile, "%g", scale[k]);
				for (j=0; j<n_columns; j++)  { if (MAD[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%g", MAD[j][k]); } ;
				fprintf (outfile, "\n");
			}//loop scales
	
			fclose(outfile);
		} //end else


		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_MAD_with_CI.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "scale");
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s\t%s_lo\t%s_hi", header[j], header[j], header[j]);
			fprintf (outfile, "\n");
	
			for (k=0; k<n_scales; k++) {
				fprintf (outfile, "%g", scale[k]);
				for (j=0; j<n_columns; j++)  { 
					if (MAD[j][k]==null) fprintf(outfile, "\t\t\t"); else {
						fprintf (outfile, "\t%g", MAD[j][k]); 
						if (MAD_lo[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%g", MAD_lo[j][k]); 
						if (MAD_hi[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%g", MAD_hi[j][k]);
					} 
				}
				fprintf (outfile, "\n");
			}//loop scales
	
			fclose(outfile);
		} //end else




		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_MAD_w_error_bars.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "scale");
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s\t%s_dn\t%s_up", header[j], header[j], header[j]);
			fprintf (outfile, "\n");
	
			for (k=0; k<n_scales; k++) {
				fprintf (outfile, "%g", scale[k]);
				for (j=0; j<n_columns; j++)  { 
					if (MAD[j][k]==null) fprintf(outfile, "\t\t\t"); else {
						fprintf (outfile, "\t%g", MAD[j][k]); 
						if (MAD_lo[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%g", MAD[j][k]-MAD_lo[j][k]); 
						if (MAD_hi[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%g", MAD_hi[j][k]-MAD[j][k]); 
					}
				}
				fprintf (outfile, "\n");
			}//loop scales
	
			fclose(outfile);
		} //end else


*/
	

return (0);
}//print results
	























int calculate_RMS( int column, int row, double scale ) {
	double sumx, sumxx, sumwt, sumwt2, t_ref, wt, pooled_var, diff;
	int bin_tally, max_bins;
	int i, j; //these hold the upper and lower indices of the two bins
	double normal_count, n_eff;
	double *avg;
	double *count;
	double var1, var2, var3, var4, df1, df2, df3, df4, mean1, mean2, mean3, mean4;


	normal_count = round(scale/sampling_interval); //normal number of points we should expect to be averaged together

	//if averaging scale is more than half the length of the record, there's no need to go further
	if (n_data<2.0*normal_count) { RMS[column][row]=null; RMS_se[column][row]=null; return(0); }

	max_bins=round((t[n_data-1]-t[0])/scale)+2 ;
	
	if ( ! (avg = calloc(max_bins , sizeof(double)) )) {fprintf(stderr, "out of memory allocating avg!\n"); exit(0);}
	if ( ! (count = calloc(max_bins , sizeof(double)) )) {fprintf(stderr, "out of memory allocating count!\n"); exit(0);}


	for (j=0; j<max_bins; j++) {avg[j]=0.0; count[j]=0.0;}  //set all bins to zero

	//now sum values from time series into bins of size <scale>, starting from t[0]
	t_ref=t[0];
	for (i=0; i<n_data; i++) {
		j=trunc((t[i]-t_ref)/scale);
		avg[j]=avg[j]+x[i]; //avg contains sums, for now
		count[j]=count[j]+1.0;
	}

	sumx=0.0;
	sumxx=0.0;
	sumwt=0.0;
	sumwt2=0.0;

	for (j=0; j<max_bins; j++) { if (count[j]>=(normal_count/2.0)) avg[j]=avg[j]/count[j]; else avg[j]=null; }

	for (j=1; j<max_bins; j++) {
		if ((avg[j]==null)||(avg[j-1]==null)) wt=0.0; else {
			wt = 1.0;
			diff=avg[j]-avg[j-1];			
			sumx=sumx+diff*diff*wt;  //weighted sum of squared differences
			sumxx=sumxx+diff*diff*diff*diff*wt;  //weighted sum of squares (of squared differences)
			sumwt=sumwt+wt; //sum of weights
		}
	}



	//reset bin boundaries to overlap by 1/2 bin, then sum values into bins again
	t_ref=t[0]-0.5*scale;
	for (j=0; j<max_bins; j++) {avg[j]=0.0; count[j]=0.0;}  //set all bins to zero
	for (i=0; i<n_data; i++) {
		j=trunc((t[i]-t_ref)/scale);
		avg[j]=avg[j]+x[i]; //avg contains sums, for now
		count[j]=count[j]+1.0;
	}


	for (j=0; j<max_bins; j++) { if (count[j]>=(normal_count/2.0)) avg[j]=avg[j]/count[j]; else avg[j]=null; }
	
	for (j=1; j<max_bins; j++) {
		if ((avg[j]==null)||(avg[j-1]==null)) wt=0.0; else {
			wt = 1.0;
			diff=avg[j]-avg[j-1];			
			sumx=sumx+diff*diff*wt;  //weighted sum of squared differences
			sumxx=sumxx+diff*diff*diff*diff*wt;  //weighted sum of squares (of squared differences)
			sumwt=sumwt+wt; //sum of weights
		}
	}



	//reset bin boundaries to overlap by 3/4 bin, then sum values into bins again
	t_ref=t[0]-0.25*scale;
	for (j=0; j<max_bins; j++) {avg[j]=0.0; count[j]=0.0;}  //set all bins to zero
	for (i=0; i<n_data; i++) {
		j=trunc((t[i]-t_ref)/scale);
		avg[j]=avg[j]+x[i]; //avg contains sums, for now
		count[j]=count[j]+1.0;
	}



	for (j=0; j<max_bins; j++) { if (count[j]>=(normal_count/2.0)) avg[j]=avg[j]/count[j]; else avg[j]=null; }

	for (j=1; j<max_bins; j++) {
		if ((avg[j]==null)||(avg[j-1]==null)) wt=0.0; else {
			wt = 1.0;
			diff=avg[j]-avg[j-1];			
			sumx=sumx+diff*diff*wt;  //weighted sum of squared differences
			sumxx=sumxx+diff*diff*diff*diff*wt;  //weighted sum of squares (of squared differences)
			sumwt=sumwt+wt; //sum of weights
		}
	}



	//reset bin boundaries to overlap by 1/4 bin, then sum values into bins again
	t_ref=t[0]-0.75*scale;
	for (j=0; j<max_bins; j++) {avg[j]=0.0; count[j]=0.0;}  //set all bins to zero
	for (i=0; i<n_data; i++) {
		j=trunc((t[i]-t_ref)/scale);
		avg[j]=avg[j]+x[i]; //avg contains sums, for now
		count[j]=count[j]+1.0;
	}


	for (j=0; j<max_bins; j++) { if (count[j]>=(normal_count/2.0)) avg[j]=avg[j]/count[j]; else avg[j]=null; }

	for (j=1; j<max_bins; j++) {
		if ((avg[j]==null)||(avg[j-1]==null)) wt=0.0; else {
			wt = 1.0;
			diff=avg[j]-avg[j-1];			
			sumx=sumx+diff*diff*wt;  //weighted sum of squared differences
			sumxx=sumxx+diff*diff*diff*diff*wt;  //weighted sum of squares (of squared differences)
			sumwt=sumwt+wt; //sum of weights
		}
	}


	if (sumwt==0.0) {RMS[column][row]=null; RMS_se[column][row]=null; }
	else {
		pooled_var=(sumxx - (sumx*sumx/sumwt))/(sumwt-1.0);
		RMS[column][row] = sqrt(sumx/sumwt); //root-mean-square of differences between averages in adjacent bins
		if (sumwt>1.0) RMS_se[column][row] = (0.5/RMS[column][row])*sqrt(pooled_var/sumwt);  //derived from Gaussian error propagation
			else RMS_se[column][row] = null;

		sumx=0.0; sumwt=0.0;  //now we just calculate the average of the whole time series
		for (i=0; i<n_data; i++) {sumx=sumx+x[i]; sumwt=sumwt+1.0;}

	}


	free(avg);
	free(count);

	return(0);

} //calculate_RMS



	











int calculate_var( int column, int row, double scale ) //this is the variance between adjacent pairs of means (this gives 2 times the Allan variance)
{
	double sumx, sumxx, sumwt, sumwt2, t_ref;
	int bin_tally, max_bins;
	int i, j; //these hold the upper and lower indices of the two bins
	double normal_count, n_eff;
	double *avg;
	double *count;
	double var1, var2, var3, var4, df1, df2, df3, df4;


	normal_count = round(scale/sampling_interval); //normal number of points we should expect to be averaged together

	//if averaging scale is more than half the length of the record, there's no need to go further
	if (n_data<2.0*normal_count) { var[column][row]=null; var_se[column][row]=null; return(0); }

	max_bins=round((t[n_data-1]-t[0])/scale)+2 ;
	
	if ( ! (avg = calloc(max_bins , sizeof(double)) )) {fprintf(stderr, "out of memory allocating avg!\n"); exit(0);}
	if ( ! (count = calloc(max_bins , sizeof(double)) )) {fprintf(stderr, "out of memory allocating count!\n"); exit(0);}


	for (j=0; j<max_bins; j++) {avg[j]=0.0; count[j]=0.0;}  //set all bins to zero

	//now sum values from time series into bins of size <scale>, starting from t[0]
	t_ref=t[0];
	for (i=0; i<n_data; i++) {
		j=trunc((t[i]-t_ref)/scale);
		avg[j]=avg[j]+x[i]; //avg contains sums, for now
		count[j]=count[j]+1.0;
	}

	sumx=0.0;
	sumxx=0.0;
	sumwt=0.0;
	sumwt2=0.0;

	for (j=0; j<max_bins; j++) {
		if (count[j]>=(normal_count/2.0)) avg[j]=avg[j]/count[j]; else avg[j]=null;
		if (avg[j]!=null) {
			sumx=sumx+avg[j]*count[j];  //weighted sum
			sumxx=sumxx+avg[j]*avg[j]*count[j];  //weighted sum of squares
			sumwt=sumwt+count[j]; //sum of weights
			sumwt2=sumwt2+count[j]*count[j];	
		}
	}	

	var1 = (sumxx/sumwt) - (sumx/sumwt)*(sumx/sumwt);
	var1 = var1 * (sumwt*sumwt)/(sumwt*sumwt - sumwt2) ; //this is the correction for effective numbers of degrees of freedom (based on evenness of weights... see wikipedia "weighted sample variance")
	df1 = sumwt*sumwt/sumwt2 - 1.0;



	//reset bin boundaries to overlap by 1/2 bin, then sum values into bins again
	t_ref=t[0]-0.5*scale;
	for (j=0; j<max_bins; j++) {avg[j]=0.0; count[j]=0.0;}  //set all bins to zero
	for (i=0; i<n_data; i++) {
		j=trunc((t[i]-t_ref)/scale);
		avg[j]=avg[j]+x[i]; //avg contains sums, for now
		count[j]=count[j]+1.0;
	}

	sumx=0.0;
	sumxx=0.0;
	sumwt=0.0;
	sumwt2=0.0;

	for (j=0; j<max_bins; j++) {
		if (count[j]>=(normal_count/2.0)) avg[j]=avg[j]/count[j]; else avg[j]=null;
		if (avg[j]!=null) {
			sumx=sumx+avg[j]*count[j];  //weighted sum
			sumxx=sumxx+avg[j]*avg[j]*count[j];  //weighted sum of squares
			sumwt=sumwt+count[j]; //sum of weights
			sumwt2=sumwt2+count[j]*count[j];	
		}
	}	

	var2 = (sumxx/sumwt) - (sumx/sumwt)*(sumx/sumwt);
	var2 = var2 * (sumwt*sumwt)/(sumwt*sumwt - sumwt2) ; //this is the correction for effective numbers of degrees of freedom (based on evenness of weights... see wikipedia "weighted sample variance")
	df2 = sumwt*sumwt/sumwt2 - 1.0;


	//reset bin boundaries to overlap by 3/4 bin, then sum values into bins again
	t_ref=t[0]-0.25*scale;
	for (j=0; j<max_bins; j++) {avg[j]=0.0; count[j]=0.0;}  //set all bins to zero
	for (i=0; i<n_data; i++) {
		j=trunc((t[i]-t_ref)/scale);
		avg[j]=avg[j]+x[i]; //avg contains sums, for now
		count[j]=count[j]+1.0;
	}

	sumx=0.0;
	sumxx=0.0;
	sumwt=0.0;
	sumwt2=0.0;

	for (j=0; j<max_bins; j++) {
		if (count[j]>=(normal_count/2.0)) avg[j]=avg[j]/count[j]; else avg[j]=null;
		if (avg[j]!=null) {
			sumx=sumx+avg[j]*count[j];  //weighted sum
			sumxx=sumxx+avg[j]*avg[j]*count[j];  //weighted sum of squares
			sumwt=sumwt+count[j]; //sum of weights
			sumwt2=sumwt2+count[j]*count[j];	
		}
	}	

	var3 = (sumxx/sumwt) - (sumx/sumwt)*(sumx/sumwt);
	var3 = var3 * (sumwt*sumwt)/(sumwt*sumwt - sumwt2) ; //this is the correction for effective numbers of degrees of freedom (based on evenness of weights... see wikipedia "weighted sample variance")
	df3 = sumwt*sumwt/sumwt2 - 1.0;


	//reset bin boundaries to overlap by 1/4 bin, then sum values into bins again
	t_ref=t[0]-0.75*scale;
	for (j=0; j<max_bins; j++) {avg[j]=0.0; count[j]=0.0;}  //set all bins to zero
	for (i=0; i<n_data; i++) {
		j=trunc((t[i]-t_ref)/scale);
		avg[j]=avg[j]+x[i]; //avg contains sums, for now
		count[j]=count[j]+1.0;
	}

	sumx=0.0;
	sumxx=0.0;
	sumwt=0.0;
	sumwt2=0.0;

	for (j=0; j<max_bins; j++) {
		if (count[j]>=(normal_count/2.0)) avg[j]=avg[j]/count[j]; else avg[j]=null;
		if (avg[j]!=null) {
			sumx=sumx+avg[j]*count[j];  //weighted sum
			sumxx=sumxx+avg[j]*avg[j]*count[j];  //weighted sum of squares
			sumwt=sumwt+count[j]; //sum of weights
			sumwt2=sumwt2+count[j]*count[j];	
		}	
	}

	var4 = (sumxx/sumwt) - (sumx/sumwt)*(sumx/sumwt);
	var4 = var4 * (sumwt*sumwt)/(sumwt*sumwt - sumwt2) ; //this is the correction for effective numbers of degrees of freedom (based on evenness of weights... see wikipedia "weighted sample variance")
	df4 = sumwt*sumwt/sumwt2 - 1.0;



	var[column][row]= (df1*var1+df2*var2+df3*var3+df4*var4)/(df1+df2+df3+df4);  
	var_se[column][row]=var[column][row]*sqrt(2.0/(0.25*df1+0.25*df2+0.25*df3+0.25*df4));
	

	free(avg);
	free(count);

	return(0);

} //calculate_var



	





int calculate_VG( int column, int row, double scale ) { //calculates variogram by finding mean-square difference between all pairs of points separated by <scale> plus or minus 1/2 of sampling interval
	double diff, sumdiff2, sumdiff4, var, lower, upper;
	int difftally;
	int j, k, start_j; 

	//if averaging scale is more than 3/4 of the length of the record, don't go further
	if (scale>((t[n_data-1]-t[0])*0.75)) { VG[column][row]=null; VG_se[column][row]=null; return(0); }


	

	difftally=0;
	sumdiff2=0.0;
	sumdiff4=0.0;
	k=-1; //k is the index of the lower point (we will advance this in a moment)
	j=0; //j is the index of the upper point
	start_j=0;
	while (k<n_data-1) {
		k++;
		lower=t[k]+scale-sampling_interval/2.0; //lower bound of the range we are interested in
		upper=lower+sampling_interval; //upper bound of the target range
		j=start_j;
		while ((t[j]<lower) && (j<n_data-1)) j++; //do this to advance into the target range
		start_j=j;	//save this point -- we'll use it to start advancing j the next time around (in case t[k] only changes a little bit, we may need to re-use some j's)
		while ((t[j]<=upper) && (j<n_data)) { //if there are any points in the target range, use them
			diff = x[j]-x[k];
			sumdiff2 = sumdiff2 + diff*diff;  //sum of squared differences
			sumdiff4 = sumdiff4 + diff*diff*diff*diff;
			difftally = difftally + 1;
			j++;
		} //loop through target range
	} //next k

	//now calculate mean-square of differences between values separated by <scale>
	if (difftally<n_data/3.0) { VG[column][row]=null; VG_se[column][row]=null; return(0); }  //if we don't have enough point pairs, let's not estimate VG
	else {
		VG[column][row]=(sumdiff2/(double)difftally);  //VG is mean-square average of differences between successive averages
		var = (sumdiff4-(sumdiff2*sumdiff2/(double)difftally))/((double)difftally-1.0); //variance of squared differences between successive averages
		VG_se[column][row]=sqrt(var/(double)difftally);  // this is from ordinary Gaussian error propagation
	}


	return(0);

} //calculate_VG









int calculate_MAD( int column, int row, double scale ) {
	double *diff, **v, t_ref;
	int difftally;
	int i, j, ii, jj, k; //these hold the upper and lower indices of the two bins
	int count1, count2; //these are the counts in the two bins
	double sum1, sum2; //these are the sums in the two bins
	double normal_count, n_eff;

	int (*comp_function) (fptr, fptr);	//pointer to sort comparison function
	comp_function=fcomp;			//set pointer to comparison function


	//if averaging scale is more than half the length of the record, there's no need to go further
	if (scale>((t[n_data-1]-t[0])/2.0)) { MAD[column][row]=null; MAD_lo[column][row]=null; MAD_hi[column][row]=null; return(0); }

	//allocate array to hold differences
	if ( ! (diff = calloc(n_data , sizeof(double)) )) {fprintf(stderr, "out of memory allocating diff!\n"); exit(0);}	

	normal_count = round(scale/sampling_interval);
	

	i=0; j=0; sum1=x[0]; count1=1; //start the left-hand bin with one point;
	ii=0; jj=0; sum2=x[0]; count2=1; //start the right-hand bin with one point. for the moment we will define this as the *same* as the left-hand bin
	k=0; //k is the index of the upper 

	difftally=0;
	while (k<n_data-2) {
		k++;
		while (((t[k]-t[k-1])>2.0*scale) && (k<n_data-2)) k++; //repeat this if the gap between the points is so wide that no adjacent pair of bins could have any elements
		t_ref=0.5*(t[k]+t[k-1]);  //t_ref is the reference point that divides the two bins.  It should always be mid-way between two points.
		
		while (t[ii]<t_ref) {sum2=sum2-x[ii]; count2=count2-1; ii++;} //move left edge of upper bin to just after t_ref
		while ((jj<n_data-2) && (t[jj+1]<t_ref+scale)) {jj++; sum2=sum2+x[jj]; count2=count2+1;} //move right edge of upper bin
		while (t[j+1]<t_ref) {j++; sum1=sum1+x[j]; count1=count1+1;} //move right edge of lower bin to just before t_ref
		while (t[i]<t_ref-scale) {sum1=sum1-x[i]; count1=count1-1; i++;} //move left edge of lower bin

		if ((count1>=0.5*normal_count) && (count2>=0.5*normal_count)) { //calculate the absolute-value difference between the means of the two bins
			diff[difftally] = fabs((sum2/(double)count2) - (sum1/(double)count1)); //we haven't incremented difftally yet, so this is the right place to store it
			difftally = difftally + 1;
		} //if

	} //step through possible dividing points (t_ref) between the two bins


	//determine median absolute difference and its confidence limits
	if (difftally<normal_count) { MAD[column][row]=null; MAD_lo[column][row]=null; MAD_hi[column][row]=null; free(diff); return(0); }  //if there are no non-overlapping difference measures, quit!
	v=(double **)calloc(difftally,sizeof(double *));  //array of pointers to each non-blank element of the vector of differences
	for(i=0; i<difftally; i++) v[i]=&diff[i];  //pointers for sorting
	hsort( (void *) v, difftally, comp_function);
	MAD[column][row]=0.5*(*v[difftally/2] + *v[(difftally-1)/2]); //median
	n_eff = (double)difftally/normal_count;  //this is an estimate of the "effective" number of differences (correcting for redundancy of the overlapping sets of bins
	j = trunc(0.5*difftally-1.96*difftally*sqrt(0.25/n_eff)); //rank of lower bound of 95% confidence interval of median
	if (j<0) MAD_lo[column][row]=null; else MAD_lo[column][row]=*v[j];
	j = trunc(0.5*difftally+1.96*difftally*sqrt(0.25/n_eff))+1; //rank of upper bound of 95% confidence interval of median
	if (j>difftally-1) MAD_hi[column][row]=null; else MAD_hi[column][row]=*v[j];

	//The number of observations below the qth quantile will be an observation from a binomial distribution with parameters n and q
	//where (in the large sample approximation) the mean is nq and the standard deviation is sqrt(nq*(1-q)).
	//Thus for a 95% CI we would have ranks of confidence bounds of nq-1.96*sqrt(nq*(1-q)) and nq+1.96*sqrt(nq*(1-q)).
	//But here we have n_eff<difftally because of redundancy in the estimates of diff.  In this case, the standard deviation
	//should grow as n*sqrt(q*(1-q)/n_eff) rather than n*sqrt(q*(1-q)/n), which is the binomial theorem formula.  This is what is implemented here.

	free(v);
	free(diff);

	return(0);

} //calculate_MAD










int main( void ) {
	int i;		//index for iterations
	int j;		//index for columns
	int k;
	char outfilename[100];  //output filename string
	char outfilespec[100];  //output filename string
	

	start_time=time(NULL);
	
	for (j=0; j<n_columns; j++) header[j]=malloc(sizeof(char[header_len]));

	//define array of scales
//	if (linear_scales==true) for (k=0; k<n_scales; k++) scale[k]=sampling_interval*(double)(k+1);
//	else {
//		scale[0]=sampling_interval;
//		for (k=1; k<n_scales; k++) scale[k]=2.0*scale[k-1];  //each averaging scale is 2x the previous one
//	}
		
//here's where the analysis begins
	
	for (j=0; j<n_columns; j++) {	//step through read_columns
		read_in_data(j+first_column);
fprintf(stderr, "\n");
fprintf(stderr, "%i data read in... elapsed time %g seconds\n", j+first_column, difftime( time(NULL) , start_time ) );
		for (k=0; k<n_scales; k++) calculate_RMS(j, k, scale[k]);
//		for (k=0; k<n_scales; k++) calculate_RMS_reldiff(j, k, scale[k]);
fprintf(stderr, "%i RMS done... elapsed time %g seconds\n", j+first_column, difftime( time(NULL) , start_time ) );
//		for (k=0; k<n_scales; k++) calculate_VG(j, k, scale[k]);
//fprintf(stderr, "%i VG done... elapsed time %g seconds\n", j+first_column, difftime( time(NULL) , start_time ) );
//		for (k=0; k<n_scales; k++) calculate_MAD(j, k, scale[k]);
//fprintf(stderr, "%i MAD done... elapsed time %g seconds\n", j+first_column, difftime( time(NULL) , start_time ) );
//		for (k=0; k<n_scales; k++) calculate_var(j, k, scale[k]);
//fprintf(stderr, "%i var done... elapsed time %g seconds\n", j+first_column, difftime( time(NULL) , start_time ) );

		free(x);
		free(t);		
	}	//finish working through read_columns
	


	fprintf(stderr, "analysis done... elapsed time %g seconds\n", difftime( time(NULL) , start_time ) );

	print_results( );
	
	
//	fclose(locfile);
//	fclose(pointfile);

	
	return 0;
}

