

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
					
					
#define float double				//this enforces double precision for all calculations (and avoids type conversions)
#define MAX_LINE_LENGTH 5000	//length, in chars, of max line in input file
#define true 1
#define false 0



/*
*******************************************************************************************************
*******************************************************************************************************
READ ME FIRST:

This source code was written by J.W. Kirchner, who holds copyright to it.  It is being made available
to the public under the Creative Commons Attribution-NonCommercial-ShareAlike License (CC BY-NC-SA).
That means you are free:
to Share  to copy, distribute and transmit the work
to Remix  to adapt the work
Under the following conditions:

Attribution  You must attribute the work in the manner specified by the author or licensor (but not in any way that suggests that they endorse you or your use of the work).
Noncommercial  You may not use this work for commercial purposes.
Share Alike  If you alter, transform, or build upon this work, you may distribute the resulting work only under the same or similar license to this one.
Any of the above conditions can be waived if you get permission from the copyright holder.

For details see:
http://creativecommons.org/licenses/by-nc-sa/3.0/
http://creativecommons.org/licenses/by-nc-sa/3.0/legalcode
*******************************************************************************************************
*******************************************************************************************************
*/










// global constants



//this routine reads time series and calculates their averages over longer and longer intervals of time,
//then calculates the p-values of the differences between successive averages





//	this is for t-statistics as a function of averaging scales for 7-hourly time series:

	const char infilename[100]="7hour UHF for scales.txt";
	const char filespec[100]="7hr_UHF";

//	const char infilename[100]="7hour LHF for scales.txt";
//	const char filespec[100]="7hr_LHF";

//	const char infilename[100]="7hour CR for scales.txt";
//	const char filespec[100]="7hr_CR";

	const int fourway=true;

	const double sampling_interval=0.2916666666667;  //should be 7.0 (days) for weekly sampling or 0.2916666666667 (=7/24 days) for 7-hour sampling

#define n_columns  45  // 45 for UHF, UHFw, LHF, LHFw, or CRw, 41 for CR		//number of data columns to read
const int first_column=48;  //48 for UHF, UHFw, LHF, LHFw, or CRw, 44 for CR  //first data column (first data column comes after day number)

	#define n_scales 6		// maximum number of different scales (note that the highest scale that is useful is log2(n_data) for log scales, or n_data/2 for linear scales)
//const int linear_scales=false;	//if true, averaging scales are 1, 2, 3, 4, 5, 6... times the sampling interval (and we need about 1200 scales for 7-hourly data)
							//if false, averaging scales are 1, 2, 4, 8... times the sampling interval (and we need about 12 scales for 7-hourly data)

	const float scale[n_scales] = { 7.0, 14.0, 30.0, 90.0, 180.0, 365.0 }; //n_scales = 6







/*
//	this is for t-statistics as a function of averaging scales for weekly time series:

	const char infilename[100]="weekly UHFw for scales.txt";
	const char filespec[100]="wkly_UHFw";

//	const char infilename[100]="weekly LHFw for scales.txt";
//	const char filespec[100]="wkly_LHFw";

//	const char infilename[100]="weekly CRw for scales.txt";
//	const char filespec[100]="wkly_CRw";

	const int fourway=true;

	const double sampling_interval=7.0;  //should be 7.0 (days) for weekly sampling or 0.2916666666667 (=7/24 days) for 7-hour sampling

	#define n_columns  45  // 45 for UHF, UHFw, LHF, LHFw, or CRw, 41 for CR		//number of data columns to read
	const int first_column=48;		//48 for UHF, UHFw, LHF, LHFw, or CRw, 44 for CR  //first data column (first data column comes after day number

	#define n_scales 6		// maximum number of different scales (note that the highest scale that is useful is log2(n_data) for log scales, or n_data/2 for linear scales)

	const float scale[n_scales] = { 90.0, 180.0, 365.0, 720.0, 1825.0, 3650.0 }; //n_scales = 6
*/















	
	const int header_len=50;		//max length, in characters, of header
	const double pi=3.14159265358979324;
	const double twopi=6.28318530717958648;
	const float null=-999.0; //null value for errors etc.    Note: system value NULL (uppercase) is different!


	
// global typedefs
	typedef float *fptr;			//pointer to float-type
	typedef double *dptr;			//pointer to double-type


// global variables 
	char *header[n_columns];
	dptr t, x;			//pointers to arrays of data point times and values
	int n_data;			//number of data values (= length of t and x vectors)
	
	FILE *outfile;
	FILE *infile;
	FILE *locfile;
	FILE *pointfile;
	time_t start_time;	//start time

	



#include <ctype.h>
int read_string(FILE *infile, char *string, int len) {	
	int i, x;	//reads an input file for a string.  reads until the first whitespace character
	i=0;
	do { 
		x=fgetc(infile);
		if ( (! isspace(x)) && (i<len-1)) {string[i]=x; i=i+1;}
	} while (! isspace(x));
	string[i]=0;	//null termination
return (0);
}//read_string


int read_string_from_string(char *source, char *string, int len) {	
	int i, x;	//reads an input string for a string.  Reads until the first whitespace character or end of string
	i=0;
	do { 
		x=source[i];
		if ( (! isspace(x)) && (i<len-1)) {string[i]=x; i=i+1;}
	} while (! isspace(x));
	string[i]=0;	//null termination
return (0);
}//read_string



	
	
int fcomp(x,y)
double *x,*y;
{ if(*x > *y) return 1;
  if(*x < *y) return -1;
  return(0);
}





/*
hsort

     Perform a heap sort with comparison function comp on the array v[].

     void hsort(void *v,int n,int (*comp)())
       v = array of pointers to sort input
            (pointers index sorted array at exit)
       n = dimension of input array
       comp = pointer to comparison function (see numcmp for example)
*/
/*  hsort.c    CCMATH mathematics library source code.
 *
 *  Copyright (C)  2000   Daniel A. Atkinson    All rights reserved.
 *  This code may be redistributed under the terms of the GNU library
 *  public license (LGPL). ( See the lgpl.license file for details.)
 * ------------------------------------------------------------------------
 */
#define Swap(x,y) (tmp=(x),(x)=(y),(y)=tmp)
static rheap();
hsort(v,n,comp)
	void *v[]; int n,(*comp)();
	{ int k; char *tmp;
	for(k=n/2-1; k>=0 ;) rheap(v,k--,n,comp);
	for(--n; n>0 ;){ Swap(v[0],v[n]); rheap(v,0,n--,comp);}
return (0);
}

static rheap(v,k,n,comp)
	void *v[]; int k,n,(*comp)();
	{ int m=n-1,j; char *tmp;
	for(j=2*k+1; j<n ;k=j,j=2*k+1){
	if(j<m && (*comp)(v[j+1],v[j])>0) ++j;
	if((*comp)(v[j],v[k])>0) Swap(v[k],v[j]);
	else break;
	}
return (0);
}







double tdist(double t, int nu) //returns two-tailed alpha value for t distribution with nu degrees of freedom, 
                               //using trigonometric series from Abromowitz and Stegun 1970, equations 26.7.3 and 26.7.4
							   //note that A&S formulas give p-values (1-alpha, the area under the "bell" curve), so here we return alpha=1-p, the area in the tails
{	int i;
	double theta, costheta, sintheta, cos2, prod, sum, cospwr, p_value;
	
	if (nu<=0) {return(1.0); exit(0);}
	theta = atan(t/sqrt(nu));

	if (nu==1) {return(1.0 - (2.0*theta/pi)); exit(0);}

	costheta = cos(theta);
	cos2 = costheta*costheta;
	sintheta = sin(theta);

	if (nu==2) {return(1.0 - sintheta); exit(0);}
	if (nu==3) {return(1.0 - (2.0/pi)*(theta+sintheta*costheta)); exit(0);}

	if (nu % 2 ==0) {  //if nu is even...
		sum=1.0; prod=1.0; cospwr=1.0;
		for(i=4; i<=nu; i=i+2) {
			prod=prod*(i-3.0)/(i-2.0); //note the 2.0 (rather than 2) is necessary here to force real division instead of integer division!
			cospwr=cospwr*cos2;
			sum=sum+prod*cospwr;
		}
		p_value = sintheta*sum;

	} else { //otherwise, if nu is odd...
		sum=costheta; prod=1.0; cospwr=costheta;
		for(i=5; i<=nu; i=i+2) {
			prod=prod*(i-3.0)/(i-2.0); //note the 2.0 (rather than 2) is necessary here to force real division instead of integer division
			cospwr=cospwr*cos2;
			sum=sum+prod*cospwr;
		}
		p_value = (2.0/pi) * (theta + sintheta*sum);
	} //nu odd

	return(1.0 - p_value);
}











int read_in_data(int read_column)		//opens infile, allocates global arrays, and reads in data
{	int i, j;
	double tt, xx; //use these to transfer to possible float-sized t[i] and x[i]
	char junk[MAX_LINE_LENGTH+1]; 
	
	
	 //open input file and verify it exists
	if( (infile=fopen(infilename,"r")) == NULL ) {fprintf(stderr, "input file %s could not be opened!\n",infilename); exit(-1); }

	// run through file and count legal points
	n_data=0;
	fgets(junk, MAX_LINE_LENGTH, infile);	//discard first line
	do {	tt=null; xx=null;
		fscanf(infile, "%lg", &tt);		//get the time value
		for (j=1; j<read_column; j++)  fscanf(infile, "%lg", &xx);  //throw out columns before the one we need
		fgets(junk, MAX_LINE_LENGTH, infile);	//get the rest of the line
		sscanf(junk, "%lg ", &xx);	//read the one we need
		if ((tt != null) && (xx != null))  n_data=n_data+1;  //increment n_data if this is a valid data point
	} while (feof(infile)==0);

	// now allocate memory for t, x, means, and counts
	if ( ! (t = calloc(n_data+1 , sizeof(double)) )) {fprintf(stderr, "out of memory allocating t!\n"); exit(0);}	//note we have defined these as one element longer than we will actually use
	if ( ! (x = calloc(n_data+1 , sizeof(double)) )) {fprintf(stderr, "out of memory allocating x!\n"); exit(0);}	//so that we don't read past the end of the array when evaluating conditionals
	
	t[n_data]=9.99e99;  //give this point (which will never be used) an absurdly large value

	// reset file, get data values, and store them in t and x
	fclose(infile);
	infile=fopen(infilename,"r");
	read_string(infile, junk, header_len);	//discard first column heading
	for (j=1; j<read_column; j++)  read_string(infile, junk, header_len);	//throw out headers before the one we need
	fgets(junk, MAX_LINE_LENGTH, infile); //get the rest of the line
	read_string_from_string(junk, header[read_column-first_column], header_len); //read that string until the first whitespace character to find the header string we need

fprintf(stderr, "\nheader %i: %s\n", read_column-first_column+1, header[read_column-first_column]); 

	i=0;
	do {	tt=null; xx=null;
		fscanf(infile, "%lg", &tt);		//get the time value
		for (j=1; j<read_column; j++)  fscanf(infile, "%lg", &xx);  //throw out columns before the one we need
		fgets(junk, MAX_LINE_LENGTH, infile);	//get the rest of the line
		sscanf(junk, "%lg ", &xx) ;	//read the one we need
		if ((tt != null) && (xx != null)) {
			t[i]=tt; x[i]=xx; i=i+1;
		} //if
	} while (feof(infile)==0);

	//check for ordering of data
	for(i=1; i<n_data; i++) if (t[i]<t[i-1]) {  //if there's a problem, dump to error file and terminate
		outfile=fopen("error_file.txt", "w+");
		for (j=0; j<n_data; j++) fprintf(outfile, "%g\t%g\n", t[j], x[j]);
		fclose(outfile);
		fprintf(stderr, "times must be sorted in ascending order!\n");
		exit(0);
	}
	
	

	fclose(infile);
	return(0);
}  //end of read_in_data













int calculate_t_stats( int column, int row, double scale, double t_ref ) {  //here we calculate the t-statistic of differences between successive means
	double r_squared, trend_t, trend_change_t, diff_p, trend_p, trend_change_p, pooled_var, stat;
	int bin_tally, max_bins;
	int i, j; //these hold the upper and lower indices of the two bins
	double normal_count, n_eff, diff;
	double *avg, *sumx, *sumxx, *sumxy, *slope, *slope_se, *sumy, *sumyy, *var, *min_t, *max_t;
	int *count;
	int df, trend_df, trend_change_df;
	double t_value, jittered_scale;


	normal_count = round(scale/sampling_interval); //normal number of points we should expect to be averaged together

	//if averaging scale is more than half the length of the record, there's no need to go further

	max_bins=round((t[n_data-1]-t[0])/scale)+2 ;
	
	if ( ! (avg = calloc(max_bins , sizeof(double)) )) {fprintf(stderr, "out of memory allocating avg!\n"); exit(0);}
	if ( ! (var = calloc(max_bins , sizeof(double)) )) {fprintf(stderr, "out of memory allocating se!\n"); exit(0);}
	if ( ! (count = calloc(max_bins , sizeof(int)) )) {fprintf(stderr, "out of memory allocating count!\n"); exit(0);}
	if ( ! (sumx = calloc(max_bins , sizeof(double)) )) {fprintf(stderr, "out of memory allocating sumx!\n"); exit(0);}
	if ( ! (sumxx = calloc(max_bins , sizeof(double)) )) {fprintf(stderr, "out of memory allocating sumxx!\n"); exit(0);}
	if ( ! (sumxy = calloc(max_bins , sizeof(double)) )) {fprintf(stderr, "out of memory allocating sumxy!\n"); exit(0);}
	if ( ! (sumy = calloc(max_bins , sizeof(double)) )) {fprintf(stderr, "out of memory allocating sumy!\n"); exit(0);}
	if ( ! (sumyy = calloc(max_bins , sizeof(double)) )) {fprintf(stderr, "out of memory allocating sumyy!\n"); exit(0);}
	if ( ! (slope = calloc(max_bins , sizeof(double)) )) {fprintf(stderr, "out of memory allocating sumyy!\n"); exit(0);}
	if ( ! (slope_se = calloc(max_bins , sizeof(double)) )) {fprintf(stderr, "out of memory allocating sumyy!\n"); exit(0);}
	if ( ! (min_t = calloc(max_bins , sizeof(double)) )) {fprintf(stderr, "out of memory allocating sumyy!\n"); exit(0);}
	if ( ! (max_t = calloc(max_bins , sizeof(double)) )) {fprintf(stderr, "out of memory allocating sumyy!\n"); exit(0);}


	for (j=0; j<max_bins; j++) {count[j]=0; sumy[j]=0.0; sumyy[j]=0.0; min_t[j]=1.0e30; max_t[j]=-1.0e30;}  //initial values for all bins

	//now sum values from time series into bins of size <scale>, starting from t_ref (which is passed to this function from outside)
	for (i=0; i<n_data; i++) {  
		j=trunc((t[i]-t_ref)/scale);
		sumy[j]=sumy[j]+x[i];  //confusion alert!  sumy and sumyy hold the sums of x[i]'s and their squares, whereas sumx and sumxx hold the sums of t[i]'s and their squares.
		sumyy[j]=sumyy[j]+x[i]*x[i];
		sumx[j]=sumx[j]+t[i];  //confusion alert!  sumy and sumyy hold the sums of x[i]'s and their squares, whereas sumx and sumxx hold the sums of t[i]'s and their squares.
		sumxx[j]=sumxx[j]+t[i]*t[i];
		sumxy[j]=sumxy[j]+t[i]*x[i];
		count[j]=count[j]+1;
		if (min_t[j]>t[i]) min_t[j]=t[i];
		if (max_t[j]<t[i]) max_t[j]=t[i];
	}
	
	//now work through of bins and calculate means and variances
	for (j=0; j<max_bins; j++) { if ((count[j]>=(normal_count*0.75)) && (count[j]>2.0)) {  //only count bins with less than 25% missing points
				avg[j]=sumy[j]/count[j]; 
				var[j]=(sumyy[j] - (sumy[j]*sumy[j]/(double)count[j]))/(count[j]-1.0);  //sample variance
				slope[j]=(sumxy[j] - (sumx[j]*sumy[j]/(double)count[j]))/(sumxx[j] - (sumx[j]*sumx[j]/(double)count[j]));  //regression slope
				r_squared = (sumxy[j] - (sumx[j]*sumy[j]/(double)count[j]))/(count[j]-1.0);  //right now this is just covariance;
				r_squared = r_squared*r_squared;  //square the covariance
				r_squared = r_squared/(var[j]*(sumxx[j] - (sumx[j]*sumx[j]/(double)count[j]))/(count[j]-1.0)); // divided by the variance of y (var[j]) and x, gives r_squared
				slope_se[j] = fabs(slope[j])*sqrt(1.0/r_squared - 1.0)/sqrt((double)count[j] - 2.0) ; //this gives the standard error of the slope
			} else {
				avg[j]=null;
				var[j]=null;
				slope[j]=null;
				slope_se[j]=null;
			}
		}

	//now work through bins and calculate differences and t-statistics
	for (j=1; j<max_bins; j++) {
		if (slope[j-1]!=null) {  //if the slope is non-null...

			jittered_scale = scale * ( 1.0 + 0.3*((double)rand()/(double)RAND_MAX -0.5) );
			trend_t = fabs(slope[j-1])/slope_se[j-1];
			trend_df = count[j-1]-2;
			trend_p = tdist(trend_t, trend_df);

			fprintf (outfile, "%i\t%s\t%g\t%g\t%g\t%g\t%g\t%g\t%i\t%g\t", column+1, header[column], sampling_interval, scale, jittered_scale, min_t[j-1], slope[j-1], trend_t, trend_df, trend_p);
			if (trend_p<=0.05) fprintf (outfile, "%i\t", 1); else fprintf (outfile, "%i\t", 0);
			if (trend_p<=0.01) fprintf (outfile, "%i\t", 1); else fprintf (outfile, "%i\t", 0);
			if (trend_p<=0.001) fprintf (outfile, "%i\t", 1); else fprintf (outfile, "%i\t", 0);
					
			if ((var[j]!=null)&&(var[j-1]!=null) && ((var[j]>0.0)||(var[j-1]>0.0))) {  //now if the following bin has sufficient points, and pooled variance is >0

				df = count[j]+count[j-1]-2;  //degrees of freedom for difference between means
				pooled_var = ((count[j]-1)*var[j]+(count[j-1]-1)*var[j-1])/df;  //pooled variance
				t_value = fabs(avg[j]-avg[j-1])/sqrt(pooled_var/(double)count[j]+pooled_var/(double)count[j-1]);  //t-statistic of difference between means
				diff_p = tdist(t_value, df);

				fprintf (outfile, "%g\t%g\t%i\t%g\t", avg[j]-avg[j-1], t_value, df, tdist(t_value, df) );
				if (diff_p<=0.05) fprintf (outfile, "%i\t", 1); else fprintf (outfile, "%i\t", 0);
				if (diff_p<=0.01) fprintf (outfile, "%i\t", 1); else fprintf (outfile, "%i\t", 0);
				if (diff_p<=0.001) fprintf (outfile, "%i\t", 1); else fprintf (outfile, "%i\t", 0);

				trend_change_t = fabs(slope[j] - slope[j-1])/sqrt(slope_se[j]*slope_se[j] + slope_se[j-1]*slope_se[j-1]);  //difference between slopes, divided by standard error of that difference
				trend_change_df = (count[j]-2)+(count[j-1]-2);
				trend_change_p = tdist(trend_change_t, trend_change_df);
	
				fprintf (outfile, "%g\t%g\t%i\t%g\t", slope[j]-slope[j-1], trend_change_t, trend_change_df, trend_change_p );

				if (trend_change_p<=0.05) fprintf (outfile, "%i\t", 1); else fprintf (outfile, "%i\t", 0);
				if (trend_change_p<=0.01) fprintf (outfile, "%i\t", 1); else fprintf (outfile, "%i\t", 0);
				if (trend_change_p<=0.001) fprintf (outfile, "%i\t", 1); else fprintf (outfile, "%i\t", 0);
				if (slope[j]*slope[j-1]<=0) fprintf (outfile, "%i\n", 1); else fprintf (outfile, "%i\n", 0);

			} else fprintf(outfile, "\n"); //if we only have a slope from one bin, and the next bin doesn't have the required data density so comparisons with it are skipped
		}
	}

	free(avg);
	free(count);
	free(var);
	free(sumy);
	free(sumyy);
	free(min_t);
	free(max_t);
	free(sumx);
	free(sumxx);
	free(sumxy);
	free(slope);
	free(slope_se);



	return(0);

} //calculate_t-stat



	





int main( void ) {
	int i;		//index for iterations
	int j;		//index for columns
	int k;
	char outfilename[100];  //output filename string
	char outfilespec[100];  //output filename string
	double t_ref;

	start_time=time(NULL);
	srand((int)start_time);	//initialize random number generator

	for (j=0; j<n_columns; j++) header[j]=malloc(sizeof(char[header_len]));

	//define array of scales
//	if (linear_scales==true) for (k=0; k<n_scales; k++) scale[k]=sampling_interval*(double)(k+1);
//	else {
//		scale[0]=sampling_interval;
//		for (k=1; k<n_scales; k++) scale[k]=2.0*scale[k-1];  //each averaging scale is 2x the previous one
//	}
		

	
//  open an output file
	strcpy(outfilespec, filespec);
	

	strcpy(outfilename, outfilespec);
		if (fourway==true) strcat(outfilename, "_4way");
		strcat(outfilename, "_pvalues.txt");
	if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
	else {fprintf (outfile, "series#\tlabel\tsampling_interval\tscale\tjittered_scale\ttime\tslope\tslope_t\tslope_df\ttslope_p\t");  //header line for output file
		fprintf (outfile, "slope_p<0.05\tslope_p<0.01\tslope_p<0.001\t");  //more header line
		fprintf (outfile, "mean_diff\tdiff_t\tdiff_df\tdiff_p\t");
		fprintf (outfile, "diff_p<0.05\tdiff_p<0.01\tdiff_p<0.001\t");  //more header line
		fprintf (outfile, "slope_change\tslope_change_t\tslope_change_df\tslope_change_p\t");  //more header line
		fprintf (outfile, "slope_change_p<0.05\tslope_change_p<0.01\tslope_change_p<0.001\tslopes_opposite_sign\n");  //more header line
	}
	
	for (j=0; j<n_columns; j++) {	//step through read_columns
		read_in_data(j+first_column);
fprintf(stderr, "\n");
fprintf(stderr, "%i data read in... elapsed time %g seconds\n", j+first_column, difftime( time(NULL) , start_time ) );

		for (k=0; k<n_scales; k++) { //step through scales
			t_ref=t[0]; calculate_t_stats(j, k, scale[k], t_ref); 
			if (fourway==true){
				t_ref=t[0]-0.25*scale[k]; calculate_t_stats(j, k, scale[k], t_ref); //redo with 1/4 bin offset, to test different break points between bins
				t_ref=t[0]-0.5*scale[k]; calculate_t_stats(j, k, scale[k], t_ref); //redo with 1/2 bin offset, to test different break points between bins
				t_ref=t[0]-0.75*scale[k]; calculate_t_stats(j, k, scale[k], t_ref); //redo with 3/4 bin offset, to test different break points between bins
			}
		}

		fprintf(stderr, "%i done... elapsed time %g seconds\n", j+first_column, difftime( time(NULL) , start_time ) );
		free(x);
		free(t);		
	}	//finish working through read_columns
	


	fprintf(stderr, "analysis done... elapsed time %g seconds\n", difftime( time(NULL) , start_time ) );

	
	
	fclose(outfile);
	
	return 0;
}

