

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
					
					
#define FTYPE double				//modify to change level of precision
#define DTYPE double				//modify to change level of precision
#define ITYPE long					//modify if needed for very long arrays
#define MAX_LINE_LENGTH 5000	//length, in chars, of max line in input file
#define sqr( a )((a) * (a))				//amazing, the functions that C leaves out!
#define true 1
#define false 0






/*
*******************************************************************************************************
*******************************************************************************************************
READ ME FIRST:

This source code was written by J.W. Kirchner, who holds copyright to it.  It is being made available
to the public under the Creative Commons Attribution-NonCommercial-ShareAlike License (CC BY-NC-SA).
That means you are free:
to Share  to copy, distribute and transmit the work
to Remix  to adapt the work
Under the following conditions:

Attribution  You must attribute the work in the manner specified by the author or licensor (but not in any way that suggests that they endorse you or your use of the work).
Noncommercial  You may not use this work for commercial purposes.
Share Alike  If you alter, transform, or build upon this work, you may distribute the resulting work only under the same or similar license to this one.
Any of the above conditions can be waived if you get permission from the copyright holder.

For details see:
http://creativecommons.org/licenses/by-nc-sa/3.0/
http://creativecommons.org/licenses/by-nc-sa/3.0/legalcode
*******************************************************************************************************
*******************************************************************************************************
*/











//This routine reads multiple spectra with different sampling frequencies and fits them with the same model spectrum,
//then uses that model to filter aliases.

//Alias filtering is performed as described in Kirchner, J.W., Aliasing in 1/f^alpha noise spectra: origins, consequences, and remedies, 
//Physical Review E, 71, 066110, 2005 (except as noted below).  References to, e.g., "equations 24 and 26" refer to numbered equations 
//in that paper.




/*
*******************************************************************************************************************
*******************************************************************************************************************
The following 3 blocks define the option instructions for alias filtering of the 7-hour and weekly Plynlimon data.
Un-comment each of these blocks to process each of the source files (Upper Hafren, Lower Hafren, and Rainfall, each
with both 7-hour and weekly samples).

The spectral alias filtering that is done in this routine differs from that which is done in the spectral analysis
code, in one respect: here, both the 7-hourly and weekly spectra are fitted with a single spectral model, rather than
fitted individually.  We thus determine the best-fit value of alpha, jointly from both spectra.  This allows a more
precise estimate of alpha.  For the same value of alpha, aliasing will affect the weekly time series more than the
7-hourly time series, since the Nyquist frequency is lower by a factor of 24 and thus the aliased power is greater.
*******************************************************************************************************************
*******************************************************************************************************************
*/





//	this is for alias filtering of the Upper Hafren stream chemistry spectra (weekly and 7hourly combined)
	const char infilename[100]="UHF spectra combined c=0.001 mode9.txt";
	const char filespec[100]="UHF c=0.001 mode9";


#define n_columns 47		//number of data columns to read
const int first_column=1;		//first data column (first data column comes after freq and fs)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1770	//this needs to be the number of frequencies in all of the spectra together (the number of rows in the input file, not counting the header)
#define n_fs 2		//this needs to be the number of different sampling frequencies, and must be consistent with the definition of fs... don't get this wrong!

	const float fs[n_fs] = {52.0, 1250.0};  //enumerate here the sampling frequencies
	const int multiple_prefactors = true;  //if true, fits a different prefactor for each sampling frequency fs

	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=false;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?

	const int do_alias_filtering=true;
	
	const int header_len=50;		//max length, in characters, of header
	const double pi=3.14159265358979324;
	const double twopi=6.28318530717958648;
	const FTYPE null=-999.0; //null value for errors etc.    Note: system value NULL (uppercase) is different!







/*
//	this is for alias filtering of the Lower Hafren stream chemistry spectra (weekly and 7hourly combined)
	const char infilename[100]="LHF spectra combined c=0.001 mode9.txt";
	const char filespec[100]="LHF c=0.001 mode9";


#define n_columns 47		//number of data columns to read
const int first_column=1;		//first data column (first data column comes after freq and fs)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1770	//this needs to be the number of frequencies in all of the spectra together (the number of rows in the input file, not counting the header)
#define n_fs 2		//this needs to be the number of different sampling frequencies, and must be consistent with the definition of fs... don't get this wrong!

	const float fs[n_fs] = {52.0, 1250.0};  //enumerate here the sampling frequencies
	const int multiple_prefactors = true;  //if true, fits a different prefactor for each sampling frequency fs

	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=false;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?

	const int do_alias_filtering=true;
	
	const int header_len=50;		//max length, in characters, of header
	const double pi=3.14159265358979324;
	const double twopi=6.28318530717958648;
	const FTYPE null=-999.0; //null value for errors etc.    Note: system value NULL (uppercase) is different!
*/







/*
//	this is for alias filtering of the RAIN chemistry spectra (weekly and 7hourly combined)
	const char infilename[100]="CR spectra combined c=0.001 mode9.txt";
	const char filespec[100]="CR c=0.001 mode9";


#define n_columns 47		//number of data columns to read
const int first_column=1;		//first data column (first data column comes after freq and fs)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1770	//this needs to be the number of frequencies in all of the spectra together (the number of rows in the input file, not counting the header)
#define n_fs 2		//this needs to be the number of different sampling frequencies, and must be consistent with the definition of fs... don't get this wrong!

	const float fs[n_fs] = {52.0, 1250.0};  //enumerate here the sampling frequencies
	const int multiple_prefactors = true;  //if true, fits a different prefactor for each sampling frequency fs

	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=true;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?

	const int do_alias_filtering=true;
	
	const int header_len=50;		//max length, in characters, of header
	const double pi=3.14159265358979324;
	const double twopi=6.28318530717958648;
	const FTYPE null=-999.0; //null value for errors etc.    Note: system value NULL (uppercase) is different!
*/








/*
*******************************************************************************************************************
*******************************************************************************************************************
The blocks below define the option instructions for alias filtering of the synthetic benchmark data sets.

The spectral alias filtering that is done in this routine differs from that which is done in the spectral analysis
code, in one respect: here, both the 7-hourly and weekly spectra are fitted with a single spectral model, rather than
fitted individually.  We thus determine the best-fit value of alpha, jointly from both spectra.  This allows a more
precise estimate of alpha.  For the same value of alpha, aliasing will affect the weekly time series more than the
7-hourly time series, since the Nyquist frequency is lower by a factor of 24 and thus the aliased power is greater.
*******************************************************************************************************************
*******************************************************************************************************************
*/









/*
//	this is for alias filtering of the spectra from benchmark RAIN data sets (weekly and 7hourly combined)
	const char infilename[100]="CR spectra combined benchmk mode9 c=0.001 alpha=1.txt";
	const char filespec[100]="CR benchmk alpha=1";


#define n_columns 47   // 47 for RAIN benchmark data		//number of data columns to read
const int first_column=1;		//first data column (first data column comes after freq and fs)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1770	//this needs to be the number of frequencies in all of the spectra together (the number of rows in the input file, not counting the header)
#define n_fs 2		//this needs to be the number of different sampling frequencies, and must be consistent with the definition of fs... don't get this wrong!

	const float fs[n_fs] = {52.0, 1250.0};  //enumerate here the sampling frequencies
	const int multiple_prefactors = true;  //if true, fits a different prefactor for each sampling frequency fs

	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=true;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?

	const int do_alias_filtering=true;
	
	const int header_len=50;		//max length, in characters, of header
	const double pi=3.14159265358979324;
	const double twopi=6.28318530717958648;
	const FTYPE null=-999.0; //null value for errors etc.    Note: system value NULL (uppercase) is different!
*/








/*
//	this is for alias filtering of the spectra from DCDFT RAIN data sets (weekly and 7hourly combined)
	const char infilename[100]="CR spectra combined DCDFT mode1 c=0 alpha=1.txt";
	const char filespec[100]="CR DCDFT alpha=1";


#define n_columns 47   // 47 for RAIN benchmark data		//number of data columns to read
const int first_column=1;		//first data column (first data column comes after freq and fs)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1770	//this needs to be the number of frequencies in all of the spectra together (the number of rows in the input file, not counting the header)
#define n_fs 2		//this needs to be the number of different sampling frequencies, and must be consistent with the definition of fs... don't get this wrong!

	const float fs[n_fs] = {52.0, 1250.0};  //enumerate here the sampling frequencies
	const int multiple_prefactors = true;  //if true, fits a different prefactor for each sampling frequency fs

	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=true;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?

	const int do_alias_filtering=true;
	
	const int header_len=50;		//max length, in characters, of header
	const double pi=3.14159265358979324;
	const double twopi=6.28318530717958648;
	const FTYPE null=-999.0; //null value for errors etc.    Note: system value NULL (uppercase) is different!

*/




/*
//	this is for alias filtering of the spectra from LSFT RAIN data sets (weekly and 7hourly combined)
	const char infilename[100]="CR spectra combined LSFT mode5 c=0 alpha=-0.5.txt";
	const char filespec[100]="CR LSFT alpha=-0.5";


#define n_columns 47   // 47 for RAIN benchmark data		//number of data columns to read
const int first_column=1;		//first data column (first data column comes after freq and fs)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1770	//this needs to be the number of frequencies in all of the spectra together (the number of rows in the input file, not counting the header)
#define n_fs 2		//this needs to be the number of different sampling frequencies, and must be consistent with the definition of fs... don't get this wrong!

	const float fs[n_fs] = {52.0, 1250.0};  //enumerate here the sampling frequencies
	const int multiple_prefactors = true;  //if true, fits a different prefactor for each sampling frequency fs

	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=true;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?

	const int do_alias_filtering=true;
	
	const int header_len=50;		//max length, in characters, of header
	const double pi=3.14159265358979324;
	const double twopi=6.28318530717958648;
	const FTYPE null=-999.0; //null value for errors etc.    Note: system value NULL (uppercase) is different!
*/







/*
//	this is for alias filtering of the spectra from benchmark data sets (weekly and 7hourly combined)

	const char infilename[100]="UHF spectra combined benchmk mode9 c=0.001 alpha=1.txt";
	const char filespec[100]="UHF benchmk alpha=1";

//	const char infilename[100]="UHF spectra combined LSFT mode5 alpha=0.txt";
//	const char filespec[100]="UHF LSFT alpha=0";

//	const char infilename[100]="UHF spectra combined benchmk DCDFT mode1 alpha=0.75.txt";
//	const char filespec[100]="UHF DCDFT alpha=0.75";


#define n_columns 47   // 92 for real chemistry data, 47 for benchmark data		//number of data columns to read
const int first_column=1;		//first data column (first data column comes after freq and fs)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1770	//this needs to be the number of frequencies in all of the spectra together (the number of rows in the input file, not counting the header)
#define n_fs 2		//this needs to be the number of different sampling frequencies, and must be consistent with the definition of fs... don't get this wrong!

	const float fs[n_fs] = {52.0, 1250.0};  //enumerate here the sampling frequencies
	const int multiple_prefactors = true;  //if true, fits a different prefactor for each sampling frequency fs

	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=false;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?

	const int do_alias_filtering=true;
	
	const int header_len=50;		//max length, in characters, of header
	const double pi=3.14159265358979324;
	const double twopi=6.28318530717958648;
	const FTYPE null=-999.0; //null value for errors etc.    Note: system value NULL (uppercase) is different!

*/





/*
//	this is for alias filtering of the spectra from benchmark RAIN data sets (weekly and 7hourly combined)

	const char infilename[100]="CR spectra combined benchmk mode9 c=0.001 alpha=1.txt";
	const char filespec[100]="CR benchmk alpha=1";

//	const char infilename[100]="CR spectra combined benchmk DCDFT mode1 alpha=-0.5.txt";
//	const char filespec[100]="CR DCDFT alpha=-0.5";


#define n_columns 47   // 92 for real chemistry data, 47 for benchmark data		//number of data columns to read
const int first_column=1;		//first data column (first data column comes after freq and fs)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1770	//this needs to be the number of frequencies in all of the spectra together (the number of rows in the input file, not counting the header)
#define n_fs 2		//this needs to be the number of different sampling frequencies, and must be consistent with the definition of fs... don't get this wrong!

	const float fs[n_fs] = {52.0, 1250.0};  //enumerate here the sampling frequencies
	const int multiple_prefactors = true;  //if true, fits a different prefactor for each sampling frequency fs

	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=true;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?

	const int do_alias_filtering=true;
	
	const int header_len=50;		//max length, in characters, of header
	const double pi=3.14159265358979324;
	const double twopi=6.28318530717958648;
	const FTYPE null=-999.0; //null value for errors etc.    Note: system value NULL (uppercase) is different!
*/














	
// global typedefs
	typedef FTYPE *fptr;			//pointer to float-type
	typedef DTYPE *dptr;			//pointer to double-type


// global variables 
	char *header[n_columns];
	fptr t, x;			//pointers to arrays of data point times and values
	
	FILE *outfile;
	FILE *infile;
	FILE *locfile;
	FILE *pointfile;
	time_t start_time;	//start time
	
	FTYPE freq[n_freq];	//circular frequency (cycles/time)
	int fsi[n_freq];	//index for sampling frequency (one value for each line of input file)
	FTYPE lambda[n_freq];	//wavelength

	FTYPE save_pwr[n_columns][n_freq];				//array to store results
	FTYPE filtered_pwr[n_columns][n_freq];
	FTYPE save_model[n_columns][n_freq];
	FTYPE save_aliased[n_columns][n_freq];
	FTYPE save_alpha[n_columns];
	FTYPE save_alpha_se[n_columns];
	FTYPE save_regr_alpha[n_columns];
	FTYPE save_regr_alpha_se[n_columns];


#include <ctype.h>
int read_string(FILE *infile, char *string, int len) {	
	int i, x;	//reads an input file for a string.  reads until the first whitespace character
	i=0;
	do { 
		x=fgetc(infile);
		if ( (! isspace(x)) && (i<len-1)) {string[i]=x; i=i+1;}
	} while (! isspace(x));
	string[i]=0;	//null termination
return (0);
}//read_string


int read_string_from_string(char *source, char *string, int len) {	
	int i, x;	//reads an input string for a string.  Reads until the first whitespace character or end of string
	i=0;
	do { 
		x=source[i];
		if ( (! isspace(x)) && (i<len-1)) {string[i]=x; i=i+1;}
	} while (! isspace(x));
	string[i]=0;	//null termination
return (0);
}//read_string






double misfit(double alpha, int *fsi, double fc, double fcutoff, double *freq, double *log_pwr, double *model, double *aliased, int avgs, int n)	
	/* 	Generates modeled power spectrum, with aliases, for a specified scaling exponent
		alpha, and returns its root-mean-square misfit score in comparison to
		the supplied log-transformed power spectrum log_pwr.
		
		Inputs to this function (none of which are changed) are:
			alpha -- power-law scaling exponent
			fs -- sampling frequency
			fc -- corner frequency
			freq -- frequency array
			log_pwr -- log(power) array
			avgs -- flag for whether spectrum is calculated from time-averaged measurements (avgs<>0) or instantaneous measurements (avgs=0)
			n -- length of power spectrum array
		
		Outputs are:
			model -- modeled spectrum
			aliased -- modeled spectrum plus aliases
	*/

{	const double pi=3.14159265358979324; 
	int i; //index for frequencies
 	int k; //index for aliases
 	int counter[n_fs];
	int counter_sum;
 	double alias;  //a single alias power component
 	double prefactor[n_fs];	//leading constant for model power spectrum
  	double tally;  //accumulates misfit score
	double beta;
	double cc, dz, zo;

#define n_integration_steps 50
 
 

 	//here is where the model spectrum is defined (equations 24 and 26)
 	#define spectrum( f )( (avgs==0) ? (pow(f , -alpha)/(1.0 + ((f)*(f))/(fc*fc)))  :   (pow(f , -alpha)/(1.0 + (f)*(f)/(fc*fc))*sin(pi*(f)/fs[fsi[i]])/(pi*(f)/fs[fsi[i]])*sin(pi*(f)/fs[fsi[i]])/(pi*(f)/fs[fsi[i]])) ) ;
 

  	for (i=0; i<n; i++) { //loop through frequencies, build up model and aliased model
 		model[i]=pow(freq[i] , -alpha)/(1.0 + (freq[i]*freq[i])/(fc*fc));   

  		aliased[i]=spectrum(freq[i]);	//Here's a subtle point that's missing in my 2005 paper.  The model spectrum (the "real world" spectrum that we assume)
									 	//does not depend on whether sampling is time-averaged or not.  However the measured spectrum (in our terms the "aliased" spectrum)
 										//will be affected by the damping of high frequencies that comes from time-averaging.  Therefore the model spectrum should be the 
										//simple form of #define spectrum( f ) above, whether the sampling is time-averaged or not.  Only the aliased spectrum should vary
										//depending on time-averaging. The updated (30 Jan 2012) formulation here implements this.  This change will not affect the best-fit
										//value of alpha (which depends only on the aliased spectrum) but it will affect the shape of the filtered spectrum.
  		for (k=1; k<=10; k++){   //loop through aliases, building up spectrum of model+aliases: equations 21 and 28 -- first we do 10 exact iterations
  			alias=spectrum(k*fs[fsi[i]]-freq[i]); aliased[i]=aliased[i]+alias;  //add alias
  			alias=spectrum(k*fs[fsi[i]]+freq[i]); aliased[i]=aliased[i]+alias;	 //add alias		
 		} 

		//After 10 exact iterations, integrate from k=11 to k=infinity
		//alias=integral(S(kfs-f) dk) for k=11 to k=infinity.
		//This integration is done by a change of variables, such that z=(k*fs-f)^(-alpha-1) 
		//and dk=-1/(alpha+1)fs z^(-(alpha+2)/(alpha+1)) dz.
		//so the limits of integration are zo=(11*fs-f)^(-alpha-1) and z=0
		//and the integrand is almost constant across the entire integral.
		//For convenience we define beta=alpha+1.

		//Below is what the program in the 2005 paper says:
		//For time-averaged sampling, we take the mean of sin^2 over the (rapid) cycles in sinc, so that
		//sinc(k*fs-f) is approximated by 1/(2*pi*(k*fs-f)/fs).  Then the entire effect of time-averaged 
		//sampling is a redefinition of beta and of the leading constant for the integration.

		//That explanation is not quite right, although the resulting equations and computer code are correct.  The correct explanation is:
		//For time-averaged sampling, we take the mean of sin^2 over the (rapid) cycles in sinc, so that
		//sinc^2(k*fs-f) is approximated by 1/[2*(pi^2)*((k*fs-f)^2)/(fs^2)].  Then the entire effect of time-averaged 
		//sampling is a redefinition of beta and of the leading constant for the integration.


		if (avgs!=0) {
			beta = alpha + 3.0;
			cc = 1.0/(2.0*pi*pi*beta/fs[fsi[i]]);
		} else {
			beta = alpha+1.0;
			cc = 1.0/(beta*fs[fsi[i]]);
		}

		zo = pow(11.0*fs[fsi[i]]-freq[i] , -beta);
		dz = zo/(double)n_integration_steps;
		for (k=1; k<=n_integration_steps; k++) aliased[i]=aliased[i] + cc / (pow((double)k*dz,(2.0/beta)) +1.0/(fc*fc)) *dz;

		//now do the same thing over again for the (k*fs+f) aliases -- note the integral is the same, just
		//the limits are (slightly) different

		zo = pow(11.0*fs[fsi[i]]+freq[i] , -beta);
		dz = zo/(double)n_integration_steps;
		for (k=1; k<=n_integration_steps; k++) aliased[i]=aliased[i] + cc / (pow((double)k*dz,(2.0/beta)) +1.0/(fc*fc)) *dz;

  	} //loop through frequencies
    

  	//calculate prefactor that minimizes root-mean-square misfit (this will be exponential of mean of logged deviations)
	//there is one prefactor for each sampling frequency

	//the index for the prefactor, just as for the fs, is fsi[i] (which takes on values of, say, 0 or 1 for fs=52 or 1250)

	for (i=0; i<n_fs; i++) {prefactor[i]=0.0;  counter[i]=0;}
	counter_sum=0;

	if (multiple_prefactors==true) {
 	 	for (i=0; i<n; i++) if ((freq[i]>fcutoff)&&(log_pwr[i]!=null)) {prefactor[fsi[i]]=prefactor[fsi[i]] + log_pwr[i]-log(aliased[i]); counter[fsi[i]]=counter[fsi[i]]+1; counter_sum=counter_sum+1;}
 	} else {
	 	for (i=0; i<n; i++) if ((freq[i]>fcutoff)&&(log_pwr[i]!=null)) {prefactor[0]=prefactor[0] + log_pwr[i]-log(aliased[i]); counter[0]=counter[0]+1; counter_sum=counter_sum+1;}
	}

	if (counter_sum==0) {fprintf( stderr, "Abnormal termination in alias_filter: all log_pwr values are null!\n"); exit(0);}
 	
	for (i=0; i<n_fs; i++) if (counter[i]>0) 
			prefactor[i]=exp(prefactor[i]/(double) counter[i]); 
		else prefactor[i]=1.0;  //need this last bit to cover the case where there is only a spectrum for one or the other fs
  	
  	tally=0.0; //now tally misfits

	if (multiple_prefactors==true) 
	  	for (i=0; i<n; i++) {
	  		model[i]=prefactor[fsi[i]]*model[i];
	  		aliased[i]=prefactor[fsi[i]]*aliased[i];
	  		if ((freq[i]>fcutoff)&&(log_pwr[i]!=null)) {tally=tally + (log_pwr[i]-log(aliased[i]))*(log_pwr[i]-log(aliased[i])) ; } //sum of squared deviations of logs
  	} else
	  	for (i=0; i<n; i++) {
	  		model[i]=prefactor[0]*model[i];
	  		aliased[i]=prefactor[0]*aliased[i];
	  		if ((freq[i]>fcutoff)&&(log_pwr[i]!=null)) {tally=tally + (log_pwr[i]-log(aliased[i]))*(log_pwr[i]-log(aliased[i])) ; } //sum of squared deviations of logs
	}

//fprintf (stderr, "%g\t%i\n", tally, counter);
	return(tally);  //return sum of squared error (sum of squared of log deviations between measured spectrum and aliased model)
					//misfit must be SSE in order for the MLE estimation of standard errors to work correctly
} //misfit












int alias_filter(double *freq, int *fsi, double *pwr, int n, double fc, double fcutoff, int avgs, double *filtered_pwr, double *model, double *aliased, double *alpha, double *alpha_se, double *regr_slope, double *regr_slope_se)
/* 
Performs alias filtering of power spectrum, using power-law model
Receives the following:
	freq -- array of length n, of frequencies in power spectrum
	pwr -- array of length n, of spectral power corresponding to frequencies "freq"
	n -- length of freq and pwr arrays
	fs -- array of length n, of sampling frequencies (one for each frequency)
	fcorner -- corner frequency for 1/f^2 steepening of power spectrum
	avgs -- flag for whether spectrum is derived from instantaneous point measurements (avgs=0)
			or from measurements averaged over each sampling interval (avgs<>1)
Returns the following:
	filtered_pwr -- array of length n, of alias-filtered spectral power
	model -- array of length n, with modeled spectral power
	aliased -- array of length n, with modeled spectral power, plus aliases
	filtered_pwr -- array of length n, of alias-filtered spectral power
	alpha -- exponent of power-law model
	alpha_se -- estimate of standard error of alpha
Notations "equation ##" refer to numbered equations in Kirchner, 2005.
*/

{	const double lower_limit=-0.99;  //lower limit for alpha values
	const double upper_limit=10.0; //upper limit for alpha values
	const int iter_limit=1000;  //limit to number of iterations
	double tol=1.0e-5;		//termination tolerance: should be larger than square root of fractional machine precision
	double *log_pwr;		//array holding logarithms of powers
	double *resid;			//array for holding residuals
	int i, counter, iter;  //index
	double sumx, sumy, sumxy, sumxx, sumyy;  //for initial estimate of alpha from log-log regression, and serial correlation
	double a, b, c, ya, yb, yc, newpt, ynew;  //for inverse parabolic interpolation
	double n_eff, correl, fisher_information;	//for MLE estimates of s.e. of alpha
	double *log_f, *log_filt_pwr, *avg_log_f, *avg_log_pwr, *tally;  //for estimating slope from filtered pwr
	double SSy, SSx, SSE;


	//allocate new arrays
	log_pwr=malloc(n*sizeof(double));
		if (log_pwr==NULL) {fprintf( stderr, "Abnormal termination: out of memory allocating log_pwr in alias_filter\n"); exit(0);}

	resid=malloc(n*sizeof(double));
		if (resid==NULL) {fprintf( stderr, "Abnormal termination: out of memory allocating resid in alias_filter\n"); exit(0);}

	log_f=malloc(n*sizeof(double));
	log_filt_pwr=malloc(n*sizeof(double));
	avg_log_f=malloc(n_fs*sizeof(double));
	avg_log_pwr=malloc(n_fs*sizeof(double));
	tally=malloc(n_fs*sizeof(double));

	//initialize
	for (i=0; i<n; i++) {
		if (pwr[i]==null) log_pwr[i]=null;
		else {if (pwr[i]>0.0) log_pwr[i]=log(pwr[i]);  //log-transform spectral power
			else  {fprintf( stderr, "Abnormal termination in alias_filter: spectrum contains zero or negative power values! row=%i freq=%g pwr=%g \n", i+1, freq[i], pwr[i]); exit(0);}
		}
		if (freq[i]==null) log_f[i]=null; else log_f[i]=log(freq[i]);
		if ((freq[i]<0) || (freq[i]>0.5*fs[fsi[i]]))  {fprintf( stderr, "Abnormal termination in alias_filter: supplied frequencies are negative or above Nyquist freq\n"); exit(0);}
	}
	
	//We will find the best-fit value of alpha by inverse parabolic interpolation.  At each step, we keep a triplet of trial 
	//values a, b, and c, such that a<b<c, and such that their misfit values ya, yb, and yc obey yb<ya and yb<yc.
	//That is, we will always keep b between a and c, and always keep yb less than ya and yc. 
	//This algorithm is not very robust, but it is simple.
	//The results have been verified against independent calculations in Excel
	



	//get a first estimate of alpha from log-log regression, allowing for an offset between different curves for different sampling frequencies

	for (i=0; i<n_fs; i++) {avg_log_f[i]=0.0; avg_log_pwr[i]=0.0; tally[i]=0.0;}  //zero these before taking sums

	for (i=0; i<n; i++) if (log_pwr[i]!=null) {  //take sums of log power and log frequency, separately for each sampling frequency fs
		avg_log_f[fsi[i]]=avg_log_f[fsi[i]]+log_f[i]; avg_log_pwr[fsi[i]]=avg_log_pwr[fsi[i]]+log_pwr[i]; tally[fsi[i]]=tally[fsi[i]]+1.0;
	}

	for (i=0; i<n_fs; i++) if (tally[i]>0.0) {avg_log_f[i]=avg_log_f[i]/tally[i]; avg_log_pwr[i]=avg_log_pwr[i]/tally[i];} //calculate averages of log power and log frequency
		else {avg_log_f[i]=null; avg_log_pwr[i]=null;}

	//now calculate sums using centered logs (individually centered for each fs) 
	sumx=0.0; sumy=0.0; sumxx=0.0; sumyy=0.0; sumxy=0.0; counter=0;
	for (i=0; i<n; i++) if (log_pwr[i]!=null) {
		sumx=sumx+(log_f[i]-avg_log_f[fsi[i]]);
		sumy=sumy+(log_pwr[i]-avg_log_pwr[fsi[i]]);
		sumxx=sumxx+(log_f[i]-avg_log_f[fsi[i]])*(log_f[i]-avg_log_f[fsi[i]]);
		sumyy=sumyy+(log_pwr[i]-avg_log_pwr[fsi[i]])*(log_pwr[i]-avg_log_pwr[fsi[i]]);
		sumxy=sumxy+(log_f[i]-avg_log_f[fsi[i]])*(log_pwr[i]-avg_log_pwr[fsi[i]]);
		counter=counter+1;
	}

	//first (center) estimate of alpha comes from the regression slope (note that it's the *negative* of the regression slope!):
	b = - (sumxy-sumx*sumy/(double)counter)/(sumxx-sumx*sumx/(double)counter);


	//now get misfit estimate for that alpha
	yb=misfit(b, fsi, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n); //first estimate of misfit
	


	//now find two alpha values, on either side of our central estimate, that have higher misfit values
	a=b; do { if (a>lower_limit+0.3) a=a-0.2; else a=a-0.05; ya=misfit(a, fsi, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n); if (ya<yb) {b=a; yb=ya;} 
		if (a<lower_limit) {
			*alpha=null; *alpha_se=1.0; 
				for (i=0; i<n; i++) filtered_pwr[i]=null;
			return(1);
		}
	} while (ya<=yb);  //repeat this until we find a point with higher misfit
	c=b+0.2; do {c=c+0.1; yc=misfit(c, fsi, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n); if (yc<yb) {b=c; yb=yc;} 
		if (c>upper_limit) {
			*alpha=null; *alpha_se=2.0; 
				for (i=0; i<n; i++) filtered_pwr[i]=null;
			return(2);
		}
	
	} while (yc<=yb);  //repeat this until we find a point with higher misfit


	iter = 0;
	do {  //inverse parabolic interpolation to search for best-fit alpha value
		iter = iter+1;
		newpt=b-0.5*((b-a)*(b-a)*(yb-yc) - (b-c)*(b-c)*(yb-ya))/((b-a)*(yb-yc) - (b-c)*(yb-ya));  //inverse parabolic interpolation
		ynew=misfit(newpt, fsi, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n);  //function value at that new point
		
		if (ynew<yb) { //if the new point is an improvement over the center point, replace the center point with it
			if (newpt<b) {c=b; yc=yb;} //shuffle rightmost point inward if new minimum is left of b
				else {if (newpt>b) {a=b; ya=yb;} //shuffle leftmost point inward if new minimum is right of b
					else {fprintf( stderr, "Abnormal termination in alias_filter: parabolic interpolation failed\n");
						*alpha=null; *alpha_se=3.0; 
						for (i=0; i<n; i++) filtered_pwr[i]=null;
						return(3);
					} } //if new point lands right on b, interpolation fails
			b=newpt; yb=ynew; }
		else {  //if the new point is higher than the center point, replace one of the outer points with it
			if (newpt<b) {a=newpt; ya=ynew;} //replace leftmost point if new minimum is left of b
				else {if (newpt>b) {c=newpt; yc=ynew;} //replace rightmost point if new minimum is right of b
					else {fprintf( stderr, "Abnormal termination in alias_filter: parabolic interpolation failed\n");
						*alpha=null; *alpha_se=4.0; 
						for (i=0; i<n; i++) filtered_pwr[i]=null;
						return(4);
				} } //if new point lands right on b, interpolation fails
			}

		//if the middle point is much closer to one boundary point than the other, do a golden section step 
		if ( (c-b)>5.0*(b-a) ) {  //golden section step in right-hand interval
			newpt=b+0.38197*(c-b);
			ynew=misfit(newpt, fsi, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n);  //function value at that new point
			if (ynew<yb) { a=b; ya=yb; b=newpt; yb=ynew; }  //if the new point is an improvement over the center point
				else  {c=newpt; yc=ynew;} //otherwise just move the right boundary inward
		}  
		
		if ( (b-a)>5.0*(c-b) ) {  //golden section step in left-hand interval
			newpt=b-0.38197*(b-a);
			ynew=misfit(newpt, fsi, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n);  //function value at that new point
			if (ynew<yb) { c=b; yc=yb; b=newpt; yb=ynew; }  //if the new point is an improvement over the center point
				else  {a=newpt; ya=ynew;} //otherwise just move the left boundary inward
		}  
				
		if ((yb>ya) || (yb>yc)) {fprintf( stderr, "Abnormal termination in alias_filter: parabolic interpolation failed\n");
								*alpha=null; *alpha_se=5.0; 
						for (i=0; i<n; i++) filtered_pwr[i]=null;
						return(5);
		}  // this should never happen, but just in case... 






	} while ( (fabs((c-a)/(c+a))>tol)&&(iter<=iter_limit));  //termination condition: endpoints are close enough that further midpoint improvement is pointless

	if (iter>iter_limit) {
		*alpha=null; *alpha_se=6.0; 
		for (i=0; i<n; i++) filtered_pwr[i]=null;
		return(6);
	}
	
	*alpha=b;
	
	for (i=0; i<n; i++) if (pwr[i]!=null) filtered_pwr[i]=pwr[i]*model[i]/aliased[i];  //here is the alias filter (equation 23)
						else filtered_pwr[i]=null;

	


	//now we estimate the standard error of alpha using maximum likelihood methods
	//first we look at the correlation between the residuals and their lags, to estimate the loss of degrees of freedom due to the smoothing of the spectrum
	
	for (i=0; i<n; i++) if (log_pwr[i]!=null) resid[i]=log_pwr[i]-log(aliased[i]); else resid[i]=null;  //calculate residuals in log space

	//now calculate the correlation between these residuals and their lags
	sumx=0.0; sumy=0.0; sumxx=0.0; sumyy=0.0; sumxy=0.0; counter=0;
	for (i=1; i<n; i++) if ((resid[i]!=null) && (resid[i-1]!=null)) {
			sumx=sumx+resid[i];
			sumy=sumy+resid[i-1];
			sumxx=sumxx+resid[i]*resid[i];
			sumyy=sumyy+resid[i-1]*resid[i-1];
			sumxy=sumxy+resid[i]*resid[i-1];
			counter=counter+1;
		}

	correl=(sumxy-sumx*sumy/(double)counter)/(sqrt(sumxx-sumx*sumx/(double)counter)*sqrt(sumyy-sumy*sumy/(double)counter));

	n_eff=(double)counter*(1.0-correl)/(1.0+correl);  //number of effective independent power estimates

	//remember, b and yb are still the best-fit alpha and the misfit value at that alpha
	newpt=b*(1.0+100.0*tol);  //now perturb the value of alpha
	ynew=misfit(newpt, fsi, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n);  //misfit at that new point

	fisher_information = n_eff * ((ynew/yb)-1.0) / ((newpt-b)*(newpt-b));  //because we're assuming Gaussian errors 
				//with sigma estimated from best-fit SSE/n_eff (effectively correcting for loss of degrees of freedom), MLE gives just this simple expression for a 1-dimensional fit!
				//This formula estimates - d2y/dx2 where y is log-likelihood = -(1/(2*sigma^2))*SSE,
				//where sigma^2 is estimated as SSE at the best-fit point, divided by the effective degrees of freedom (approximated by n_eff).  

	*alpha_se = 1.0/sqrt(fisher_information);  //normally this would involve matrix inversion but it's just a one-dimensional fit!




	//now we estimate alpha and its standard error by linear regression of log(filtered power) against log(freq),
	//allowing a different constant intercept (parallel lines) for each sampling frequency.  This has been verified against independent calculations in Excel and JMP

	for (i=0; i<n; i++) {  //log-transform filtered power
		if (filtered_pwr[i]==null) log_filt_pwr[i]=null; else log_filt_pwr[i]=log(filtered_pwr[i]);
	}

	for (i=0; i<n_fs; i++) {avg_log_f[i]=0.0; avg_log_pwr[i]=0.0; tally[i]=0.0;}  //zero these before taking sums

	for (i=0; i<n; i++) if (log_filt_pwr[i]!=null) {  //take sums of log power and log frequency, separately for each sampling frequency fs
		avg_log_f[fsi[i]]=avg_log_f[fsi[i]]+log_f[i]; avg_log_pwr[fsi[i]]=avg_log_pwr[fsi[i]]+log_filt_pwr[i]; tally[fsi[i]]=tally[fsi[i]]+1.0;
	}

	for (i=0; i<n_fs; i++) if (tally[i]>0.0) {avg_log_f[i]=avg_log_f[i]/tally[i]; avg_log_pwr[i]=avg_log_pwr[i]/tally[i];} //calculate averages of log power and log frequency
		else {avg_log_f[i]=null; avg_log_pwr[i]=null;}

	//now calculate sums using centered logs (individually centered for each fs) 
	sumx=0.0; sumy=0.0; sumxx=0.0; sumyy=0.0; sumxy=0.0; counter=0;
	for (i=0; i<n; i++) if (log_filt_pwr[i]!=null) {
		sumx=sumx+(log_f[i]-avg_log_f[fsi[i]]);
		sumy=sumy+(log_filt_pwr[i]-avg_log_pwr[fsi[i]]);
		sumxx=sumxx+(log_f[i]-avg_log_f[fsi[i]])*(log_f[i]-avg_log_f[fsi[i]]);
		sumyy=sumyy+(log_filt_pwr[i]-avg_log_pwr[fsi[i]])*(log_filt_pwr[i]-avg_log_pwr[fsi[i]]);
		sumxy=sumxy+(log_f[i]-avg_log_f[fsi[i]])*(log_filt_pwr[i]-avg_log_pwr[fsi[i]]);
		counter=counter+1;
	}

	*regr_slope = (sumxy-sumx*sumy/(double)counter)/(sumxx-sumx*sumx/(double)counter);
	correl=(sumxy-sumx*sumy/(double)counter)/(sqrt(sumxx-sumx*sumx/(double)counter)*sqrt(sumyy-sumy*sumy/(double)counter));
	SSy = sumyy-sumy*sumy/(double)counter;
	SSE = SSy*(1.0-correl*correl);
	SSx = sumxx-sumx*sumx/(double)counter;

	//now we look at the correlation between the residuals and their lags, to estimate the loss of degrees of freedom due to the smoothing of the spectrum
	for (i=0; i<n; i++) if (log_filt_pwr[i]!=null) resid[i]=(log_filt_pwr[i]- avg_log_pwr[fsi[i]]) - *regr_slope * (log_f[i]-avg_log_f[fsi[i]]); else resid[i]=null;  //calculate residuals in log space

	//now calculate the correlation between these residuals and their lags
	sumx=0.0; sumy=0.0; sumxx=0.0; sumyy=0.0; sumxy=0.0; counter=0;
	for (i=1; i<n; i++) if ((resid[i]!=null) && (resid[i-1]!=null) && (fsi[i]==fsi[i-1])) { //note that we should only calculate serial correlation among pwr estimates that share the same fs
		sumx=sumx+resid[i];
		sumy=sumy+resid[i-1];
		sumxx=sumxx+resid[i]*resid[i];
		sumyy=sumyy+resid[i-1]*resid[i-1];
		sumxy=sumxy+resid[i]*resid[i-1];
		counter=counter+1;
	}

	correl=(sumxy-sumx*sumy/(double)counter)/(sqrt(sumxx-sumx*sumx/(double)counter)*sqrt(sumyy-sumy*sumy/(double)counter));
	n_eff=(double)counter*(1.0-correl)/(1.0+correl);  //number of effective independent power estimates

	*regr_slope_se = sqrt((SSE/SSx)/n_eff);

	free(log_pwr);  	//free array before exiting
	free(resid);
	free(log_f);
	free(log_filt_pwr);
	free(avg_log_f);
	free(avg_log_pwr);
	free(tally);



	return(0);
} //alias_filter








int read_in_data(int read_column, double *pwr)		//opens infile, allocates global arrays, and reads in data
{	ITYPE i, j, k;
	double ff, pp, fsfs; //use these to transfer to possible float-sized freq[i], pwr[i], and fs values
	char junk[MAX_LINE_LENGTH+1]; 
	
	
	 //open input file and verify it exists
	if( (infile=fopen(infilename,"r")) == NULL ) {fprintf(stderr, "input file %s could not be opened!\n",infilename); exit(-1); }

	read_string(infile, junk, header_len);	//discard first column heading
	read_string(infile, junk, header_len);	//discard fs column heading
	for (j=1; j<read_column; j++)  read_string(infile, junk, header_len);	//throw out headers before the one we need
	fgets(junk, MAX_LINE_LENGTH, infile); //get the rest of the line
	read_string_from_string(junk, header[read_column-first_column], header_len); //read that string until the first whitespace character to find the header string we need
	

	for (i=0; i<n_freq; i++) if (feof(infile)==0)
	{	ff=null; pp=null; fsfs=1.0;
		fscanf(infile, "%lg", &ff);		//get the frequency value
		fscanf(infile, "%lg", &fsfs);		//get the fs value
		for (j=1; j<read_column; j++)  fscanf(infile, "%lg", &pp);  //throw out columns before the one we need
		fgets(junk, MAX_LINE_LENGTH, infile);	//get the rest of the line
		sscanf(junk, "%lg ", &pp) ;	//read the one we need -- this is the power value
		freq[i]=ff; pwr[i]=pp; 

		//here's where we assign the fs index, fsi:
		fsi[i]=-1; for (j=0; j<n_fs; j++) {if (fsfs==fs[j]) fsi[i]=j;} 
		if (fsi[i]==-1) {fprintf(stderr, "fatal error: fs doesn't match one of the pre-defined values! column=%i row=%i freq=%g fs=%g\n", read_column, i+1, ff, fsfs); exit(-1); }

	} ; //loop through rows of file

	fclose(infile);



	



	return(0);
}  //end of read_in_data





int print_results( void ) {
	ITYPE j,k;
	char outfilename[100];  //output filename string
	char outfilespec[100];  //output filename string
	
	strcpy(outfilespec, filespec);
	

		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_filtered.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "fs\tfreq");
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s", header[j]);
			fprintf (outfile, "\n");
	
			for (k=0; k<n_freq; k++) {
				fprintf (outfile, "%g\t%g", fs[fsi[k]], freq[k]);
				for (j=0; j<n_columns; j++)  { if (save_pwr[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%g", filtered_pwr[j][k]); } ;
				fprintf (outfile, "\n");
			}//loop frequencies
	
			fclose(outfile);
		} //end else


	

	

	//write file of all powers
		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_allpwr.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "fs\tfreq");
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s.pwr\t%s.filt_pwr\t%s.model\t%s.aliased", header[j],  header[j], header[j], header[j]);
			fprintf (outfile, "\n");
	
			for (k=0; k<n_freq; k++) {
				fprintf (outfile, "%g\t%g", fs[fsi[k]], freq[k]);
				for (j=0; j<n_columns; j++)  { if (save_pwr[j][k]==null) fprintf(outfile, "\t\t\t\t"); else fprintf (outfile, "\t%g\t%g\t%g\t%g", save_pwr[j][k], filtered_pwr[j][k], save_model[j][k], save_aliased[j][k]); } ;
				fprintf (outfile, "\n");
			}//loop frequencies
	
			fclose(outfile);
		} //end else




	//write file of all alphas
		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_alpha.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "series\talpha\talpha_se\tregr_alpha\tregr_alpha_se\n");
			for (j=0; j<n_columns; j++) fprintf (outfile, "%s\t%g\t%g\t%g\t%g\n", header[j], save_alpha[j], save_alpha_se[j], save_regr_alpha[j], save_regr_alpha_se[j]);   //loop frequencies
	
			fclose(outfile);
		} //end else



return (0);
}//print results
	




int main( void ) {
	ITYPE i;		//index for iterations
	ITYPE smooth_width;
	fptr logf;
	int j;		//index for columns
	int k, kmin, kmax;	//index for frequencies
	FTYPE weight, sumwt, wtd_sum;
	FTYPE meanx, meany, meanxx, meanxy;
	char outfilename[100];  //output filename string
	char outfilespec[100];  //output filename string
	
	double filtered[n_freq];
	double model[n_freq];
	double aliased[n_freq];
	double raw[n_freq];
	double alpha, alpha_se, regr_slope, regr_slope_se;

	start_time=time(NULL);
	
	for (j=0; j<n_columns; j++) header[j]=malloc(sizeof(char[header_len]));


		
//here's where the analysis begins
	
	for (j=0; j<n_columns; j++) {	//step through read_columns
		read_in_data(j+first_column, raw);
		for (i=0; i<n_freq; i++) save_pwr[j][i]=raw[i];

fprintf(stderr, "\n");
fprintf(stderr, "%i data read in... elapsed time %g seconds\n", j+first_column, difftime( time(NULL) , start_time ) );

		
	}	//finish working through read_columns
	



if (do_alias_filtering==true) {
	//here we do the alias filtering
	for (j=0; j<n_columns; j++) {
		for (i=0; i<n_freq; i++) raw[i]=save_pwr[j][i]; //input spectra for alias filtering


		alias_filter(freq, fsi, raw, n_freq, fc, fcutoff, avgs, filtered, model, aliased, &alpha, &alpha_se, &regr_slope, &regr_slope_se);  //do alias filtering here
		for (i=0; i<n_freq; i++) {
			filtered_pwr[j][i]=filtered[i];
			save_model[j][i]=model[i];
			save_aliased[j][i]=aliased[i];
			save_alpha[j]=alpha; 
			save_alpha_se[j]=alpha_se;
			save_regr_alpha[j]=-regr_slope;
			save_regr_alpha_se[j]=regr_slope_se;
		}

	fprintf(stderr, "col %i  alpha=%g  s.e=%g   regr_alpha=%g   regr_alpha_s.e.=%g\n", j+1, alpha, alpha_se, -regr_slope, regr_slope_se);

	} //next j
} //if


	fprintf(stderr, "analysis done... elapsed time %g seconds\n", difftime( time(NULL) , start_time ) );

	print_results( );
	
	
//	fclose(locfile);
//	fclose(pointfile);

	
	return 0;
}

