
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
					
#define FTYPE long double				//modify to change level of precision
#define DTYPE long double				//modify to change level of precision
#define ITYPE long					//modify if needed for very long arrays
#define MAX_LINE_LENGTH 5000	//length, in chars, of max line in input file
#define sqr( a )((a) * (a))				//amazing, the functions that C leaves out!
#define true 1
#define false 0



/*
*******************************************************************************************************
*******************************************************************************************************
READ ME FIRST:

This source code was written by J.W. Kirchner, who holds copyright to it.  It is being made available
to the public under the Creative Commons Attribution-NonCommercial-ShareAlike License (CC BY-NC-SA).
That means you are free:
to Share  to copy, distribute and transmit the work
to Remix  to adapt the work
Under the following conditions:

Attribution  You must attribute the work in the manner specified by the author or licensor (but not in any way that suggests that they endorse you or your use of the work).
Noncommercial  You may not use this work for commercial purposes.
Share Alike  If you alter, transform, or build upon this work, you may distribute the resulting work only under the same or similar license to this one.
Any of the above conditions can be waived if you get permission from the copyright holder.

For details see:
http://creativecommons.org/licenses/by-nc-sa/3.0/
http://creativecommons.org/licenses/by-nc-sa/3.0/legalcode
*******************************************************************************************************
*******************************************************************************************************
*/







/*
*******************************************************************************************************
*******************************************************************************************************
These blocks of constants/settings are used to generate benchmark data sets with aliasing.  Need to change
"slope" (alpha) for all desired spectral slopes.  Note that use_random_seed should be false, min_freq 
should be 0.05, and n_freq should be 292009, for both 7-hourly and weekly sampling, so that the theoretical
signal for 7-hourly and weekly sampling is the same.

Users would need to replace the "slope" parameter with the desired alpha value.  Note that 1/f noise has slope=1 (NOT -1).
*******************************************************************************************************
*******************************************************************************************************
*/



// benchmark generation settings block 1
// this is for colored noises with aliasing in 7-hourly streamflow data
#define n_columns 47 //number of data columns to read; actual number of columns MUST be equal or greater!
#define n_freq 292009 //number of frequencies used in synthetic noise: this should be max_freq/min_freq + 9 (to provide low-frequency tail extending to 1/10 of min_freq)
#define max_data 2500 //maximum number of sample times
	const double slope=1.0;
	const FTYPE min_freq=0.05;		//minimum frequency	
									//n_freq*min_freq is maximum frequency

	const char infilename[50]="7hour UHF for spectra.txt";  //for 7-hourly sampling
	const char filespec[50]="7hour_UHF";					//for 7-hourly sampling
	const char label_suffix[50]="fc=1460";
		
	const int white_noise_limit=false; //provides whitenoise cutoff at wavelengths longer than 1.0
	const int seed=4;
	const int use_random_seed=false;	//if true, uses random seed determined by start_time; otherwise uses fixed <seed>
	const double fc=1460.0;			//corner frequency where spectrum rolls off
	const double fs=1250.0; 	 	//sampling frequency (need this for representing time-averaged inputs in precipitation collector)
	const int avgs=false;		//should be true for time-averaged sampling (i.e., precipitation), otherwise should be false



/*
// benchmark generation settings block 2
// this is for colored noises with aliasing in weekly streamflow data
#define n_columns 47 //number of data columns to read; actual number of columns MUST be equal or greater!
#define n_freq 292009 //number of frequencies used in synthetic noise: this should be max_freq/min_freq + 9 (to provide low-frequency tail extending to 1/10 of min_freq)
#define max_data 2500 //maximum number of sample times
	const double slope=1.0;
	const FTYPE min_freq=0.05;		//minimum frequency	
									//n_freq*min_freq is maximum frequency

	const char infilename[50]="weekly UHFw for spectra.txt";	//for weekly sampling
	const char filespec[50]="wkly_UHFw";						//for weekly sampling
	const char label_suffix[50]="fc=1460";
		
	const int white_noise_limit=false; //provides whitenoise cutoff at wavelengths longer than 1.0
	const int seed=4;
	const int use_random_seed=false;	//if true, uses random seed determined by start_time; otherwise uses fixed <seed>
	const double fc=1460.0;			//corner frequency where spectrum rolls off
	const double fs=52.0; 	 	//sampling frequency (need this for representing time-averaged inputs in precipitation collector)
	const int avgs=false;		//should be true for time-averaged sampling (i.e., precipitation), otherwise should be false
*/





/*
// benchmark generation settings block 3
// this is for colored noises with aliasing in 7-hourly RAINFALL data
#define n_columns 43 //number of data columns to read; actual number of columns MUST be equal or greater!
#define n_freq 292009 //number of frequencies used in synthetic noise: this should be max_freq/min_freq + 9 (to provide low-frequency tail extending to 1/10 of min_freq)
#define max_data 2500 //maximum number of sample times
	const double slope=0.5;
	const FTYPE min_freq=0.05;		//minimum frequency	
									//n_freq*min_freq is maximum frequency

	const char infilename[50]="7hour CR for spectra.txt";  //for 7-hourly sampling
	const char filespec[50]="7hour_CR";					//for 7-hourly sampling
	const char label_suffix[50]="fc=1460";
		
	const int white_noise_limit=false; //provides whitenoise cutoff at wavelengths longer than 1.0
	const int seed=4;
	const int use_random_seed=false;	//if true, uses random seed determined by start_time; otherwise uses fixed <seed>
	const double fc=1460.0;			//corner frequency where spectrum rolls off
	const double fs=1250.0; 	 	//sampling frequency (need this for representing time-averaged inputs in precipitation collector)
	const int avgs=true;		//should be true for time-averaged sampling (i.e., precipitation), otherwise should be false
*/





/*
// benchmark generation settings block 4
// this is for colored noises with aliasing in weekly RAINFALL data
#define n_columns 47 //number of data columns to read; actual number of columns MUST be equal or greater!
#define n_freq 292009 //number of frequencies used in synthetic noise: this should be max_freq/min_freq + 9 (to provide low-frequency tail extending to 1/10 of min_freq)
#define max_data 2500 //maximum number of sample times
	const double slope=0.5;
	const FTYPE min_freq=0.05;		//minimum frequency	
									//n_freq*min_freq is maximum frequency

	const char infilename[50]="weekly CRw for spectra.txt";	//for weekly sampling
	const char filespec[50]="wkly_CRw";						//for weekly sampling
	const char label_suffix[50]="fc=1460";
		
	const int white_noise_limit=false; //provides whitenoise cutoff at wavelengths longer than 1.0
	const int seed=4;
	const int use_random_seed=false;	//if true, uses random seed determined by start_time; otherwise uses fixed <seed>
	const double fc=1460.0;			//corner frequency where spectrum rolls off
	const double fs=52.0; 	 	//sampling frequency (need this for representing time-averaged inputs in precipitation collector)
	const int avgs=true;		//should be true for time-averaged sampling (i.e., precipitation), otherwise should be false
*/









/*
*******************************************************************************************************
*******************************************************************************************************
Constants/settings below are common to all benchmark generation runs.
*******************************************************************************************************
*******************************************************************************************************
*/




											
//note spectral power should be 0.5 (variance per sine wave),
//times amplitude of sine wave^2, 
//times 1/min_freq (which is density of sine waves in frequency space)
//however, note that here, amplitude is normalized by density of sine waves, by multiplying amplitude by sqrt(2.0*min_freq).
//thus spectral power should equal amplitude^2, for smooth broad-band signals

	
	const int header_len=50;		//max length, in characters, of header
	const double pi=3.14159265358979324;
	const double twopi=6.28318530717958648;
	const FTYPE null=-999.0; //null value for errors etc.    Note: system value NULL (uppercase) is different!

	
// global typedefs
	typedef FTYPE *fptr;			//pointer to float-type
	typedef DTYPE *dptr;			//pointer to double-type


// global variables 
	int n_data;
	char *header[n_columns]; //holds header strings

	double t[max_data]; //will calloc this to create t[time] to hold sample times
	double x[max_data][n_columns]; //will calloc this to create x[column][time] to hold raw data values (and then, later, the noise time series)
	double amp[n_freq]; //amplitudes of waves
	double cos_phase[n_freq][n_columns], sin_phase[n_freq][n_columns]; //these hold the random phases for each frequency (different for each data column to provide a different random time series)
	
	FILE *outfile;
	FILE *infile;
	time_t start_time;	//start time
	




#include <ctype.h>
int read_string(FILE *infile, char *string, int len) {	
	int i, x;	//reads an input file for a string.  reads until the first whitespace character
	i=0;
	do { 
		x=fgetc(infile);
		if ( (! isspace(x)) && (i<len-1)) {string[i]=x; i=i+1;}
	} while (! isspace(x));
	string[i+1]=0;	//null termination
	return(0);
}//read_string




int read_string_from_string(char *source, char *string, int len) {	
	int i, x;	//reads an input string for a string.  Reads until the first whitespace character or end of string
	i=0;
	do { 
		x=source[i];
		if ( (! isspace(x)) && (i<len-1)) {string[i]=x; i=i+1;}
	} while (! isspace(x));
	string[i]=0;	//null termination
return (0);
}//read_string




int read_in_data(void)		//opens infile, allocates global arrays, and reads in data
{	ITYPE i,j,k;
	double tt, xx; //use these to transfer to possible float-sized t[i], x[i], and pointwt[i]
	char junk[MAX_LINE_LENGTH+1]; 
	
	 //open input file and verify it exists
	if( (infile=fopen(infilename,"r")) == NULL ) {fprintf(stderr, "input file %s could not be opened!\n",infilename); exit(-1); }

	// run through file and count legal points
	n_data=0;
	fgets(junk, MAX_LINE_LENGTH, infile);	//discard first line
	do {	tt=null; xx=null;
		fscanf(infile, "%lg", &tt);		//get the time value
		fgets(junk, MAX_LINE_LENGTH, infile);	//get the rest of the line
		if (tt != null) n_data=n_data+1;  //increment n_data if this is a valid data point
	} while (feof(infile)==0);

	if (n_data>max_data) {fprintf(stderr, "ERROR: max_data is not big enough! Must be bigger than %i\n", n_data); exit(0);}

	// now allocate memory for header
	for (j=0; j<n_columns; j++) header[j]=malloc(sizeof(char[header_len]));


	// reset file, get data values, and store them in t and x
	fclose(infile);
	infile=fopen(infilename,"r");
	read_string(infile, junk, header_len);	//discard first column heading (time)
	for (j=0; j<n_columns-1; j++)  read_string(infile, header[j], header_len);	//read headers
	fgets(junk, MAX_LINE_LENGTH, infile);	//get the rest of the header line in one gulp
	read_string_from_string(junk, header[n_columns-1], header_len); //read that string until the first whitespace character to find last header string
	i=0;
	do {	tt=null; 
		fscanf(infile, "%lg", &tt);		//get the time value
		if (tt != null) {
			t[i]=tt;  //copy to t vector
			for (j=0; j<n_columns-1; j++)  {
				fscanf(infile, "%lg", &xx);  //read across columns
				x[i][j]=xx;  //copy to x array
			}
			fgets(junk, MAX_LINE_LENGTH, infile);	//get the rest of the line
			sscanf(junk, "%lg ", &xx) ;	//get the last x
			x[i][n_columns-1]=xx;  //copy to x array

 			i=i+1;
		} //if
		else fgets(junk, MAX_LINE_LENGTH, infile);	//if time is null then discard the whole line
	} while (feof(infile)==0);
	

	fclose(infile);
	
	

return(0);	
}  //end of read_in_data




int recover_storage( void ) {
	int j;	

	for (j=0; j<n_columns; j++) {
		free(header[j]);
	}
	
	return(1);
} //recover storage





void print_results( void ) {
	ITYPE i,k,j;
	char outfilename[100];  //output filename string
	char outfilespec[100];  //output filename string
	
	double last_cos, last_sin, cos_omega_t, sin_omega_t, cos_omega1, sin_omega1, sin_term, cos_term, omega, dt;
	double *sin_omega_dt, *sin_omega_halfdt;
	double norm, freq, phase;

	strcpy(outfilename, filespec);
	sprintf(outfilespec, "_benchmk_alpha=%g_%s.txt", slope, label_suffix );  
	strcat(outfilename, outfilespec);

	dt=1.0/fs;

	if (avgs==true) { //these terms are used to calculate time-integrated values for time-averaged sampling (avgs==true)
		if ( ! (sin_omega_dt = calloc(n_freq , sizeof(double)) )) {fprintf(stderr, "out of memory allocating sin_omega_dt!\n"); exit(0);}
		if ( ! (sin_omega_halfdt = calloc(n_freq , sizeof(double)) )) {fprintf(stderr, "out of memory allocating sin_omega_halfdt!\n"); exit(0);}
		for (k=0; k<n_freq; k++) {
			if (k<=9) omega=twopi*min_freq*(double)(k+1)/10.0; else omega=twopi*min_freq*(double)(k-8);
			sin_omega_dt[k]=sin(omega*dt);
			sin_omega_halfdt[k]=sin(omega*dt/2.0);
		}
	}

	
	if (use_random_seed==true) srand((int)start_time); else srand(seed);	//initialize random number generator

	norm = sqrt(2.0*min_freq);
	//Normalize by the density of frequencies, by multiplying amplitude by sqrt(2.0*min_freq).  
	//This maintains the same average level of spectral power, regardless of how closely the frequency components are spaced.

	for(k=0; k<n_freq; k++)  { //define power-law amplitude function
		if (k<=9) {
			freq = min_freq*(double)(k+1)/10.0; //the first 10 frequencies are spaced 1/10 as far apart, and have 1/10 of the normalized power (to maintain the correct spectral density, since the first 10 waves are 10x as dense in frequency space)
			amp[k] = (norm/sqrt(10.0))*pow( freq , -0.5*slope )/sqrt(1.0 + (freq*freq)/(fc*fc));        //power-law amplitude function to yield correct power-law slope in spectral power
		} else {
			freq = min_freq*(double)(k-8);
			amp[k] = norm*pow( freq , -0.5*slope )/sqrt(1.0 + (freq*freq)/(fc*fc));        //power-law amplitude function to yield correct power-law slope in spectral power
		}//fc is corner frequency that transitions from 1/f^alpha to 1/f^(alpha+2).  This roll-off damps out higher frequency components.

		for (j=0; j<n_columns; j++) {
			phase = twopi * (double)rand()/(double)RAND_MAX;           //random phase in radians
			cos_phase [k][j] = cos(phase);  //save sine and cosine of phase so we can use them at every point in time.
			sin_phase [k][j] = sin(phase);  //note that phases will be different for each column of the input file (i.e., each random time series)
		}
	}






	if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
	else {
		fprintf (outfile, "time");
		for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s", header[j]);
//		for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s_1/f^%g_%s", header[j], slope, label_suffix);
		fprintf (outfile, "\n");
		
//here's where we actually calculate the time series
		for (i=0; i<n_data; i++) {
			for (j=0; j<n_columns; j++) if (x[i][j]!=null) x[i][j]=0.0;  //if value of x is null (no real data at that time), skip it, otherwise set to zero so we can start summing
			cos_omega_t=1.0;  sin_omega_t=0.0;  //these hold the current values of omega*t
			cos_omega1=cos(twopi*min_freq*t[i]);  //sines and cosines of the fundamental frequency
			sin_omega1=sin(twopi*min_freq*t[i]);
			for (k=0; k<n_freq; k++) {
				if (k<=9) omega=twopi*min_freq*(double)(k+1)/10.0; else omega=twopi*min_freq*(double)(k-8);
				if ((k<=9) || (k % 20 == 10)) {  //every 20 steps, do a real sine and cosine; this should trip for k=0 through k=9 (the 1/10 steps) then for k=10, k=30, and so forth
					cos_omega_t=cos(omega*t[i]);
					sin_omega_t=sin(omega*t[i]); 
				}
				else {  //in between, do sine and cosine by recurrence
					last_cos=cos_omega_t; last_sin=sin_omega_t;  //need these to store the previous sine and cosine
															//because we can't update them till they've been used in *two* formulas
					cos_omega_t=last_cos*cos_omega1 - last_sin*sin_omega1;  //current cosine(omega*t[i]), by recurrence
					sin_omega_t=last_sin*cos_omega1 + last_cos*sin_omega1;  //current sine(omega*t[i]), by recurrence
				} //else
				if (avgs==true) {
					if (k<=9) omega=twopi*min_freq*(double)(k+1)/10.0; else omega=twopi*min_freq*(double)(k-8);
					//this is the average of sin(omega*t) from t-dt to t, using trig identities to reduce roundoff error
					sin_term = (sin_omega_t*sin_omega_dt[k] - 2.0*cos_omega_t*sin_omega_halfdt[k]*sin_omega_halfdt[k])/(omega*dt) ;
					//this is the average of cos(omega*t) from t-dt to t, using trig identities to reduce roundoff error
					cos_term = (cos_omega_t*sin_omega_dt[k] + 2.0*sin_omega_t*sin_omega_halfdt[k]*sin_omega_halfdt[k])/(omega*dt) ;
					for (j=0; j<n_columns; j++) if (x[i][j]!=null) x[i][j] = x[i][j] + amp[k]*( sin_term*cos_phase[k][j] + cos_term*sin_phase[k][j] );  //this is sin(omega*t[i]+phase[k]) 
				}	
				else for (j=0; j<n_columns; j++) if (x[i][j]!=null) x[i][j] = x[i][j] + amp[k]*( sin_omega_t*cos_phase[k][j] + cos_omega_t*sin_phase[k][j] );  //this is amp*sin(omega*t[i]+phase[k]) 

		}//loop frequencies
			
 if (i % 10==9) fprintf(stderr, "\r%i elapsed time %g seconds " ,i+1,  difftime( time(NULL) , start_time )); //here's where we keep track of progress		
			
			fprintf (outfile, "%.14g", t[i] );
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%.14g", x[i][j]);
			fprintf (outfile, "\n");
		} // loop times

		fclose(outfile);
	} //end else

	if (avgs==true) {
		free (sin_omega_dt);
		free (sin_omega_halfdt);
	}

	
}//print results
	




int main( void ) {
	start_time=time(NULL);
	read_in_data( );
	
	print_results( );
	recover_storage( );

	
	return 0;
}

