

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
					
					
#define FTYPE double				//modify to change level of precision
#define DTYPE double				//modify to change level of precision
#define ITYPE long					//modify if needed for very long arrays
#define MAX_LINE_LENGTH 5000		//length, in chars, of max line in input file
#define sqr( a )((a) * (a))			//amazing, the functions that C leaves out!
#define true 1
#define false 0

#define white_noise 111
#define random_walk 222
#define colored_noise 333




/*
*******************************************************************************************************
*******************************************************************************************************
READ ME FIRST:

This source code was written by J.W. Kirchner, who holds copyright to it.  It is being made available
to the public under the Creative Commons Attribution-NonCommercial-ShareAlike License (CC BY-NC-SA).
That means you are free:
to Share  to copy, distribute and transmit the work
to Remix  to adapt the work
Under the following conditions:

Attribution  You must attribute the work in the manner specified by the author or licensor (but not in any way that suggests that they endorse you or your use of the work).
Noncommercial  You may not use this work for commercial purposes.
Share Alike  If you alter, transform, or build upon this work, you may distribute the resulting work only under the same or similar license to this one.
Any of the above conditions can be waived if you get permission from the copyright holder.

For details see:
http://creativecommons.org/licenses/by-nc-sa/3.0/
http://creativecommons.org/licenses/by-nc-sa/3.0/legalcode
*******************************************************************************************************
*******************************************************************************************************
*/






/*
*******************************************************************************************************************
*******************************************************************************************************************
The following 6 blocks define the option instructions for spectral analysis of the 7-hour and weekly Plynlimon data.
Un-comment each of these blocks to process each of the source files (Upper Hafren, Lower Hafren, and Rainfall, each
with both 7-hour and weekly samples).

To run Date-Compensated Discrete Fourier Transform (DCDFT), set calculation_mode=1 and c=0.0
To run Lomb-Scargle Fourier Transform, set calculation_mode=5 and c=0.0
*******************************************************************************************************************
*******************************************************************************************************************
*/





//	this is for analysis of the Upper Hafren 7-hour stream chemistry samples
	const char infilename[100]="7hour UHF for spectra.txt";
	const char filespec[50]="7hr_UHF";


#define n_columns 47   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to read
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1250		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=625.0;			//just below Nyquist

	const FTYPE min_freq=0.5;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=1250.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=false;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=9; //mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.001;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets




/*
//	this is for analysis of the Lower Hafren 7-hour stream chemistry samples
	const char infilename[100]="7hour LHF for spectra.txt";
	const char filespec[50]="7hr_LHF";


#define n_columns 47   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to readc
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1250		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=625.0;			//just below Nyquist

	const FTYPE min_freq=0.5;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=1250.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=false;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=9; //mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.001;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets
*/





/*
//	this is for analysis of the Upper Hafren weekly stream chemistry samples
	const char infilename[100]="weekly UHFw for spectra.txt";
	const char filespec[50]="wkly_UHFw";


#define n_columns 47   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to read
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 520		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=26.0;			//just below Nyquist

	const FTYPE min_freq=0.05;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=52.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=false;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=9; //mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.001;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets

*/




/*
//	this is for analysis of the Lower Hafren weekly stream chemistry samples
	const char infilename[100]="weekly LHFw for spectra.txt";
	const char filespec[50]="wkly_LHFw";


#define n_columns 47   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to read
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 520		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=26.0;			//just below Nyquist

	const FTYPE min_freq=0.05;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=52.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=false;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=9; //mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.001;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets
*/



/*
//	this is for analysis of the 7-hourly rain chemistry samples
	const char infilename[100]="7hour CR for spectra.txt";
	const char filespec[50]="7hr_CR";


#define n_columns 43   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to read
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1250		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=625.0;			//just below Nyquist

	const FTYPE min_freq=0.;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=1250.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=true;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=9; //mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.001;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets
*/



/*
//	this is for analysis of the weekly rain chemistry samples
	const char infilename[100]="weekly CRw for spectra.txt";
	const char filespec[50]="wkly_CRw";


#define n_columns 47   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to read
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 520		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=26.0;			//just below Nyquist

	const FTYPE min_freq=0.05;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=52.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=true;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=9; //mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.001;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets

*/









/*
*******************************************************************************************************************
*******************************************************************************************************************
The following blocks define the option instructions for analyzing benchmark data sets.  Those data sets are generated
using benchmark_generator.c.  Un-comment each block to perform each of the benchmark tests.  For each instruction 
block, a user would need to substitute different values of alpha into the specification of infilename and filespec,
for example, for random walk benchmark data sets:
	const char infilename[100]="7hour_UHF_benchmk_alpha=2.txt";
	const char filespec[50]="7hr_UHF_alpha=2";
or for white noise data sets:
	const char infilename[100]="7hour_UHF_benchmk_alpha=0.txt";
	const char filespec[50]="7hr_UHF_alpha=0";
or for 1/f noise data sets:
	const char infilename[100]="7hour_UHF_benchmk_alpha=1.txt";
	const char filespec[50]="7hr_UHF_alpha=1";
and so forth.  A user would also need to substitute LHF (Lower Hafren) or CR (Carreg Wen Rainfall) for UHF (Upper
Hafren) if tests of those sampling patterns were desired.  For CR, set avgs=true; for the others, set avgs=false.
These various permutations are not individually listed below, in the interests of brevity.
*******************************************************************************************************************
*******************************************************************************************************************
*/






/*
//	this is for benchmark tests of Foster's weighted wavelet power spectrum, 
//  calculated by phase rotation and convolution rather than matrix inversion (the method used in the paper)
//  with previously generated synthetic noise time series with 7-hourly sampling
	const char infilename[100]="7hour_UHF_benchmk_alpha=1_fc=1460.txt";
	const char filespec[50]="7hr_UHF_benchmk_alpha=1";


#define n_columns 47   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to read
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1250		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=625.0;			//just below Nyquist

	const FTYPE min_freq=0.5;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=1250.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=false;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=9;	//mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.001;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets
*/




/*
//	this is for benchmark tests of Foster's weighted wavelet power spectrum (the method used in the paper)
//  with previously generated synthetic noise time series with weekly sampling
	const char infilename[100]="wkly_UHFw_benchmk_alpha=1_fc=1460.txt";
	const char filespec[50]="wkly_UHFw_benchmk_alpha=1";


#define n_columns 47   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to read
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 520		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=26.0;			//just below Nyquist

	const FTYPE min_freq=0.5;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=52.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=false;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=9;	//mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.001;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets

*/






/*
//	this is for benchmark tests of the date-compensated discrete Fourier transform (DCDFT)
//  with previously generated synthetic noise time series with 7-hourly sampling
	const char infilename[100]="7hour_UHF_benchmk_alpha=2_fc=1460.txt";
	const char filespec[50]="7hr_UHF_DCDFT_alpha=2";


#define n_columns 47   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to read
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1250		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=625.0;			//just below Nyquist

	const FTYPE min_freq=0.5;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=1250.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=false;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=1;	//mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.0;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets

*/






/*
//	this is for benchmark tests of the date-compensated discrete Fourier transform (DCDFT)
//  with previously generated synthetic noise time series with weekly sampling
	const char infilename[100]="wkly_UHFw_benchmk_alpha=2_fc=1460.txt";
	const char filespec[50]="wkly_UHF_DCDFT_alpha=2";


#define n_columns 47   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to read
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 520		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=26.0;			//just below Nyquist

	const FTYPE min_freq=0.05;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=52.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=false;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=1;	//mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.0;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets

*/






/*
//	this is for benchmark tests of the Lomb-Scargle Fourier Transform (LSFT)
//  with previously generated synthetic noise time series with 7-hourly sampling
	const char infilename[100]="7hour_UHF_benchmk_alpha=2_fc=1460.txt";
	const char filespec[50]="7hr_UHF_LSFT_alpha=2";


#define n_columns 47   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to read
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1250		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=625.0;			//just below Nyquist

	const FTYPE min_freq=0.5;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=1250.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=false;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=5;	//mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.0;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets
*/






/*
//	this is for benchmark tests of the Lomb-Scargle Fourier Transform (LSFT)
//  with previously generated synthetic noise time series with weekly sampling
	const char infilename[100]="wkly_UHFw_benchmk_alpha=0_fc=1460.txt";
	const char filespec[50]="wkly_UHF_LSFT_alpha=0";


#define n_columns 47   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to read
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 520		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=26.0;			//just below Nyquist

	const FTYPE min_freq=0.05;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=52.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=false;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=5;	//mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.0;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets

*/
















/*
*******************************************************************************************************************
*******************************************************************************************************************
The following blocks define the option instructions for analyzing benchmark RAINFALL data sets.  Those data sets are generated
using benchmarks_generator.c.  Un-comment each block to perform each of the benchmark tests.  For each instruction 
block, a user would need to substitute different values of alpha into the specification of infilename and filespec,
for example, for random walk benchmark data sets:
	const char infilename[100]="7hour_CR_benchmk_alpha=2.txt";
	const char filespec[50]="7hr_CR_alpha=2";
then for white noise data sets:
	const char infilename[100]="7hour_CR_benchmk_alpha=0.txt";
	const char filespec[50]="7hr_CR_alpha=0";
then for 1/f noise data sets:
	const char infilename[100]="7hour_UHF_benchmk_alpha=1.txt";
	const char filespec[50]="7hr_CR_alpha=1";
and so forth.  A user would also need to substitute LHF (Lower Hafren) or CR (Carreg Wen Rainfall) for UHF (Upper
Hafren) if tests of those sampling patterns were desired.  For CR, set avgs=true; for the others, set avgs=false.
These various permutations are not individually listed below, in the interests of brevity.
*******************************************************************************************************************
*******************************************************************************************************************
*/




/*
//	this is for benchmark tests of Foster's weighted wavelet power spectrum, 
//  calculated by phase rotation and convolution rather than matrix inversion (the method used in the paper)
//  with previously generated synthetic RAINFALL noise time series with 7-hourly sampling
	const char infilename[100]="7hour_CR_benchmk_alpha=0.5_fc=1460.txt";
	const char filespec[50]="7hr_CR_benchmk_alpha=0.5";


#define n_columns 43   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to read
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1250		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=625.0;			//just below Nyquist

	const FTYPE min_freq=0.5;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=1250.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=true;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=9;	//mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.001;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets
*/










/*
//	this is for benchmark tests of Foster's weighted wavelet power spectrum, 
//  calculated by phase rotation and convolution rather than matrix inversion (the method used in the paper)
//  with previously generated synthetic RAINFALL time series with WEEKLY sampling
	const char infilename[100]="wkly_CRw_benchmk_alpha=0.5_fc=1460.txt";
	const char filespec[50]="wkly_CRw_benchmk_alpha=0.5";


#define n_columns 47   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to read
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 520		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=26.0;			//just below Nyquist

	const FTYPE min_freq=0.05;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=52.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=true;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=9;	//mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.001;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets
*/











/*
//	this is for benchmark tests of DCDFT on RAINFALL data, 
//  with previously generated synthetic RAINFALL noise time series with 7-hourly sampling
	const char infilename[100]="7hour_CR_benchmk_alpha=-0.5_fc=1460.txt";
	const char filespec[50]="7hr_CR_DCDFT_alpha=-0.5";


#define n_columns 43   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to read
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 1250		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=625.0;			//just below Nyquist

	const FTYPE min_freq=0.5;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=1250.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=true;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=1;	//mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.0;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets
*/









/*
//	this is for benchmark tests of DCDFT on WEEKLY RAINFALL data, 
//  with previously generated synthetic RAINFALL noise time series with WEEKLY sampling
	const char infilename[100]="wkly_CRw_benchmk_alpha=-0.5_fc=1460.txt";
	const char filespec[50]="wkly_CRw_DCDFT_alpha=-0.5";


#define n_columns 47   // 47 for UHF, UHFw, LHF, LHFw, or CRw, 43 for CR		//number of data columns to read
const int first_column=1;		//first data column (leftmost column--time--counts as zero)
								//note if there's a weight column and use_pointwt is false, then the weight column is treated as an unused data column)

#define n_freq 520		//for X-fold oversampling, set number of frequencies=X*max_freq*length_of_record (length in time, not # of points)

	const FTYPE max_freq=26.0;			//just below Nyquist

	const FTYPE min_freq=0.5;				//not used unless freq_even_in_logs or delimited_freq is true
	const int freq_even_in_logs=false; 
	const int delimited_freq=false;		// use this to generate frequencies evenly spaced between min_freq and max_freq -- for example to examine diurnal peaks in detail

	const double fs=52.0;  //sampling frequency, for alias filter
	const double fc=1460;  //corner frequency, for alias filter
	const double fcutoff=0.0;	//cutoff frequency, below which we ignore misfits in alias filter
	const int avgs=true;		//flag for alias filter: are samples intrinsically time-averaged (as in precip samples) or not?
	const int calculation_mode=1;	//mode 9 is Foster's weighted wavelet spectrum, calculated by phase rotation and convolution
									//mode 5 (with c=0.0) is the Lomb-Scargle Fourier Transform LSFT
									//mode 1 (with c=0.0) is the date-compensated discrete Fourier transform DCDFT 
	const FTYPE c=0.0;	//scaling factor for gaussian weights
						//effective sigma is lambda/(2pi*sqrt(2c)).  Note that c=0 yields conventional Fourier transform of whole data set
						//rather than wavelets

*/













/*
*******************************************************************************************************************
*******************************************************************************************************************
Below are options settings that do not change among all the simulations used in the paper and the supplementary
information.
*******************************************************************************************************************
*******************************************************************************************************************
*/



	const int test_phase_space=false;	//if true, it skips cases where points are too uneven to clearly define the phase space
	const int apply_wavelength_limits=true;	//discards wavelengths that are below Nyquist or longer than length of record
	const int do_alias_filtering=false;
	const int do_detrend=false;




	const int do_benchmark=false;  //do_benchmark=true activates the internal benchmark generating routines -- for the paper we use externally generated benchmark data sets instead
	const int benchmark_type=colored_noise;  //types are white_noise, random_walk, and colored_noise
	const FTYPE min_bench_freq=0.5;	//minimum frequency in simulated noise
	const int n_bench_freq=87600;	//number of freq's in simulated noise
	const FTYPE bench_fc=1460;		//corner frequency in simulated noise
	const FTYPE bench_alpha=1.0; 	//true alpha in simulated noise
	const int save_benchmarks=false;
	const int save_curves=false;










	const int use_window=true;			//window raw time series before analysis -- will only do this if it's a Fourier transform (c==0.0), will skip for wavelets	
	const int smooth_over_bins=false;		//smooths over a fixed interval of frequencies
	const int triangular_smooth=false;		//smooths power using triangular weighting rather than gaussian (only when smoothing over bins)
	const int half_width=1;				//half-width for power smoothing (in # of frequency bins) if smooth_over_logs is false.  68% of weight will be in +/- 1 half-width.
	const int smooth_over_logs=true;		//smooths over log-frequency
	const FTYPE log_smooth_width=0.05;	//power smoothing width in log units, used if smooth_over_logs is true.  68% of weight will be in 1 log_smooth_width surrounding central frequency (for gaussian smooth) or 100% within +/- log_smooth_width (for triangular smooth).
	//NOTE if smooth_over_bins and smooth_over_logs are both true, will smooth over bins first, then smooth (the result of that first smooth) again over log units.
	

	const int use_pointwt=false;			//allows weighting each point (e.g. by flow)
		const int do_LOWESS=false;
	const int deseasonalize=false;		
	const FTYPE LOWESS_smooth_width=2.0;	//if de-seasonalizing, this is in years -- so should be a small fraction.  If doing a lowess smooth of the time series, this should usually be a significant fraction of the record length
	const int jackknife_LOWESS_fits=true;
	const int do_WOSA=false;				//WOSA=Welch Overlapping Segment Algorithm; do_WOSA =true fixes weighting function width at specified value
	const FTYPE WOSA_sigma=0.5; 
	const int use_DFT_normalization=false;	//if true, we do DFT periodogram (square of amplitude) rather than power spectral density  SHOULD NORMALLY BE FALSE






			//calculation modes:
			// 1 -- Foster's method -- multiple regression on sin(omega*t) and cos(omega*t)
			// 5 -- Lomb-Scargle Fourier Transform (with weights corresponding to exponential term in wavelet if c>0)
			// 7 -- true least-squares (Foster) -- i.e., DCDFT (but done as phase rotation and convolution instead of matrix inversion)
			// 8 -- Correct rotation, then coefficients calculated as covar/s.d.
			// 9 -- The right way to do it -- rotation to decorrelate sine and cosine--not Lomb's rotation, which doesnt.... then convolution rather than regression
			//      This is close to Foster's weighted wavelet spectrum, but formulating it as a convolution rather than regression means that it is stable (rather
			//		than ill-behaved) for samplings under which the basis functions are degenerate.
			

	const int use_evenness_factor=false;		//evenness factor--using it gives better match to random walks in LSFT when sampling is uneven,
										//whereas leaving it out gives better match to white noise in LSFT when sampling is uneven
										//Note that on uneven sparse data sets, using this factor leads to jagginess as unevenness changes from wavelength to wavelength

	const int line_smooth=false;		//if true, smooths spectra by fitting line in log-log space and then estimating from predicted value along line
	const int do_logfile=false;

	const int do_LN_transform=false;
	const int do_rank_transform=false;
	const int normalize_amplitude=false;
	
	const FTYPE loc_spacing=0.5;				//spacing between wavelet locations, in units of sigma
	const double te=1.0e-3;  //threshold for weights
	
	const int header_len=50;		//max length, in characters, of header
//	const double pi=3.14159265358979324;
//	const double twopi=6.28318530717958648;

	const double pi=3.14159265358979323846264338327;
	const double twopi=6.28318530717958647692528676654;

	const FTYPE null=-999.0; //null value for errors etc.    Note: system value NULL (uppercase) is different!

	
// global typedefs
	typedef FTYPE *fptr;			//pointer to float-type
	typedef DTYPE *dptr;			//pointer to double-type
	typedef DTYPE smatrix[3][3];	//3x3 matrix for s-values
	typedef smatrix *sptr;			//pointer to s-matrix
	
	struct lstruct			//struct to pass info for locations
		{
		FTYPE	center;	//center value of time
		ITYPE	offset;	//index of first non-zero weight (on sorted x's)
		ITYPE	len;		//number of points with non-zero weight
		ITYPE	last_startpt;  //index of last starting point for search for nonzero weights
		DTYPE	sumwt;	//sum of point weights
		FTYPE	n_eff;	//effective number of points
		FTYPE 	omega;	//radial frequency
		FTYPE	ycos;	//cosine coefficient
		FTYPE	ysin;	//sine coefficient
		FTYPE	pwr;		//squared amplitude
		FTYPE	phase;	//phase in radians
		FTYPE	Vx;		//wtd variance of data
		FTYPE	Vy;		//wtd variance of model function
		FTYPE	sgnl;	//my estimate of signal variance, corrected for noise leakage
		FTYPE	noise;	//my estimate of total noise variance
		FTYPE 	model_noise;
		FTYPE	wwz;	//Foster's wwz statistic
		FTYPE 	s2n;		//signal-to-noise ratio
		} ;
	typedef struct lstruct *ls_ptr;	//pointer to location struct
	
	struct wstruct			//struct to pass info for a given wavelength
		{
		FTYPE omega;	//radial frequency
		ITYPE	j;		//data column index
		ITYPE	k;		//frequency index
		} ;
	typedef struct wstruct *ws_ptr;	//pointer to wavelength struct


// global variables 
	ITYPE n_data;	//number of data points
	FTYPE median_delta_t; 	//keeps median delta_t for estimating Nyquist frequency
	char *header[n_columns];
	fptr t, x;			//pointers to arrays of data point times and values
	fptr unity;			//pointer to array of 1's (unity vector)
	fptr cos_t, sin_t;	//pointer to cosine and sine arrays
	fptr cos_ttau, sin_ttau;	//pointer to cosine and sine of t-tau
	fptr wt;			//pointer to weight array
	fptr pointwt;		//pointer to individual point weights (flow weights, etc)
	ws_ptr top;		//pointer to array of wavelength structs
	
	FILE *outfile;
	FILE *infile;
	FILE *locfile;
	FILE *pointfile;
	time_t start_time;	//start time
	
	FTYPE freq[n_freq];	//circular frequency (cycles/time)
	FTYPE lambda[n_freq];	//wavelength

	FTYPE results[n_columns][n_freq];			//array to store results from real data
	FTYPE sine_coeff[n_columns][n_freq];			//array to store raw sine coefficients
	FTYPE cosine_coeff[n_columns][n_freq];		//array to store raw cosine coefficients
	FTYPE xxsignal[n_columns][n_freq];			//array to store results
	FTYPE xxwwz[n_columns][n_freq];				//array to store results
	FTYPE xxmodel_noise[n_columns][n_freq];				//array to store results
	FTYPE sm_pwr[n_columns][n_freq];				//array to store results
	FTYPE filtered_pwr[n_columns][n_freq];
	FTYPE save_model[n_columns][n_freq];
	FTYPE save_aliased[n_columns][n_freq];
	FTYPE save_alpha[n_columns];
	FTYPE save_alpha_se[n_columns];
	FTYPE save_regr_alpha[n_columns];
	FTYPE save_regr_alpha_se[n_columns];
	
	FTYPE smooth[n_freq];						//smoothing array for analyze_top (NOTE different arrays "smooth" are used elsewhere)

	double *benchmark[n_columns];  //this will hold each column of synthetic benchmark data, so that they can be printed all together
	double *original_t; //this will hold the vector of original times in the data table, so that the benchmark data can be printed all together
						//note that the vector "t" that is created from the input file will only include times for non-null data values, and so may
						//differe from one data column to the next
	int *rowi;			//vector that holds the row in the data table that an original value comes from (needed to create an "image" of the data table for saving the benchmark values
	int n_rows;			//number of rows in original data table




// ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	
// ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	
// ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	
// ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	
/*  bran.c    CCMATH mathematics library source code.
 *
 *  Copyright (C)  2000   Daniel A. Atkinson    All rights reserved.
 *  This code may be redistributed under the terms of the GNU library
 *  public license (LGPL). ( See the lgpl.license file for details.)
 * ------------------------------------------------------------------------
 */
static unsigned long bran_s,bran_h,bran_sbuf[256];
static unsigned long bran_a=1664525U, bran_c=4098479U;
static unsigned long bran_r=0xffff;
int bran(n)
int n;
{ register int i,j;
  i=(int)(bran_s>>24); bran_s=bran_sbuf[i];
  bran_h=bran_h*bran_a+bran_c; bran_sbuf[i]=bran_h;
  i=n*(bran_s>>16); j=n*(bran_s&bran_r); i+=(j>>16);
  return (i>>16);
}
void setbran(sa)
unsigned long sa;
{ int c;
  for(bran_h=sa, c=0; c<=256 ;++c){
    bran_h=bran_h*bran_a+c;
    if(c<256) bran_sbuf[c]=bran_h; else bran_s=bran_h;
   }
}



/*  shuffl.c    CCMATH mathematics library source code.
 *
 *  Copyright (C)  2000   Daniel A. Atkinson    All rights reserved.
 *  This code may be redistributed under the terms of the GNU library
 *  public license (LGPL). ( See the lgpl.license file for details.)
 * ------------------------------------------------------------------------
 jk's adaptation follows...
 */ 
void shuffle(fptr x)
{ int i,j; FTYPE dummy;
	for(j=n_data; j>0 ;){
		i=bran(j--);
		dummy=x[i]; x[i]=x[j]; x[j]=dummy;
	}
	return;
}



/*
hsort

     Perform a heap sort with comparison function comp on the array v[].

     void hsort(void *v,int n,int (*comp)())
       v = array of pointers to sort input
            (pointers index sorted array at exit)
       n = dimension of input array
       comp = pointer to comparison function (see numcmp for example)
*/
/*  hsort.c    CCMATH mathematics library source code.
 *
 *  Copyright (C)  2000   Daniel A. Atkinson    All rights reserved.
 *  This code may be redistributed under the terms of the GNU library
 *  public license (LGPL). ( See the lgpl.license file for details.)
 * ------------------------------------------------------------------------
 */
#define Swap(x,y) (tmp=(x),(x)=(y),(y)=tmp)
static rheap();
hsort(v,n,comp)
	void *v[]; int n,(*comp)();
	{ int k; char *tmp;
	for(k=n/2-1; k>=0 ;) rheap(v,k--,n,comp);
	for(--n; n>0 ;){ Swap(v[0],v[n]); rheap(v,0,n--,comp);}
return (0);
}

static rheap(v,k,n,comp)
	void *v[]; int k,n,(*comp)();
	{ int m=n-1,j; char *tmp;
	for(j=2*k+1; j<n ;k=j,j=2*k+1){
	if(j<m && (*comp)(v[j+1],v[j])>0) ++j;
	if((*comp)(v[j],v[k])>0) Swap(v[k],v[j]);
	else break;
	}
return (0);
}



/*  nrml.c    CCMATH mathematics library source code.
 *
 *  Copyright (C)  2000   Daniel A. Atkinson    All rights reserved.
 *  This code may be redistributed under the terms of the GNU library
 *  public license (LGPL). ( See the lgpl.license file for details.)
 * ------------------------------------------------------------------------
 */
#include <math.h>
static unsigned int nrml_a=1664525U,nrml_c=907633387U;
static unsigned int nrml_s,nrml_h,nrml_sbuf[256];
static int nrml_kf;
static double nrml_rc=4.6566128730773926e-10,nrml_e;
double nrml()        //returns random normally distributed values (mean of zero and variance of one)
{ double x,y,r; int i;
  if(nrml_kf){
    do{ i=(int)(nrml_s>>24); nrml_s=nrml_sbuf[i];
      nrml_h=nrml_a*nrml_h+nrml_c; nrml_sbuf[i]=nrml_h;
      x=nrml_rc*nrml_s-1.;
      i=(int)(nrml_s>>24); nrml_s=nrml_sbuf[i];
      nrml_h=nrml_a*nrml_h+nrml_c; nrml_sbuf[i]=nrml_h;
      y=nrml_rc*nrml_s-1.; r=x*x+y*y;
     } while(r>=1.);
    r=sqrt(-2.*log(r)/r);
    x*=r; nrml_e=y*r;
   }
  else x=nrml_e;
  nrml_kf^=1; return x;
}
void setnrml(unsigned int sa)        //initializes random number generator nrml
{ int j;
  for(nrml_h=sa,j=0; j<=256 ;++j){
    nrml_h=nrml_a*nrml_h+nrml_c;
    if(j<256) nrml_sbuf[j]=nrml_h; else nrml_s=nrml_h;
   }
  nrml_kf=1;
}


// ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	
// ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	
// ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	
// ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	 ccmath stuff     	
	
	
	
	
int fcomp(x,y)
FTYPE *x,*y;
{ if(*x > *y) return 1;
  if(*x < *y) return -1;
  return(0);
}



#include <ctype.h>
int read_string(FILE *infile, char *string, int len) {	
	int i, x;	//reads an input file for a string.  reads until the first whitespace character
	i=0;
	do { 
		x=fgetc(infile);
		if ( (! isspace(x)) && (i<len-1)) {string[i]=x; i=i+1;}
	} while (! isspace(x));
	string[i]=0;	//null termination
return (0);
}//read_string


int read_string_from_string(char *source, char *string, int len) {	
	int i, x;	//reads an input string for a string.  Reads until the first whitespace character or end of string
	i=0;
	do { 
		x=source[i];
		if ( (! isspace(x)) && (i<len-1)) {string[i]=x; i=i+1;}
	} while (! isspace(x));
	string[i]=0;	//null termination
return (0);
}//read_string






double misfit(double alpha, double fs, double fc, double fcutoff, double *freq, double *log_pwr, double *model, double *aliased, int avgs, int n)	
	/* 	Generates modeled power spectrum, with aliases, for a specified scaling exponent
		alpha, and returns its root-mean-square misfit score in comparison to
		the supplied log-transformed power spectrum log_pwr.
		
		Inputs to this function (none of which are changed) are:
			alpha -- power-law scaling exponent
			fs -- sampling frequency
			fc -- corner frequency
			freq -- frequency array
			log_pwr -- log(power) array
			avgs -- flag for whether spectrum is calculated from time-averaged measurements (avgs<>0) or instantaneous measurements (avgs=0)
			n -- length of power spectrum array
		
		Outputs are:
			model -- modeled spectrum
			aliased -- modeled spectrum plus aliases
	*/

{	const double pi=3.14159265358979324; 
	int i; //index for frequencies
 	int k; //index for aliases
 	int counter;
 	double alias;  //a single alias power component
 	double prefactor;	//leading constant for model power spectrum
  	double tally;  //accumulates misfit score
	double beta;
	double cc, dz, zo;
 
  	//here is where the model spectrum is defined (equations 24 and 26)
 	#define spectrum( f )( (avgs==0) ? (pow(f , -alpha)/(1.0 + ((f)*(f))/(fc*fc)))  :   (pow(f , -alpha)/(1.0 + (f)*(f)/(fc*fc))*(sin(pi*(f)/fs)/(pi*(f)/fs))*(sin(pi*(f)/fs)/(pi*(f)/fs)) ) ) ;
 

  	for (i=0; i<n; i++) { //loop through frequencies, build up model and aliased model
  		model[i]=pow(freq[i] , -alpha)/(1.0 + (freq[i]*freq[i])/(fc*fc));     
  		aliased[i]=spectrum(freq[i]);	//Here's a subtle point that's missing in my 2005 paper.  The model spectrum (the "real world" spectrum that we assume)
									 	//does not depend on whether sampling is time-averaged or not.  However the measured spectrum (in our terms the "aliased" spectrum)
 										//will be affected by the damping of high frequencies that comes from time-averaging.  Therefore the model spectrum should be the 
										//simple form of #define spectrum( f ) above, whether the sampling is time-averaged or not.  Only the aliased spectrum should vary
										//depending on time-averaging. The updated (30 Jan 2012) formulation here implements this.  This change will not affect the best-fit
										//value of alpha (which depends only on the aliased spectrum) but it will affect the shape of the filtered spectrum.
  		for (k=1; k<=10; k++){   //loop through aliases, building up spectrum of model+aliases: equations 21 and 28 -- first we do 10 exact iterations
  			alias=spectrum(k*fs-freq[i]); aliased[i]=aliased[i]+alias;  //add alias
  			alias=spectrum(k*fs+freq[i]); aliased[i]=aliased[i]+alias;	 //add alias		
  		} 

		//After 10 exact iterations, integrate from k=11 to k=infinity
		//alias=integral(S(kfs-f) dk) for k=11 to k=infinity.
		//This integration is done by a change of variables, such that z=(k*fs-f)^(-alpha-1) 
		//and dk=-1/(alpha+1)fs z^(-(alpha+2)/(alpha+1)) dz.
		//so the limits of integration are zo=(11*fs-f)^(-alpha-1) and z=0
		//and the integrand is almost constant across the entire integral.
		//For convenience we define beta=alpha+1.

		//For time-averaged sampling, we take the mean of sin^2 over the (rapid) cycles in sinc, so that
		//sinc(k*fs-f) is approximated by 1/(2*pi*(k*fs-f)/fs).  Then the entire effect of time-averaged 
		//sampling is a redefinition of beta and of the leading constant for the integration.

		if (avgs!=0) {
			beta = alpha + 3.0;
			cc = 1.0/(2.0*pi*pi*beta/fs);
		} else {
			beta = alpha+1.0;
			cc = 1.0/(beta*fs);
		}

		zo = pow(11.0*fs-freq[i] , -beta);
		dz = zo/50.0;
		for (k=1; k<=50; k++) aliased[i]=aliased[i] + cc / (pow((double)k*dz,(2.0/beta)) +1.0/(fc*fc)) *dz;

		//now do the same thing over again for the (k*fs+f) aliases -- note the integral is the same, just
		//the limits are (slightly) different

		zo = pow(11.0*fs+freq[i] , -beta);
		dz = zo/50.0;
		for (k=1; k<=50; k++) aliased[i]=aliased[i] + cc / (pow((double)k*dz,(2.0/beta)) +1.0/(fc*fc)) *dz;

  	} //loop through frequencies
    


  	//calculate prefactor that minimizes root-mean-square misfit (this will be exponential of mean of logged deviations)
	prefactor=0.0;  counter=0;
  	for (i=0; i<n; i++) if ((freq[i]>fcutoff)&&(log_pwr[i]!=null)) {prefactor=prefactor + log_pwr[i]-log(aliased[i]); counter=counter+1;}
 if (counter==0) {fprintf( stderr, "Abnormal termination in alias_filter: all log_pwr values are null!\n"); exit(0);}
 	prefactor=exp(prefactor/(double) counter);
  	
  	tally=0.0;  counter=0;  //now tally misfits
  	for (i=0; i<n; i++) {
  		model[i]=prefactor*model[i];
  		aliased[i]=prefactor*aliased[i];
  		if ((freq[i]>fcutoff)&&(log_pwr[i]!=null)) {tally=tally + (log_pwr[i]-log(aliased[i]))*(log_pwr[i]-log(aliased[i])) ;  counter=counter+1; } //sum of squared deviations of logs
  	}
//fprintf (stderr, "%g\t%i\n", tally, counter);
	return(tally);  //return sum of squared error (sum of squared of log deviations between measured spectrum and aliased model)
					//misfit must be SSE in order for the MLE estimation of standard errors to work correctly
} //misfit












int alias_filter(double *freq, double *pwr, int n, double fs, double fc, double fcutoff, int avgs, double *filtered_pwr, double *model, double *aliased, double *alpha, double *alpha_se, double *regr_slope, double *regr_slope_se)
/* 
Performs alias filtering of power spectrum, using power-law model
Receives the following:
	freq -- array of length n, of frequencies in power spectrum
	pwr -- array of length n, of spectral power corresponding to frequencies "freq"
	n -- length of freq and pwr arrays
	fs -- sampling frequency
	fcorner -- corner frequency for 1/f^2 steepening of power spectrum
	avgs -- flag for whether spectrum is derived from instantaneous point measurements (avgs=0)
			or from measurements averaged over each sampling interval (avgs<>1)
Returns the following:
	filtered_pwr -- array of length n, of alias-filtered spectral power
	model -- array of length n, with modeled spectral power
	aliased -- array of length n, with modeled spectral power, plus aliases
	filtered_pwr -- array of length n, of alias-filtered spectral power
	alpha -- exponent of power-law model
	alpha_se -- estimate of standard error of alpha
Notations "equation ##" refer to numbered equations in Kirchner, 2005.
*/

{	const double lower_limit=-0.9;  //lower limit for alpha values
	const double upper_limit=10.0; //upper limit for alpha values
	const int iter_limit=1000;  //limit to number of iterations
	double tol=1.0e-5;		//termination tolerance: should be larger than square root of fractional machine precision
	double *log_pwr;		//array holding logarithms of powers
	double *resid;			//array for holding residuals
	int i, counter, iter;  //index
	double sumx, sumy, sumxy, sumxx, sumyy;  //for initial estimate of alpha from log-log regression, and serial correlation
	double a, b, c, ya, yb, yc, newpt, ynew;  //for inverse parabolic interpolation
	double n_eff, correl, fisher_information;	//for MLE estimates of s.e. of alpha
	double x, y, SSy, SSE, SSx, meanx, meany;

	//allocate new arrays
	log_pwr=malloc(n*sizeof(double));
		if (log_pwr==NULL) {fprintf( stderr, "Abnormal termination: out of memory allocating log_pwr in alias_filter\n"); exit(0);}

	resid=malloc(n*sizeof(double));
		if (resid==NULL) {fprintf( stderr, "Abnormal termination: out of memory allocating resid in alias_filter\n"); exit(0);}

	//initialize
	for (i=0; i<n; i++) {
		if (pwr[i]==null) log_pwr[i]=null;
		else {if (pwr[i]>0.0) log_pwr[i]=log(pwr[i]);  //log-transform spectral power
			else  {fprintf( stderr, "Abnormal termination in alias_filter: spectrum contains zero or negative power values\n"); exit(0);}
		}
		if ((freq[i]<0) || (freq[i]>0.5*fs))  {fprintf( stderr, "Abnormal termination in alias_filter: supplied frequencies are negative or above Nyquist freq\n"); exit(0);}
	}
	
	//We will find the best-fit value of alpha by inverse parabolic interpolation.  At each step, we keep a triplet of trial 
	//values a, b, and c, such that a<b<c, and such that their misfit values ya, yb, and yc obey yb<ya and yb<yc.
	//That is, we will always keep b between a and c, and always keep yb less than ya and yc. 
	//This algorithm is not very robust, but it is simple.
	
	//obtain a first estimate of alpha from log-log regression
	sumx=0.0; sumy=0.0; sumxy=0.0; sumxx=0.0; counter=0; //initialize tallies
	for (i=0; i<n; i++) if (log_pwr[i]!=null) { a=log(freq[i]); sumx=sumx+a; sumxx=sumxx+a*a; sumxy=sumxy+a*log_pwr[i]; sumy=sumy+log_pwr[i]; counter=counter+1; }  //sums and cross-products
	b=-(sumxy-sumx*sumy/(double) counter)/(sumxx-sumx*sumx/(double) counter); //linear regression estimate for alpha -- a starting point
														  //negative sign arises because alpha is the negative of the log-log slope
	yb=misfit(b, fs, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n); //first estimate of misfit
	

//fprintf (stderr, "got first misfit value\n");

	//now find two alpha values, on either side of our central estimate, that have higher misfit values
	a=b; do {a=a-0.1; ya=misfit(a, fs, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n); if (ya<yb) {b=a; yb=ya;} 
//fprintf (stderr, "%g\t%g\n", a, ya);	
		if (a<lower_limit) {
			*alpha=null; *alpha_se=1.0; 
				for (i=0; i<n; i++) filtered_pwr[i]=null;
			return(1);
		}
	} while (ya<=yb);  //repeat this until we find a point with higher misfit
	c=b+0.05; do {c=c+0.1; yc=misfit(c, fs, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n); if (yc<yb) {b=c; yb=yc;} 
//fprintf (stderr, "%g\t%g\n", b, yb);	
		if (c>upper_limit) {
			*alpha=null; *alpha_se=2.0; 
				for (i=0; i<n; i++) filtered_pwr[i]=null;
			return(2);
		}
	
	} while (yc<=yb);  //repeat this until we find a point with higher misfit
//fprintf (stderr, "\n");




//	a=0.0; ya=misfit(a, fs, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n);
//		if (ya<yb) fprintf( stderr, "Abnormal termination in alias_filter: ya<yb\n");
//	c=5.0; yc=misfit(c, fs, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n);
//		if (yc<yb) fprintf( stderr, "Abnormal termination in alias_filter: yc<yb\n");


	iter = 0;
	do {  //inverse parabolic interpolation to search for best-fit alpha value
		iter = iter+1;
		newpt=b-0.5*((b-a)*(b-a)*(yb-yc) - (b-c)*(b-c)*(yb-ya))/((b-a)*(yb-yc) - (b-c)*(yb-ya));  //inverse parabolic interpolation
		ynew=misfit(newpt, fs, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n);  //function value at that new point
		
		if (ynew<yb) { //if the new point is an improvement over the center point, replace the center point with it
			if (newpt<b) {c=b; yc=yb;} //shuffle rightmost point inward if new minimum is left of b
				else {if (newpt>b) {a=b; ya=yb;} //shuffle leftmost point inward if new minimum is right of b
					else {fprintf( stderr, "Abnormal termination in alias_filter: parabolic interpolation failed\n");
						*alpha=null; *alpha_se=3.0; 
						for (i=0; i<n; i++) filtered_pwr[i]=null;
						return(3);
					} } //if new point lands right on b, interpolation fails
			b=newpt; yb=ynew; }
		else {  //if the new point is higher than the center point, replace one of the outer points with it
			if (newpt<b) {a=newpt; ya=ynew;} //replace leftmost point if new minimum is left of b
				else {if (newpt>b) {c=newpt; yc=ynew;} //replace rightmost point if new minimum is right of b
					else {fprintf( stderr, "Abnormal termination in alias_filter: parabolic interpolation failed\n");
						*alpha=null; *alpha_se=4.0; 
						for (i=0; i<n; i++) filtered_pwr[i]=null;
						return(4);
				} } //if new point lands right on b, interpolation fails
			}

		//if the middle point is much closer to one boundary point than the other, do a golden section step 
		if ( (c-b)>5.0*(b-a) ) {  //golden section step in right-hand interval
			newpt=b+0.38197*(c-b);
			ynew=misfit(newpt, fs, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n);  //function value at that new point
			if (ynew<yb) { a=b; ya=yb; b=newpt; yb=ynew; }  //if the new point is an improvement over the center point
				else  {c=newpt; yc=ynew;} //otherwise just move the right boundary inward
		}  
		
		if ( (b-a)>5.0*(c-b) ) {  //golden section step in left-hand interval
			newpt=b-0.38197*(b-a);
			ynew=misfit(newpt, fs, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n);  //function value at that new point
			if (ynew<yb) { c=b; yc=yb; b=newpt; yb=ynew; }  //if the new point is an improvement over the center point
				else  {a=newpt; ya=ynew;} //otherwise just move the left boundary inward
		}  
				
		if ((yb>ya) || (yb>yc)) {fprintf( stderr, "Abnormal termination in alias_filter: parabolic interpolation failed\n");
								*alpha=null; *alpha_se=5.0; 
						for (i=0; i<n; i++) filtered_pwr[i]=null;
						return(5);
		}  // this should never happen, but just in case... 

	} while ( (fabs((c-a)/(c+a))>tol)&&(iter<=iter_limit));  //termination condition: endpoints are close enough that further midpoint improvement is pointless

	if (iter>iter_limit) {
		*alpha=null; *alpha_se=6.0; 
		for (i=0; i<n; i++) filtered_pwr[i]=null;
		return(6);
	}
	
	*alpha=b;
	
	for (i=0; i<n; i++) if (pwr[i]!=null) filtered_pwr[i]=pwr[i]*model[i]/aliased[i];  //here is the alias filter (equation 23)
						else filtered_pwr[i]=null;

	


	//now we estimate the standard error of alpha using maximum likelihood methods

	//first we look at the correlation between the residuals and their lags, to estimate the loss of degrees of freedom due to the smoothing of the spectrum
	
	for (i=0; i<n; i++) if (log_pwr[i]!=null) resid[i]=log_pwr[i]-log(aliased[i]); else resid[i]=null;  //calculate residuals in log space

	//now calculate the correlation between these residuals and their lags
	sumx=0.0; sumy=0.0; sumxx=0.0; sumyy=0.0; sumxy=0.0; counter=0;
	for (i=1; i<n; i++) if ((resid[i]!=null) && (resid[i-1]!=null)) {
			sumx=sumx+resid[i];
			sumy=sumy+resid[i-1];
			sumxx=sumxx+resid[i]*resid[i];
			sumyy=sumyy+resid[i-1]*resid[i-1];
			sumxy=sumxy+resid[i]*resid[i-1];
			counter=counter+1;
		}

	correl=(sumxy-sumx*sumy/(double)counter)/(sqrt(sumxx-sumx*sumx/(double)counter)*sqrt(sumyy-sumy*sumy/(double)counter));

	n_eff=(double)counter*(1.0-correl)/(1.0+correl);  //number of effective independent power estimates

	//remember, b and yb are still the best-fit alpha and the misfit value at that alpha
	newpt=b*(1.0+100.0*tol);  //now perturb the value of alpha
	ynew=misfit(newpt, fs, fc, fcutoff, freq, log_pwr, model, aliased, avgs, n);  //misfit at that new point

	fisher_information = n_eff * ((ynew/yb)-1.0) / ((newpt-b)*(newpt-b));  //because we're assuming Gaussian errors 
				//with sigma estimated from best-fit SSE/n_eff (effectively correcting for loss of degrees of freedom), MLE gives just this simple expression for a 1-dimensional fit!
				//This formula estimates - d2y/dx2 where y is log-likelihood = -(1/(2*sigma^2))*SSE,
				//where sigma^2 is estimated as SSE at the best-fit point, divided by the effective degrees of freedom (approximated by n_eff).  

	*alpha_se = 1.0/sqrt(fisher_information);  //normally this would involve matrix inversion but it's just a one-dimensional fit!





	//now we estimate alpha and its standard error by linear regression of log(filtered power) against log(freq).

	sumx=0.0; sumy=0.0; sumxx=0.0; sumyy=0.0; sumxy=0.0; counter=0;
	for (i=0; i<n; i++) if (filtered_pwr[i]!=null) {
		x=log(freq[i]);
		y=log(filtered_pwr[i]);
		sumx=sumx+x;
		sumy=sumy+y;
		sumxx=sumxx+x*x;
		sumyy=sumyy+y*y;
		sumxy=sumxy+x*y;
		counter=counter+1;
	}

	*regr_slope = (sumxy-sumx*sumy/(double)counter)/(sumxx-sumx*sumx/(double)counter);
	correl=(sumxy-sumx*sumy/(double)counter)/(sqrt(sumxx-sumx*sumx/(double)counter)*sqrt(sumyy-sumy*sumy/(double)counter));
	SSy = sumyy-sumy*sumy/(double)counter;
	SSE = SSy*(1.0-correl*correl);
	SSx = sumxx-sumx*sumx/(double)counter;
	meanx=sumx/(double)counter;
	meany=sumy/(double)counter;

	//now we look at the correlation between the residuals and their lags, to estimate the loss of degrees of freedom due to the smoothing of the spectrum
	for (i=0; i<n; i++) if (filtered_pwr[i]!=null) resid[i]=(log(filtered_pwr[i])- meany) - *regr_slope * (log(freq[i])-meanx); else resid[i]=null;  //calculate residuals in log space

	//now calculate the correlation between these residuals and their lags
	sumx=0.0; sumy=0.0; sumxx=0.0; sumyy=0.0; sumxy=0.0; counter=0;
	for (i=1; i<n; i++) if ((resid[i]!=null) && (resid[i-1]!=null)) {
		sumx=sumx+resid[i];
		sumy=sumy+resid[i-1];
		sumxx=sumxx+resid[i]*resid[i];
		sumyy=sumyy+resid[i-1]*resid[i-1];
		sumxy=sumxy+resid[i]*resid[i-1];
		counter=counter+1;
	}

	correl=(sumxy-sumx*sumy/(double)counter)/(sqrt(sumxx-sumx*sumx/(double)counter)*sqrt(sumyy-sumy*sumy/(double)counter));
	n_eff=(double)counter*(1.0-correl)/(1.0+correl);  //number of effective independent power estimates

	*regr_slope_se = sqrt((SSE/SSx)/n_eff);



	free(log_pwr);  	//free array before exiting
	free(resid);
	
} //alias_filter








int read_in_data(int read_column)		//opens infile, allocates global arrays, and reads in data
{	ITYPE i, j, row;
	double tt, xx, ww; //use these to transfer to possible float-sized t[i], x[i], and pointwt[i]
	char junk[MAX_LINE_LENGTH+1]; 
	FTYPE **v;					//pointer to pointer array for sort function
	fptr delta_t;
	int (*comp_function) (fptr, fptr);	//pointer to sort comparison function
	comp_function=fcomp;			//set pointer to comparison function
	
	
	
	 //verify that output file can be opened
//	if( (infile=fopen(outfilename,"r")) == NULL ) {fprintf(stderr, "output file %s will not be accessible!\n",outfilename); exit(-1); }
//	fclose(infile);
	
	 //open input file and verify it exists
	if( (infile=fopen(infilename,"r")) == NULL ) {fprintf(stderr, "input file %s could not be opened!\n",infilename); exit(-1); }

	// run through file and count legal points
	n_data=0;
	fgets(junk, MAX_LINE_LENGTH, infile);	//discard first line
	do {	tt=null; xx=null; ww=1.0;
		fscanf(infile, "%lg", &tt);		//get the time value
		if (use_pointwt==true) 	fscanf(infile, "%lg", &ww);		//get the point weight--note pointwt will be 1 unless use_pointwt is read
		for (j=1; j<read_column; j++)  fscanf(infile, "%lg", &xx);  //throw out columns before the one we need
		fgets(junk, MAX_LINE_LENGTH, infile);	//get the rest of the line
		sscanf(junk, "%lg ", &xx);	//read the one we need
		if ((tt != null) && (xx != null) && (ww != null))  n_data=n_data+1;  //increment n_data if this is a valid data point
	} while (feof(infile)==0);

	// now allocate memory for t, x, and pointwt
	if ( ! (t = calloc(n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating t!\n"); exit(0);}
	if ( ! (x = calloc(n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating x!\n"); exit(0);}
	if ( ! (pointwt = calloc(n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating pointwt!\n"); exit(0);}

	// now allocate memory for sin, cos, and weight
	if ( ! (sin_t = calloc(n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating sin_t!\n"); exit(0);}
	if ( ! (cos_t = calloc(n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating cos_t!\n"); exit(0);}
	if ( ! (wt = calloc(n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating wt!\n"); exit(0);}

	if (save_benchmarks==true) 
		if ( ! (rowi = calloc(n_data , sizeof(int)) )) {fprintf(stderr, "out of memory allocating rowi!\n"); exit(0);}  //this vector will hold the numbers of the rows in the input file that the i'th data values came from

	// reset file, get data values, and store them in t and x
	fclose(infile);
	infile=fopen(infilename,"r");
	read_string(infile, junk, header_len);	//discard first column heading
	if (use_pointwt==true) read_string(infile, junk, header_len);	//discard pointwt column heading
	for (j=1; j<read_column; j++)  read_string(infile, junk, header_len);	//throw out headers before the one we need
	fgets(junk, MAX_LINE_LENGTH, infile); //get the rest of the line
	read_string_from_string(junk, header[read_column-first_column], header_len); //read that string until the first whitespace character to find the header string we need

fprintf(stderr, "\nheader %i: %s\n", read_column-first_column+1, header[read_column-first_column]); 

	
//	read_string(infile, header[read_column-first_column], header_len);	//read the header we want
//	if (read_column<first_column+n_actual_columns-1) fgets(junk, MAX_LINE_LENGTH, infile);	//discard the rest of the line

	if (save_benchmarks==true) {
		i=0; row=0;
		do {	tt=null; xx=null; ww=1.0;
			fscanf(infile, "%lg", &tt);		//get the time value
			if (use_pointwt==true) 	fscanf(infile, "%lg", &ww);		//get the point weight--note pointwt will be 1 unless use_pointwt is read
			for (j=1; j<read_column; j++)  fscanf(infile, "%lg", &xx);  //throw out columns before the one we need
			fgets(junk, MAX_LINE_LENGTH, infile);	//get the rest of the line
			sscanf(junk, "%lg ", &xx) ;	//read the one we need
			if ((tt != null) && (xx != null) && (ww != null)) {
				t[i]=tt; x[i]=xx; pointwt[i]=ww; rowi[i]=row; i=i+1;
			} //if
			row=row+1; //update the row counter whether or not that was a non-null point
		} while (feof(infile)==0);	

	} else {  //if we're not saving benchmarks and therefore don't need the row counter
		i=0;
		do {	tt=null; xx=null; ww=1.0;
			fscanf(infile, "%lg", &tt);		//get the time value
			if (use_pointwt==true) 	fscanf(infile, "%lg", &ww);		//get the point weight--note pointwt will be 1 unless use_pointwt is read
			for (j=1; j<read_column; j++)  fscanf(infile, "%lg", &xx);  //throw out columns before the one we need
			fgets(junk, MAX_LINE_LENGTH, infile);	//get the rest of the line
			sscanf(junk, "%lg ", &xx) ;	//read the one we need
			if ((tt != null) && (xx != null) && (ww != null)) {
				t[i]=tt; x[i]=xx; pointwt[i]=ww; i=i+1;
			} //if
		} while (feof(infile)==0);
	}

	//check for ordering of data
	for(i=1; i<n_data; i++) if (t[i]<t[i-1]) {  //if there's a problem, dump to error file and terminate
		outfile=fopen("error_file.txt", "w+");
		for (j=0; j<n_data; j++) fprintf(outfile, "%g\t%g\n", t[j], x[j]);
		fclose(outfile);
		fprintf(stderr, "times must be sorted in ascending order!\n");
		exit(0);
	}
	
	
	//determine median delta_t
	delta_t=calloc(n_data-1,sizeof(FTYPE));
	v=(FTYPE **)calloc(n_data-1,sizeof(FTYPE *));
	for(i=0; i<n_data-1; i++) {
		delta_t[i]=t[i+1]-t[i];
		v[i]=&delta_t[i];  //pointers for sorting
	}
	hsort( (void *) v, n_data-1, comp_function);
	median_delta_t=0.5*(*v[(n_data-1)/2] + *v[2+(n_data-1)-(n_data-1)/2]);	
	free(v);
	free(delta_t);
	
	
	//allocate and fill global unity array
	if ( ! (unity = calloc(n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating unity!\n"); exit(0);}
	for (i=0; i<n_data; i++) unity[i]=1.0;

//printf("%s\n", header[read_column-first_column]);	
// for(i=n_data-10; i<n_data; i++) printf("%g\t%g\n", t[i],x[i]);
// printf("%li\t%li\n", i, n_data);
// printf("%li \t %li \t %li \n", t, x, NULL);
//exit(0);

	fclose(infile);
	return(0);
}  //end of read_in_data




int LOWESS( 	fptr x, 		//note, here time is x and data values are y (rather than t and x, respectively)
			fptr y, 
			fptr smooth, 
			ITYPE n_data)
{	const int iterlimit=3;
	fptr distwt;		//distance weights -- note: to do LOWESS on multiple sets of same length, define this as an array (just once for a given set of t[i]
	fptr residwt;		//residual weights
	fptr resid;
	ITYPE pt, i, iter, firstpt, lastpt, npoints, m1, m2;
	FTYPE dist, wt, mad6;
	DTYPE sumx, sumy, sumxx, sumyy, sumxy, sumwt, meanx, meany, slope;

	FTYPE **v;					//pointer to pointer array for sort function
	int (*comp_function) (fptr, fptr);	//pointer to sort comparison function
	comp_function=fcomp;			//set pointer to comparison function
	v=(FTYPE **)calloc(n_data,sizeof(FTYPE *));
	
	
	if ( ! (distwt = calloc(n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating distwt!\n"); exit(0);}
	if ( ! (residwt = calloc(n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating residwt!\n"); exit(0);}
	if ( ! (resid = calloc(n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating resid!\n"); exit(0);}
	
	firstpt=0;
	for (pt=0; pt<n_data; pt++) {	//loop through central points

		i=firstpt-1;	//find new firstpt
		do {
			i=i+1; 
			dist=fabs(x[i]-x[pt])/(LOWESS_smooth_width/2.0); 
//fprintf(stderr, "looking for firstpt %i\t%g\t%i\t%g\t%g\n", pt, x[pt], i, x[i], dist);
		} while ( (dist>1.0) && (i<n_data-2) ) ;
		firstpt=i;
		
		i=firstpt-1;	//find new lastpt, and define distwt's--note this REQUIRES that time values be sorted into either ascending or descending order
		do {
			i=i+1; 
			dist=fabs(x[i]-x[pt])/(LOWESS_smooth_width/2.0); 
			if (dist<0.001) distwt[i]=1.0;
				else {if (dist==1.0) distwt[i]=0.0; else distwt[i] = exp( 3.0*log(1.0-exp(3.0*log(dist))));}	//tricube distance weighting
			residwt[i]=1.0;		//initial residual weights
//fprintf(stderr, "looking for lastpt %i\t%g\t%i\t%g\t%g\n", pt, x[pt], i, x[i], dist); 
		}while ( (dist<=1.0) && (i<n_data-2) ) ;
		if (dist<=1.0) lastpt=i; else lastpt=i-1;
		
		if (jackknife_LOWESS_fits==true) distwt[pt]=0.0;
		
		npoints=lastpt-firstpt+1;
if (npoints<3) {fprintf(stderr, "npoints<3 in LOWESS"); exit(0);}
		for (iter=1; iter<=iterlimit; iter++) { //iteration loop
			sumx=0.0; sumy=0.0; sumxx=0.0; sumy=0.0; sumyy=0.0; sumxy=0.0; sumwt=0.0; 

			for (i=firstpt; i<=lastpt; i++) { //loop through points
				wt=residwt[i]*distwt[i];
				sumx=sumx+wt*x[i]; sumy=sumy+wt*y[i]; sumwt=sumwt+wt;
				sumxx=sumxx+wt*x[i]*x[i]; sumyy=sumyy+wt*y[i]*y[i]; sumxy=sumxy+wt*x[i]*y[i];
			} //loop through points

			meanx=sumx/sumwt;	//weighted regression
			meany=sumy/sumwt;
			slope=(sumxy-meanx*meany*sumwt)/(sumxx-meanx*meanx*sumwt);

			for (i=firstpt; i<=lastpt; i++) { //loop through points, do residuals
				resid[i]=fabs(y[i]-(meany+(x[i]-meanx)*slope));
				v[i-firstpt]=&resid[i];	//pointers for sorting--note indexes on v go from 0 to npoints-1
			} //loop through points
			
			hsort( (void *) v, npoints, comp_function);
			
			m1=1+(npoints/2); m2=npoints-m1+1;
			mad6=3.0*(*v[m1-1] + *v[m2-1]);	//6 times median absolute deviation

			for (i=firstpt; i<=lastpt; i++) { //loop through points, update residual weights
				resid[i]=resid[i]/mad6;
				if (resid[i]>1.0) residwt[i]=0.0;
					else {if (resid[i]<0.001) residwt[i]=1.0;
						else residwt[i]=sqr(1.0-sqr(resid[i]));}	//bisquare weighting
			} //loop through points
			
		}//iteration loop
		
		smooth[pt]=meany+(x[pt]-meanx)*slope;
if(! (fabs(smooth[pt])<1.0e10)) {fprintf(stderr, "problems with LOWESS"); exit(0);}
	}//next pt

	free(distwt);
	free(residwt);
	free(resid);
	free(v);
	
	return(0);
}//LOWESS








//benchmark generation routines
//benchmark generation routines
//benchmark generation routines
//benchmark generation routines

int generate_benchmark(int j)
{	const double pi=3.14159265358979324;
	const double twopi=6.28318530717958648;

	char outfilename[100];  //output filename string
	char filetag[50];  //output filename string
	
	int i, k;
	
	double cos_omega_t, sin_omega_t, cos_omega1, sin_omega1, last_cos, last_sin, freq, norm, phase;
	double cos_omega1_dt, sin_omega1_dt, last_cos_dt, last_sin_dt;
	double cos_omega1_halfdt, sin_omega1_halfdt, last_cos_halfdt, last_sin_halfdt;
	double omega, dt, sin_term, cos_term;

	fptr cos_phase, sin_phase, amp;

//	double cos_omega_dt, sin_omega_dt, cos_omega_halfdt, sin_omega_halft;
	fptr cos_omega_dt, sin_omega_dt, cos_omega_halfdt, sin_omega_halfdt;

	fptr zzcos_omega1, zzsin_omega1, zzcos_omega_t, zzsin_omega_t, zzt, zz_dt;

	if ((benchmark_type != white_noise) && (benchmark_type != random_walk) && (benchmark_type != colored_noise))  {fprintf(stderr, "benchmark type specification is not valid!\n"); exit(0);}

	
	// here we generate white noise benchmark time series, on same samplings as the original data
	if (benchmark_type==white_noise){
		for (i=0; i<n_data; i++) x[i]=nrml();	//replace data values with random normals
	}


	// here we generate random walk benchmark time series, on same samplings as the original data
	if (benchmark_type==random_walk){
		x[0]=0.0;
		for (i=1; i<n_data; i++) x[i]=x[i-1]+nrml()*sqrt(t[i]-t[i-1]);	//replace data values with random walk (steps are random normals, scaled by square root of time interval)
	}
	

	if (benchmark_type==colored_noise) {

		if ( ! (cos_phase = calloc(n_bench_freq , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating phase!\n"); exit(0);}
		if ( ! (sin_phase = calloc(n_bench_freq , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating phase!\n"); exit(0);}
		if ( ! (amp = calloc(n_bench_freq , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating phase!\n"); exit(0);}

		if ( ! (cos_omega_dt = calloc(n_bench_freq , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating cos_omega_dt!\n"); exit(0);}
		if ( ! (sin_omega_dt = calloc(n_bench_freq , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating sin_omega_dt!\n"); exit(0);}
		if ( ! (cos_omega_halfdt = calloc(n_bench_freq , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating cos_omega_halfdt!\n"); exit(0);}
		if ( ! (sin_omega_halfdt = calloc(n_bench_freq , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating sin_omega_halfdt!\n"); exit(0);}


		norm = sqrt(2.0*min_bench_freq);
		//new modification introduced 12 August 2005: normalize by density of sine waves, by multiplying amplitude by sqrt(2.0*min_freq).

		for(k=0; k<n_bench_freq; k++)  { 
			freq = min_bench_freq*(float)(k+1);
			amp[k] = norm*powf( freq , -0.5*bench_alpha )/sqrt(1.0 + (freq*freq)/(bench_fc*bench_fc));        //power-law amplitude function to yield correct power-law slope in spectral power
			//fc is corner frequency that transitions to from 1/f^alpha to 1/f^(alpha+2) roll-off that damps out higher frequency components
			
			phase = twopi * (float)rand()/(float)RAND_MAX;           //random phase in radians
			cos_phase[k] = cos(phase);  //save sine and cosine of phase so we can use them at every point in time.
			sin_phase[k] = sin(phase);
		}

		//here's where we actually calculate the time series
		//for each point we add up the contribution of each frequency component in turn
		if (avgs==false) { //here is the simple case where each point is a grab sample
			for (i=0; i<n_data; i++) {
				x[i]=0.0;
				cos_omega_t=1.0;  sin_omega_t=0.0;  //these hold the current values of omega*t
				cos_omega1=cos(twopi*min_freq*t[i]);  //sines and cosines of the fundamental frequency
				sin_omega1=sin(twopi*min_freq*t[i]);
				for (k=0; k<n_bench_freq; k++) {
					if (k % 20 == 0) {  //every 20 steps, do a real sine and cosine
						cos_omega_t=cos(twopi*min_freq*(double)(k+1)*t[i]);
						sin_omega_t=sin(twopi*min_freq*(double)(k+1)*t[i]); 
					}
					else {  //in between, do sine and cosine by recurrence
						last_cos=cos_omega_t; last_sin=sin_omega_t;  //need these to store the previous sine and cosine
																//because we can't update them till they've been used in *two* formulas
						cos_omega_t=last_cos*cos_omega1 - last_sin*sin_omega1;  //current cosine(omega*t[i]), by recurrence
						sin_omega_t=last_sin*cos_omega1 + last_cos*sin_omega1;  //current sine(omega*t[i]), by recurrence
					} //else
					x[i]=x[i] + amp[k]*( sin_omega_t*cos_phase[k] + cos_omega_t*sin_phase[k] );  //this is sin(omega*t[i]+phase[k]) 
				}//loop frequencies
				
			} // loop times
	
	} else { //if instead each point is a time-integrated sample over the interval since the last point, 
			 //we need to INTEGRATE over the sampling interval, then divide by dt to get the average
			 //we will not do this by the simple integral formulas, because they could have problematic roundoff error when omega*dt is small.

			dt = 1.0/fs;   //assume a constant sampling interval

			for (k=0; k<n_bench_freq; k++) {
				omega=twopi*min_freq*(double)(k+1);
				cos_omega_dt[k]=cos(omega*dt);
				sin_omega_dt[k]=sin(omega*dt);
				cos_omega_halfdt[k]=cos(omega*dt/2.0);
				sin_omega_halfdt[k]=sin(omega*dt/2.0);
			}

			for (i=0; i<n_data; i++) {  //step through each point

				x[i]=0.0;  //this will hold the summation of the contributions from each frequency

				 //these hold the current values of trig functions of omega*t 
				cos_omega_t=1.0;  sin_omega_t=0.0;  
				cos_omega1=cos(twopi*min_freq*t[i]);  //sines and cosines at the fundamental frequency
				sin_omega1=sin(twopi*min_freq*t[i]);

				for (k=0; k<n_bench_freq; k++) {
					omega=twopi*min_freq*(double)(k+1);
					if (k % 20 == 0) {  //every 20 steps, do real trig functions
						cos_omega_t=cos(omega*t[i]);
						sin_omega_t=sin(omega*t[i]);
					}
					else {  //in between, do trig functions by recurrence
						last_cos=cos_omega_t; last_sin=sin_omega_t;  
																//need these to store the previous sines and cosines
																//because we can't update them till they've been used in *two* formulas
						cos_omega_t=last_cos*cos_omega1 - last_sin*sin_omega1;  //current cosine(omega*t[i]), by recurrence
						sin_omega_t=last_sin*cos_omega1 + last_cos*sin_omega1;  //current sine(omega*t[i]), by recurrence
					} //else


					//this is the average of sin(omega*t) from t-dt to t, using trig identities to reduce roundoff error
					sin_term = (sin_omega_t*sin_omega_dt[k] - 2.0*cos_omega_t*sin_omega_halfdt[k]*sin_omega_halfdt[k])/(omega*dt) ;

					//this is the average of cos(omega*t) from t-dt to t, using trig identities to reduce roundoff error
					cos_term = (cos_omega_t*sin_omega_dt[k] + 2.0*sin_omega_t*sin_omega_halfdt[k]*sin_omega_halfdt[k])/(omega*dt) ;


					x[i]=x[i] + amp[k]*( sin_term*cos_phase[k] + cos_term*sin_phase[k] );  //this is sin(omega*t[i]+phase[k]) 

				}//loop frequencies
				
			} // loop times

		} //else



		free(amp);
		free(cos_phase);
		free(sin_phase);
		free(cos_omega_dt);
		free(sin_omega_dt);
		free(cos_omega_halfdt);
		free(sin_omega_halfdt);

	}


	if (save_benchmarks==true) for (i=0; i<n_data; i++) benchmark[j][rowi[i]] = x[i] ; //save benchmark time series into the correct rows of the master compilation of benchmarks

	



}  //generate benchmark


//benchmark generation routines
//benchmark generation routines
//benchmark generation routines
//benchmark generation routines





	








int transform_and_normalize(int j)
{	ITYPE i, ii, k;
	DTYPE sumx, sumxx, sumy, sumyy, sumxy, meanx, meany, slope, intercept, stdev;
	DTYPE sumwt, wtd_sum, wtd_mean, avg_weight, weight;
	fptr xcopy;
	fptr smooth;
	fptr tt;
	fptr xx;
	fptr t3;
	fptr x3;
	fptr ptr;
	char outfilename[100];  //output filename string
	char filetag[20];  //another string we need for filenames
	
	FTYPE **v;					//pointer to pointer array for sort function
	int (*comp_function) (fptr, fptr);	//pointer to sort comparison function
	comp_function=fcomp;			//set pointer to comparison function


	

	if (do_benchmark==true) generate_benchmark(j);



	if (do_LOWESS==true) { 
		if ( ! (xcopy = calloc(n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating xcopy!\n"); exit(0);}
		if ( ! (smooth = calloc(n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating smooth!\n"); exit(0);}
		
		LOWESS(t, x, smooth, n_data);
		for (i=0; i<n_data; i++) xcopy[i]=x[i];	//copy values to temp vector
		for (i=0; i<n_data; i++) x[i]=x[i]-smooth[i];
//for (i=0; i<n_data; i++) fprintf(stderr, "%i\t%g\t%g\t%g\n", i, xcopy[i], smooth[i], x[i]);
		if (save_curves==true) {
			strcpy(outfilename, filespec);  strcat(outfilename, "_low_data");  sprintf(filetag, "%i.txt", j+1);  strcat(outfilename, filetag);
			if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
			else {
				fprintf (outfile, "time\t%s.raw\t%s.lowess\t%s.residual\n", header[j], header[j], header[j]);
				for (i=0; i<n_data; i++) 	fprintf (outfile, "%.10g\t%g\t%g\t%g\n", t[i], xcopy[i], smooth[i], x[i]);
				fclose(outfile);
			} //end else
		} //save LOWESS curves
		free(xcopy);
		free(smooth);
	} //LOWESS transform




	if (do_LN_transform) { 
		LOWESS(t, x, smooth, n_data);
		for (i=0; i<n_data; i++) xcopy[i]=x[i];	//copy values to temp vector
		for (i=0; i<n_data; i++) x[i]=x[i]/smooth[i];	//LN (="LOWESS-normalized") transform is data values divided by LOWESS smooth
		if (save_curves==true) {
			strcpy(outfilename, filespec);  strcat(outfilename, "_LN_data");  sprintf(filetag, "%i.txt", j+1);  strcat(outfilename, filetag);
			if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
			else {
				fprintf (outfile, "time\t%s.raw\t%s.lowess\t%s.LN\n", header[j], header[j], header[j]);
				for (i=0; i<n_data; i++) 	fprintf (outfile, "%.10g\t%g\t%g\t%g\n", t[i], xcopy[i], smooth[i], x[i]);
				fclose(outfile);
			} //end else
		} //save LOWESS curves
	} //LN transform

	
	if (do_rank_transform) {
		v=(FTYPE **)calloc(n_data,sizeof(FTYPE *));
		if ( ! (xcopy = calloc(n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating xcopy!\n"); exit(0);}
		for (i=0; i<n_data; i++) xcopy[i]=x[i];	//copy values to temp vector
		for (i=0; i<n_data; i++) v[i]=&x[i]; //copy addresses to sort vector
		hsort( (void *) v, n_data, comp_function); //sort addresses so that v points to the addresses of the x's in order of ascending value of their contents

		if (n_data==1) *v[0]=0; else { k=-1; //start with k at -1 so the do loop will start right...
			do { i=k+1; k=i;   //note this will not work if n_data=1
				if (k<n_data-2) while ((*v[i] == *v[k+1]) && (k<n_data-2)) k++; //if the ith and k+1st element have the same value, step k up.  This will end with k=i (if there are no ties) or k=(rank of last tie).
				for (ii=i; ii<=k; ii++) *v[ii]=0.5*(i+k)+1.0; //this will be either a) the rank of i, if k=i, or b) the average rank between i and k (if there is a string of ties).  The "+1.0" is so that the ranks run from 1 to n_data rather than 0 to n_data-1
			} while (k<n_data-1);
		} //else 

		if (save_curves==true) {
			strcpy(outfilename, filespec);  strcat(outfilename, "_rank_data");  sprintf(filetag, "%i.txt", j+1);  strcat(outfilename, filetag);
			if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
			else {
				fprintf (outfile, "time\t%s.raw\t%s.rank\n", header[j], header[j]);
				for (i=0; i<n_data; i++) 	fprintf (outfile, "%.10g\t%g\t%g\n", t[i], xcopy[i], x[i]);
				fclose(outfile);
			} //end else
		} //save rank transforms
		free(v);
		free(xcopy);
	}//rank transform





	if (deseasonalize==true) { //to de-seasonalize time series, superimpose fractional years (with wraparound) and do LOWESS fit to the average pattern over all years
		v=(FTYPE **)calloc(n_data+n_data,sizeof(FTYPE *));
		if ( ! (xx = calloc(n_data+n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating xx!\n"); exit(0);}
		if ( ! (tt = calloc(n_data+n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating tt!\n"); exit(0);}	
			
		if ( ! (t3 = calloc(n_data+n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating t3!\n"); exit(0);}
		if ( ! (x3 = calloc(n_data+n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating x3!\n"); exit(0);}
		if ( ! (smooth = calloc(n_data+n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating smooth!\n"); exit(0);}
		if ( ! (xcopy = calloc(n_data , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating xcopy!\n"); exit(0);}
	
		for (i=0; i<n_data; i++) {
			xx[i]=x[i]; 
			tt[i]=t[i]-floor(t[i]); //copy values to first half of temp vectors, truncate year to year fraction
			xx[i+n_data]=xx[i]; 
			if (tt[i]>0.5) tt[i+n_data]=tt[i]-1.0; else tt[i+n_data]=tt[i]+1.0;
				//in second half of temp vectors, copy values with early half-years thrown forward 
				//(to provide wraparound for end-of-year points) and late half-years thrown backward (to provide wraparound for late-year points
		}

		
		for (i=0; i<n_data+n_data; i++) v[i]=&tt[i]; //copy addresses to sort vector
		hsort( (void *) v, n_data+n_data, comp_function); //sort addresses
		
		for (i=0; i<n_data+n_data; i++) {t3[i]=*v[i]; ptr=v[i]-tt+xx; x3[i]=*ptr; }	//sort times into t3, place corresponding xx values into x3		

		LOWESS(t3, x3, smooth, n_data+n_data);
		for (i=0; i<n_data+n_data; i++) {ptr=v[i]-tt+xx; *ptr=*ptr-smooth[i];}  //de-seasonalize values in xx (pointer un-sorts times to do this)

		for (i=0; i<n_data; i++) xcopy[i]=x[i];	//copy values to temp vector
		for (i=0; i<n_data; i++) x[i]=xx[i];  //remember that the original x's, in original order, were in the first half of xx.  these are now de-seasonalized, so copy them back
		
		if (save_curves==true) {
			strcpy(outfilename, filespec);  strcat(outfilename, "_deseason");  sprintf(filetag, "%i.txt", j+1);  strcat(outfilename, filetag);
			if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
			else {
				fprintf (outfile, "time\t%s.raw\t%s.lowess\t%s.residual\n", header[j], header[j], header[j]);
				for (i=0; i<n_data; i++) 	fprintf (outfile, "%.10g\t%g\t%g\t%g\n", t[i], xcopy[i], xcopy[i]-x[i], x[i]);
				fclose(outfile);
			} //end else
		} //save deseasonalized curves
		free(v);
		free(xcopy);
		free(smooth);
		free(xx);
		free(tt);
		free(t3);
		free(x3);
	} //deseasonalize



	
	
	if (do_detrend) {	
		sumx=0.0; sumy=0.0; sumxx=0.0; sumyy=0.0; sumxy=0.0;
		for (i=0; i<n_data; i++) {
			sumx=sumx+t[i]; sumy=sumy+x[i]; sumxx=sumxx + t[i]*t[i]; sumyy=sumyy + x[i]*x[i]; sumxy=sumxy + t[i]*x[i];
		}  
		meanx=sumx/(FTYPE) n_data; 
		meany=sumy/(FTYPE) n_data;
		slope=(sumxy-meanx*meany*n_data)/(sumxx-meanx*meanx*n_data);
		intercept=meany-(meanx*slope);
		for (i=0; i<n_data; i++)  x[i]=x[i]-(intercept+slope*t[i]);   //detrend
		
	}

	
	//normalize amplitude and/or subtract mean
	sumx=0.0; sumxx=0.0;
	for (i=0; i<n_data; i++) {sumx=sumx+x[i]; sumxx=sumxx+x[i]*x[i];}	
	meanx=sumx/n_data;
	stdev=sqrt((sumxx-sumx*sumx/n_data)/n_data);
	if (normalize_amplitude) { for (i=0; i<n_data; i++) x[i]=(x[i]-meanx)/stdev; }
		else { for (i=0; i<n_data; i++) x[i]=x[i]-meanx; }
		
	if ((use_window == true) && (c==0.0)) { 
		sumwt=0.0; wtd_sum=0.0;
		for (i=0; i<n_data; i++) {
//			weight=(1.0-sqr((t[i]-0.5*(t[n_data-1]+t[0]))/(0.5*(t[n_data-1]-t[0])))); //Welch window	             ***IF YOU CHANGE WINDOW SELECTION HERE, YOU HAVE TO DO LIKEWISE BELOW
			weight=(1.0-cos(twopi*(t[i]-t[0])/(t[n_data-1]-t[0])))/2.0; //cosine window
//			weight=(1.0-fabs((t[i]-0.5*(t[n_data-1]+t[0]))/(0.5*(t[n_data-1]-t[0])))); //Parzen window
			sumwt=sumwt+weight;
			wtd_sum=wtd_sum+x[i]*weight;}
		wtd_mean=wtd_sum/sumwt;
		avg_weight=sumwt/(float) n_data;
		for (i=0; i<n_data; i++) {
//			weight=(1.0-sqr((t[i]-0.5*(t[n_data-1]+t[0]))/(0.5*(t[n_data-1]-t[0])))); //Welch window
			weight=(1.0-cos(twopi*(t[i]-t[0])/(t[n_data-1]-t[0])))/2.0; //cosine window
//			weight=(1.0-fabs((t[i]-0.5*(t[n_data-1]+t[0]))/(0.5*(t[n_data-1]-t[0])))); //Parzen window
			x[i]=(weight/avg_weight)*(x[i]-wtd_mean);}  
	}  //Welch window function
	
	return(1);
}//transform_and_normalize







//wavelet or DCDFT calculations
//wavelet or DCDFT calculations
//wavelet or DCDFT calculations
//wavelet or DCDFT calculations

DTYPE inner_product( 	fptr x, 
					fptr y, 
					fptr wt, 
					fptr pointwt,
					ITYPE offset, 
					ITYPE len,
					DTYPE sumwt ) {
	ITYPE i; DTYPE sum;   //returns the weighted inner product of two i-vectors, from startpt to endpt
	sum=0.0;
	for( i=0 ; i<len; i=i+1) {
		sum=sum + x[i+offset] * y[i+offset] * pointwt[i+offset] * wt[i+offset]; }
	if (sumwt==0.0)  return( null ); else return( sum/sumwt );
}




int invert3( smatrix s, smatrix inverse ){		//brute-force inversion of 3x3 matrix
	double det;		
//	double avg;
	
	det=s[0][0]*s[1][1]*s[2][2] + s[0][1]*s[1][2]*s[2][0] + s[0][2]*s[1][0]*s[2][1] - s[0][0]*s[1][2]*s[2][1] - s[0][1]*s[1][0]*s[2][2] - s[0][2]*s[1][1]*s[2][0] ;
								//Sarrus's rule for 3x3 determinant
	
	if (fabs(det)<1.0e-6) return (-1);  //return with error condition -- singular matrix
	
	inverse[0][0]=(s[1][1]*s[2][2] - s[1][2]*s[2][1])/det;
	inverse[1][0]=-(s[1][0]*s[2][2] - s[1][2]*s[2][0])/det;
	inverse[2][0]=(s[1][0]*s[2][1] - s[1][1]*s[2][0])/det;
	inverse[0][1]=-(s[0][1]*s[2][2] - s[2][1]*s[0][2])/det;
	inverse[1][1]=(s[0][0]*s[2][2] - s[2][0]*s[0][2])/det;
	inverse[2][1]=-(s[0][0]*s[2][1] - s[2][0]*s[0][1])/det;
	inverse[0][2]=(s[0][1]*s[1][2] - s[1][1]*s[0][2])/det;
	inverse[1][2]=-(s[0][0]*s[1][2] - s[1][0]*s[0][2])/det;
	inverse[2][2]=(s[0][0]*s[1][1] - s[1][0]*s[0][1])/det;
	
	return(1);  //return with OK condition		
}







int analyze_location(ls_ptr ls) {
	ITYPE i, offset, len; 
	DTYPE sumwt, sum2wt;
	DTYPE phi_x[3]; //inner products of test functions and data
	FTYPE yconst, n_eff; 
	smatrix s, s_inv;
	DTYPE tau, tcos, tsin, mean_tcos, mean_tsin, mean_tcos2, mean_tsin2, mean_x2;
	DTYPE sin_tau, cos_tau, real, imag, norm2;
	DTYPE half_width, delta_t ;
	
	DTYPE mean_x_tsin, mean_x_tcos, mean_x, unwtd_mean_x, unwtd_mean_tcos, unwtd_mean_tsin, weight;
	DTYPE mean_sincos, mean_sin, mean_cos, mean_sin2, mean_cos2;


	if ((c==0.0) && (do_WOSA==false)) {
		ls->last_startpt=0; offset=0; len=n_data;
		for (i=offset; i<offset+len; i++) wt[i]=1.0;}
	else {
		//find first weight >threshold
		i=ls->last_startpt;
		if (do_WOSA==true) half_width = sqrt(-log(te)/0.5)*WOSA_sigma; else half_width = sqrt(-log(te)/c)/ls->omega;
		while (( t[i] < (ls->center - half_width) ) && (i<n_data-2) )  i=i+1;
		ls->last_startpt=i; 
		offset=i;
		while (( t[i] < (ls->center + half_width) ) && (i<n_data-2) )  i=i+1;
		if (t[i] > (ls->center + half_width) ) i=i-1;  //back up one if we've gone beyond end
		len = i - offset + 1;  //calculate number of points in set
		if (do_WOSA==true) {for (i=offset; i<offset+len; i++) wt[i]=exp(-0.5*sqr((t[i]-ls->center)/WOSA_sigma));}
			else {for (i=offset; i<offset+len; i++) wt[i]=exp(-c*sqr(ls->omega)*sqr(t[i]-ls->center)); }
		}

	delta_t=(t[offset+len-1] - t[offset]) / ( ((float) len) - 1.0 ) ; 


	
	if (len<3) return(-1);  // location fails--length too small

	sumwt=0.0; sum2wt=0.0;
	for (i=offset; i<offset+len; i++) {sumwt=sumwt+wt[i]*pointwt[i]; sum2wt=sum2wt+wt[i]*wt[i]*pointwt[i]*pointwt[i];}
	n_eff = sqr(sumwt)/sum2wt;  //calculate n_eff;
	ls->n_eff = n_eff ; 
	ls->len=len;
	
	if (n_eff<3.0) return(-1);  // location fails--n_eff too small
	
	if (sumwt<1.0e-10) return(-1);  //location fails--all points have zero weight (can happen if pointwt's are all zeroes!)

	if (calculation_mode==0) {		//FT of delta functions at event times (i.e. just a summation of sine and cosine terms)
		mean_sin=0.0; mean_cos=0.0;
		for (i=offset; i<offset+len; i++) {  
			weight=wt[i]*pointwt[i]/sumwt;
			mean_sin=mean_sin+sin_t[i]*weight;
			mean_cos=mean_cos+cos_t[i]*weight;}    //note weights are already normalized by sumwt, so these sums are means
		ls->ycos=mean_cos; ls->ysin=mean_sin; 
	}





	if (calculation_mode==1) {		//Foster's method
		//construct s-matrix, and invert
		s[0][0]=inner_product( unity, unity, wt, pointwt, offset, len, sumwt );  //unity vector with itself
		s[1][1]=inner_product( cos_t, cos_t, wt, pointwt, offset, len, sumwt );  //cosine with cosine
		s[2][2]=inner_product( sin_t, sin_t, wt, pointwt, offset, len, sumwt );  //sine with sine
		s[0][1]=inner_product( unity, cos_t, wt, pointwt, offset, len, sumwt ); s[1][0]=s[0][1]; //unity vector with cosine
		s[0][2]=inner_product( unity, sin_t, wt, pointwt, offset, len, sumwt ); s[2][0]=s[0][2]; //unity vector with sine
		s[1][2]=inner_product( cos_t, sin_t, wt, pointwt, offset, len, sumwt ); s[2][1]=s[1][2]; //cosine with sine
		if (invert3( s, s_inv )==-1) { return (-1); }//exit w/ error condition: singular matrix!
		
		// calculate inner products of test fcns and data
		phi_x[0]=inner_product( x, unity, wt, pointwt, offset, len, sumwt ); //unity vector with measurements
		phi_x[1]=inner_product( x, cos_t, wt, pointwt, offset, len, sumwt ); //cosine vector with measurements
		phi_x[2]=inner_product( x, sin_t, wt, pointwt, offset, len, sumwt ); //sine vector with measurements
		
		// now do Foster's eqn. 4-4
		ls->ycos=0.0; ls->ysin=0.0; yconst=0.0;
		for (i=0; i<=2; i++) {
			yconst=yconst+s_inv[0][i]*phi_x[i];
			ls->ycos=ls->ycos+s_inv[1][i]*phi_x[i];  //best-fit coefficients for sine and cosine
			ls->ysin=ls->ysin+s_inv[2][i]*phi_x[i]; 
		}
		
		ls->Vx=inner_product( x, x, wt, pointwt, offset, len, sumwt ) - sqr(phi_x[0]);		//total wtd mean square
	
		//foster's calculation of Vy
		ls->Vy=yconst*phi_x[0] + ls->ycos*phi_x[1] + ls->ysin*phi_x[2] - sqr(phi_x[0]);		//model wtd mean square
	

	} //calculation mode 1
	
	


	if (calculation_mode >1 ) {        

		mean_sincos=0.0; mean_sin=0.0; mean_cos=0.0; mean_sin2=0.0; mean_cos2=0.0; 
		for (i=offset; i<offset+len; i++) {
			weight=wt[i]*pointwt[i]/sumwt;
			mean_sincos=mean_sincos+sin_t[i]*cos_t[i]*weight;
			mean_sin=mean_sin+sin_t[i]*weight;
			mean_cos=mean_cos+cos_t[i]*weight;
			mean_sin2=mean_sin2+sin_t[i]*sin_t[i]*weight;
			mean_cos2=mean_cos2+cos_t[i]*cos_t[i]*weight;}    //note weights are already normalized by sumwt, so these sums are means
	
		if (calculation_mode == 5) tau=(0.5/ls->omega)*atan2(2.0*mean_sincos,mean_cos2-mean_sin2);  //if doing LSFT, use Scargle's rotation
			else tau=(0.5/ls->omega)*atan2(2.0*(mean_sincos-mean_sin*mean_cos),(mean_cos2-mean_sin2)-(mean_cos*mean_cos-mean_sin*mean_sin));  
			//otherwise use the CORRECT rotation, NOT Scargle's rotation!

if ((mean_cos2-mean_cos*mean_cos)+(mean_sin2-mean_sin*mean_sin)<=1.0e-4) return (-1);  
//if we've hit the sampling frequency itself, and thus the phase space has collapsed, return without result (since any result would be ill-defined)

		sin_tau=sin(ls->omega*tau); cos_tau=cos(ls->omega*tau);    //here are the rotation coefficients

		mean_x_tsin=0.0; mean_x_tcos=0.0;	mean_tcos=0.0; mean_tsin=0.0; mean_tcos2=0.0; mean_tsin2=0.0; mean_x=0.0; mean_x2=0.0;
		unwtd_mean_x=0.0; unwtd_mean_tcos=0.0; unwtd_mean_tsin=0.0;

		for (i=offset; i<offset+len; i++) {
			tcos=cos_t[i]*cos_tau+sin_t[i]*sin_tau; 
			tsin=sin_t[i]*cos_tau-cos_t[i]*sin_tau;    //using trig identities for (omega[k]*(t[i]-tau)) 
			weight=wt[i]*pointwt[i]/sumwt;
			mean_x_tcos=mean_x_tcos+x[i]*tcos*weight;
			mean_x_tsin=mean_x_tsin+x[i]*tsin*weight;
			mean_tcos=mean_tcos+tcos*weight;
			mean_tsin=mean_tsin+tsin*weight;
			unwtd_mean_tcos=unwtd_mean_tcos+tcos/(float) len;
			unwtd_mean_tsin=unwtd_mean_tsin+tsin/(float) len;
			mean_tcos2=mean_tcos2+tcos*tcos*weight;
			mean_tsin2=mean_tsin2+tsin*tsin*weight;
			mean_x=mean_x+x[i]*weight;
			unwtd_mean_x=unwtd_mean_x+x[i]/(float) len;
			mean_x2=mean_x2+x[i]*x[i]*weight;}   //note weights are already normalized by sumwt, so these sums are means
			
		if (test_phase_space==true) { if (2.0*sqrt((mean_tcos2-mean_tcos*mean_tcos)*(mean_tsin2-mean_tsin*mean_tsin))<0.30) return(-1); }  //not enough of phase space is spanned by data
			// expected phase space volume is stdev(cos)*stdev(sin)=1/2 if points fill the circle... so number on the right is the fraction of phase space that needs to be filled in order to go ahead with calculations
		
		if (calculation_mode == 5) { //Scargle's equation
			if (mean_tcos2<0.0001) real=0.0; else  //if it's almost a divide-by-zero, don't! -- this protects against blowup at Nyquist frequency
				real=sqrt(2.0)*mean_x_tcos/sqrt(mean_tcos2); 
			if (mean_tsin2<0.0001) imag=0.0; else  //if it's almost a divide-by-zero, don't! -- this protects against blowup at Nyquist frequency
				imag=sqrt(2.0)*mean_x_tsin/sqrt(mean_tsin2);}
		
		if (calculation_mode == 9) {
			real=2.0*(mean_x_tcos - mean_x*mean_tcos); //here's the convolution
			imag=2.0*(mean_x_tsin - mean_x*mean_tsin); }

		if (calculation_mode == 7) {		//this is the DCDFT
				real=(mean_x_tcos - mean_x*mean_tcos)/(mean_tcos2-mean_tcos*mean_tcos); //here's the least-squares regression
				imag=(mean_x_tsin - mean_x*mean_tsin)/(mean_tsin2-mean_tsin*mean_tsin);}

		if (calculation_mode == 8) {		//scargle-esque approach, with centered cos and sin and correct rotation
				real=sqrt(2.0)*(mean_x_tcos - mean_x*mean_tcos)/sqrt(mean_tcos2-mean_tcos*mean_tcos); //here's the least-squares regression
				imag=sqrt(2.0)*(mean_x_tsin - mean_x*mean_tsin)/sqrt(mean_tsin2-mean_tsin*mean_tsin);}		

		ls->Vx=mean_x2-mean_x*mean_x;		//total wtd mean square
		ls->Vy=real*real*(mean_tcos2-mean_tcos*mean_tcos)+imag*imag*(mean_tsin2-mean_tsin*mean_tsin);		//model wtd mean square
		
		ls->ycos=real*cos_tau-imag*sin_tau;  //now rotate back into original time base
		ls->ysin=imag*cos_tau+real*sin_tau;

	} //calculation modes other than 1



		//power as square of amplitude
		ls->pwr=(sqr(ls->ycos) + sqr(ls->ysin));

		if (use_DFT_normalization==false) ls->pwr = ls->pwr*0.5*n_eff*delta_t ;  //UNIVERSAL NORMALIZATION FOR POWER SPECTRAL DENSITY

//		for square of amplitude (rather than power spectral density) of sinusoidal signals, don't correct ls->pwr, even if it's a wavelet.	

		ls->sgnl=ls->Vy*(n_eff-1.0)/(n_eff-3.0) - ls->Vx*(2.0/(n_eff-3.0));  //best estimate of true signal in model (by subtracting expected noise variance that leaks into model)
		ls->noise=(ls->Vx - ls->Vy)*(1.0 + 2.0/(n_eff-3.0));  //best estimate of total noise in data (including noise that leaks into the model)
		ls->model_noise=2.0/(n_eff-3.0)*(ls->Vx - ls->Vy);  //noise that has leaked into model
		ls->wwz= ((ls->n_eff - 3.0) * ls->Vy) / (2.0 * (ls->Vx - ls->Vy));  //ratio of model variance to model_noise
		ls->s2n=ls->Vy/(ls->Vx - ls->Vy);  //this is Foster's signal-to-noise
		ls->phase=atan2( ls->ysin , ls->ycos );

		if (do_logfile==true) {for (i=offset; i<offset+len; i++) {
			fprintf (outfile, "%i\t%8.6g\t%8.6g\t%8.6g\t%8.6g\t%8.6g\t%8.6g\t%8.6g\t%8.6g\t%8.6g\t%8.6g", offset, ls->omega, ls->pwr, t[i], cos_t[i], sin_t[i], cos_t[i]*cos_tau+sin_t[i]*sin_tau, sin_t[i]*cos_tau-cos_t[i]*sin_tau, wt[i]*pointwt[i]/sumwt, x[i], mean_x+real*(cos_t[i]*cos_tau+sin_t[i]*sin_tau-mean_tcos)+imag*(sin_t[i]*cos_tau-cos_t[i]*sin_tau-mean_tsin) );
			fprintf (outfile, "\t%8.6g\t%8.6g\t%8.6g\t%8.6g\n", atan2(sin_t[i]*cos_tau-cos_t[i]*sin_tau,cos_t[i]*cos_tau+sin_t[i]*sin_tau), atan((sin_t[i]*cos_tau-cos_t[i]*sin_tau)/(cos_t[i]*cos_tau+sin_t[i]*sin_tau)), real, imag );
		}}

	return(1); // exit w/ success condition
} //analyze location




int analyze_wavelength( ITYPE j, ITYPE k ) {
	ITYPE i, nom_loc, tally, n_loc;
	FTYPE spacing, omega;
	DTYPE sum_pwr, sum_df;
	ls_ptr ls;
	
	omega=twopi*freq[k];
	
	for (i=0; i<n_data; i=i+1) cos_t[i]=cos(omega*t[i]);
	for (i=0; i<n_data; i=i+1) sin_t[i]=sin(omega*t[i]);
	

	//determine number of locations
	if (do_WOSA==true) {
		spacing=loc_spacing*WOSA_sigma;	//if this is WOSA, then set spacing by (fixed) sigma-width
		nom_loc= 1+ floor((t[n_data-1]-t[0])/spacing);
	} else {
		if (c==0.0) {nom_loc=1; spacing=1.0;}	//if this is just DCDFT
			else { spacing=loc_spacing*(1.0/freq[k])/(twopi*sqrt(2.0*c));	//for wavelets, spacing=loc_spacing * sigma
				nom_loc= 1+ floor((t[n_data-1]-t[0])/spacing);}
	};
	
	//allocate space for array of location structs
	if ( ! (ls = malloc(sizeof(struct lstruct)) ))  {fprintf(stderr, "out of memory allocating loc at freq=%g!\n",freq[k]); exit(0);};

	ls->omega=omega;
	
	ls->last_startpt=0;
	n_loc=0;
	tally=0;
	sum_df=0.0;
	sum_pwr=0.0;
	xxsignal[j][k]=0.0;
	xxwwz[j][k]=0.0;
	xxmodel_noise[j][k]=0.0;
	for (i=0; i<nom_loc; i=i+1) {
		ls->center = (0.5*t[0]+0.5*t[n_data-1])-0.5*spacing*(nom_loc-1)+i*spacing;  //determine locations of centers
		if (analyze_location( ls )>0 )   //analyze_location returns 1 if successful, 0 or less otherwise
			{ tally=tally+ls->len; //add length to point tally
				sum_pwr=sum_pwr + ls->pwr * (ls->n_eff - 3.0);  //sum power, weighted by degrees of freedom
				sum_df=sum_df + (ls->n_eff - 3.0);
				xxsignal[j][k]=xxsignal[j][k]+ls->sgnl;
				xxwwz[j][k]=xxwwz[j][k]+ls->wwz;
				xxmodel_noise[j][k]=xxmodel_noise[j][k]+ls->model_noise;
				n_loc=n_loc+1;
			} // if successful location
	} //for loop
	results[j][k]=sum_pwr / sum_df ;  //average power across each wavelet, weighted by degrees of freedom
	xxwwz[j][k]=xxwwz[j][k] / (double) n_loc;  //average noise across each wavelet
	xxsignal[j][k]=xxsignal[j][k] / (double) n_loc;  //average signal across each wavelet
	xxmodel_noise[j][k]=xxmodel_noise[j][k] / (double) n_loc;  //average model noise across each wavelet


//	if (c>0.0) results[j][k]=results[j][k]/n_data;	//for wavelets, divide by n_data so different data sets will have same power at long wavelength...right?
	if (tally<n_data/2.0) {
		results[j][k]=null;
		sm_pwr[j][k]=null;
		xxwwz[j][k]=null;
		xxsignal[j][k]=null;
		xxmodel_noise[j][k]=null;}


	if (apply_wavelength_limits==true) {
		if ( (freq[k]<1.0001/(t[n_data-1]-t[0])) || (freq[k]>0.5/median_delta_t) ) {   //if wavelength is longer than data set or shorter than nyquist, null things out
			results[j][k]=null;
			sm_pwr[j][k]=null;
			xxwwz[j][k]=null;
			xxsignal[j][k]=null;
			xxmodel_noise[j][k]=null;}
	} //if


 if ( (float)(k+1)/10.0 == trunc((float)(k+1)/10.0))fprintf(stderr,"%i\t%i\t%6.4g\t%8.6g\t%8.6g\t%i\t%i\t%i\r", j+1, k+1, c, twopi/omega, results[j][k], tally, n_data, n_loc); //******
	
	
	if (c==0.0) {
		sine_coeff[j][k]=ls->ysin;               //save raw sine and cos terms here
		cosine_coeff[j][k]=ls->ycos;
	}
	

	free(ls);
	return(n_loc);
} //analyze wavelength





//wavelet or DCDFT calculations
//wavelet or DCDFT calculations
//wavelet or DCDFT calculations
//wavelet or DCDFT calculations











int recover_storage( void ) {
	
	free(x);
	free(t);
	free(unity);
	free(cos_t);
	free(sin_t);
	free(wt);
	
	return(1);
} //recover storage





int print_results( void ) {
	ITYPE j,k;
	char outfilename[100];  //output filename string
	char outfilespec[100];  //output filename string
	
	strcpy(outfilespec, filespec);
	sprintf( outfilename, "_%i_%i_mode%i", first_column, first_column+n_columns-1, calculation_mode );  	//we'll re-use this for its real purpose later
	strcat(outfilespec, outfilename);
	if (do_benchmark) {if (benchmark_type==random_walk) strcat(outfilespec, "_WALK"); 
	else if (benchmark_type==white_noise) strcat(outfilespec, "_WHITE");
	else if (benchmark_type==colored_noise) {sprintf( outfilename, "_NOISE_alpha=%g", bench_alpha); strcat(outfilespec, outfilename);}
	}

	if (do_WOSA) sprintf( outfilename, "_WOSA=%g", WOSA_sigma);
		else sprintf( outfilename, "_c=%g", c );  	//we'll re-use this for its real purpose later
	strcat(outfilespec, outfilename);
	if ((use_window) && (c==0.0)) strcat(outfilespec, "w");
	if (do_LOWESS) strcat(outfilespec, "_low");
	if (do_LN_transform) strcat(outfilespec, "_LN");
	if (do_rank_transform) strcat(outfilespec, "_rank");
	if (do_detrend) strcat(outfilespec, "_det");
	if (deseasonalize) strcat(outfilespec, "_des");
	if (normalize_amplitude) strcat(outfilespec, "_norm");

	strcpy(outfilename, outfilespec);
	if (use_DFT_normalization) strcat(outfilename, "_DFT.txt"); 
		else strcat(outfilename, "_pwr.txt");
	if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
	else {
		fprintf (outfile, "freq\tfs");
		if (use_DFT_normalization) {for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s", header[j]);} 
			else {for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s", header[j]);};
		fprintf (outfile, "\n");

		for (k=0; k<n_freq; k++) {
			fprintf (outfile, "%g\t%g", freq[k], fs);
			for (j=0; j<n_columns; j++)  { if (results[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%g", results[j][k]); } ;
			fprintf (outfile, "\n");
		}//loop frequencies

		fclose(outfile);
	} //end else


		strcpy(outfilename, outfilespec);
		if (use_DFT_normalization) strcat(outfilename, "_smDFT.txt");
			else strcat(outfilename, "_smpwr.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "freq\tfs");
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s", header[j]);
			fprintf (outfile, "\n");
	
			for (k=0; k<n_freq; k++) {
				fprintf (outfile, "%g\t%g", freq[k], fs);
				for (j=0; j<n_columns; j++)  { if (results[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%g", sm_pwr[j][k]); } ;
				fprintf (outfile, "\n");
			}//loop frequencies
	
			fclose(outfile);
		} //end else


	

if (do_alias_filtering==true) {
		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_filtered.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "lambda\tfreq");
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s", header[j]);
			fprintf (outfile, "\n");
	
			for (k=0; k<n_freq; k++) {
				fprintf (outfile, "%g\t%g", lambda[k], freq[k]);
				for (j=0; j<n_columns; j++)  { if (results[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%g", filtered_pwr[j][k]); } ;
				fprintf (outfile, "\n");
			}//loop frequencies
	
			fclose(outfile);
		} //end else
}

	

	

if (do_alias_filtering==true) {
	//write file of all powers
		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_allpwr.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "lambda\tfreq");
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s.pwr\t%s.sm_pwr\t%s.filt_pwr\t%s.model\t%s.aliased", header[j], header[j], header[j], header[j], header[j]);
			fprintf (outfile, "\n");
	
			for (k=0; k<n_freq; k++) {
				fprintf (outfile, "%g\t%g", lambda[k], freq[k]);
				for (j=0; j<n_columns; j++)  { if (results[j][k]==null) fprintf(outfile, "\t\t\t\t\t"); else fprintf (outfile, "\t%g\t%g\t%g\t%g\t%g", results[j][k], sm_pwr[j][k], filtered_pwr[j][k], save_model[j][k], save_aliased[j][k]); } ;
				fprintf (outfile, "\n");
			}//loop frequencies
	
			fclose(outfile);
		} //end else
} else {
	//write file of all powers
		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_allpwr.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "lambda\tfreq");
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s.pwr\t%s.sm_pwr", header[j], header[j]);
			fprintf (outfile, "\n");
	
			for (k=0; k<n_freq; k++) {
				fprintf (outfile, "%g\t%g", lambda[k], freq[k]);
				for (j=0; j<n_columns; j++)  { if (results[j][k]==null) fprintf(outfile, "\t\t"); else fprintf (outfile, "\t%g\t%g", results[j][k], sm_pwr[j][k]); } ;
				fprintf (outfile, "\n");
			}//loop frequencies
	
			fclose(outfile);
		} //end else
}



if (do_alias_filtering==true) {
	//write file of all alphas
		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_alpha.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "series\talpha\talpha_se\tregr_alpha\tregr_alpha_se\n");
			for (j=0; j<n_columns; j++) fprintf (outfile, "%s\t%g\t%g\t%g\t%g\n", header[j], save_alpha[j], save_alpha_se[j], save_regr_alpha[j], save_regr_alpha_se[j]);   //loop frequencies
	
			fclose(outfile);
		} //end else
}



	
	if (c==0.0) {	//write file of sine and cosine coefficients
		strcpy(outfilename, outfilespec);
		strcat(outfilename, "_coeffs.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "lambda\tfreq");
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s.cos\t%s.sin", header[j], header[j]);
			fprintf (outfile, "\n");
	
			for (k=0; k<n_freq; k++) {
				fprintf (outfile, "%g\t%g", lambda[k], freq[k]);
				for (j=0; j<n_columns; j++)  { if (results[j][k]==null) fprintf(outfile, "\t\t"); else fprintf (outfile, "\t%g\t%g", cosine_coeff[j][k], sine_coeff[j][k]); }
				fprintf (outfile, "\n");
			}//loop frequencies
			fclose(outfile);
		} //end else
	}





if (save_benchmarks==true) {
		strcpy(outfilename, outfilespec);
			strcat(outfilename, "_benchmk.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else {
			fprintf (outfile, "time");
			for (j=0; j<n_columns; j++) fprintf (outfile, "\t%s", header[j]);
			fprintf (outfile, "\n");
	
			for (k=0; k<n_rows; k++) {
				fprintf (outfile, "%.10g", original_t[k]);
				for (j=0; j<n_columns; j++)  { if (benchmark[j][k]==null) fprintf(outfile, "\t"); else fprintf (outfile, "\t%.8g", benchmark[j][k]); } ;
				fprintf (outfile, "\n");
			}//loop rows
	
			fclose(outfile);
		} //end else
}

	




return (0);
}//print results
	







int setup_benchmark_saves( void ) {//opens infile, counts lines, and sets up the benchmark save arrays
	ITYPE i, j;
	double tt; //use these to transfer to possible float-sized t[i], x[i], and pointwt[i]
	char junk[MAX_LINE_LENGTH+1]; 

fprintf(stderr,"setting up benchmark saves \n");
		
	 //open input file and verify it exists
	if( (infile=fopen(infilename,"r")) == NULL ) {fprintf(stderr, "input file %s could not be opened!\n",infilename); exit(-1); }

	// run through file and count legal points
	n_rows=0;
	fgets(junk, MAX_LINE_LENGTH, infile);	//discard first line
	do {n_rows=n_rows+1;
		fgets(junk, MAX_LINE_LENGTH, infile);	
	} while (feof(infile)==0);
	n_rows=n_rows-1;
	
	// now allocate memory for vectors that will save t values and benchmark data
	if ( ! (original_t = calloc(n_rows , sizeof(double)) )) {fprintf(stderr, "out of memory allocating original_t!\n"); exit(0);}
	for (j=0; j<n_columns; j++)	if ( ! (benchmark[j] = calloc(n_rows , sizeof(double)) )) {fprintf(stderr, "out of memory allocating benchmark!\n"); exit(0);}

	for (j=0; j<n_columns; j++)	for (i=0; i<n_rows; i++) benchmark[j][i]=null;  //set all benchmark array values to null

	// reset file, get data values, and store them in t and x
	fclose(infile);
	infile=fopen(infilename,"r");
	fgets(junk, MAX_LINE_LENGTH, infile);	//discard first line

	for (i=0; i<n_rows; i++) {
		tt=null;   //now fill the original_time vector
		fscanf(infile, "%lg", &tt);		//get the time value
		fgets(junk, MAX_LINE_LENGTH, infile);	//get the rest of the line
		original_t[i] = tt;			//save the time value in original_time vector
	} 
	
	fclose(infile);
	return(0);
}  //end of setup_benchmark_saves











int main( void ) {
	ITYPE i;		//index for iterations
	ITYPE smooth_width;
	fptr logf;
	int j;		//index for columns
	int k, kmin, kmax;	//index for frequencies
	FTYPE weight, sumwt, wtd_sum;
	FTYPE meanx, meany, meanxx, meanxy;
	char outfilename[100];  //output filename string
	char outfilespec[100];  //output filename string
	int (*comp_function) (fptr, fptr);	//pointer to sort comparison function
	comp_function=fcomp;			//set pointer to comparison function
	
	double filtered[n_freq];
	double model[n_freq];
	double aliased[n_freq];
	double raw[n_freq];
	double alpha, alpha_se, regr_slope, regr_slope_se;
	
	
	
	if (do_logfile==true) {
		strcpy(outfilespec, filespec);
		sprintf( outfilename, "_mode%i", calculation_mode );  	//we'll re-use this for its real purpose later
		strcat(outfilespec, outfilename);
		if (do_WOSA) sprintf( outfilename, "_WOSA=%g", WOSA_sigma);
			else sprintf( outfilename, "_c=%g", c );  	//we'll re-use this for its real purpose later
		strcat(outfilespec, outfilename);
		if ((use_window) && (c==0.0)) strcat(outfilespec, "w");
		if (do_LOWESS) strcat(outfilespec, "_low");
		if (do_LN_transform) strcat(outfilespec, "_LN");
		if (do_rank_transform) strcat(outfilespec, "_rank");
		if (do_detrend) strcat(outfilespec, "_det");
		if (normalize_amplitude) strcat(outfilespec, "_norm");
	
		strcpy(outfilename, outfilespec);
		strcat(outfilename, "_log.txt");
		if ( ! (outfile=fopen(outfilename, "w+")) ) {fprintf(stderr, "output file %s could not be opened!\n", outfilename); exit(0);}
		else fprintf (outfile, "offset\tomega\tpwr\tt[i]\tcos_t\tsin_t\tcos_t-tau\tsin_t-tau\twt[i]\tx[i]\tfit[i]\tphase[i]\tphase2[i]\treal\timag\n");
		
	}

	if (save_benchmarks==true) setup_benchmark_saves( );

		
	for (j=0; j<n_columns; j++) header[j]=malloc(sizeof(char[header_len]));

	start_time=time(NULL);
	setbran(start_time);		//initialize random number generator
	setnrml(start_time);		//initialize random normal generator
	srand(start_time);			//initialize random number generator







	
	//initialize wavelengths
	for (k=0; k<n_freq; k=k+1) {	
		if (freq_even_in_logs == true) freq[k] = exp( log(10.0)*(log10(min_freq) + (log10(max_freq)-log10(min_freq))*k/(n_freq-1)) ) ;
			else if (delimited_freq == true) freq[k] = min_freq + (max_freq-min_freq) * (k+1)/(FTYPE) n_freq;
			else freq[k] = max_freq * (k+1)/(FTYPE) n_freq;
		lambda[k]=1.0/freq[k];
	} //loop through k






		
//here's where the analysis begins
	
	for (j=0; j<n_columns; j++) {	//step through read_columns
		read_in_data(j+first_column);
		transform_and_normalize(j);
fprintf(stderr, "\n");
fprintf(stderr, "%i data read in... elapsed time %g seconds\n", j+first_column, difftime( time(NULL) , start_time ) );
	
		for (k=0; k<n_freq; k++) { analyze_wavelength(j,k);	}

fprintf(stderr, "\n%i data analyzed... elapsed time %g seconds\n", j+first_column, difftime( time(NULL) , start_time ) );

		recover_storage( );		//recover storage	
		
	}	//finish working through read_columns
	



//fprintf(stderr, "got to end of FT's \n");


	if (smooth_over_bins==true) {  // here we do an ordinary smooth over a given number of bins
		if (triangular_smooth==true) smooth_width=half_width;
			else smooth_width=half_width+half_width+half_width;
		for (j=0; j<n_columns; j++) {	//step through read_columns
			for (k=0; k<n_freq; k=k+1) {
				sumwt=0.0;
				wtd_sum=0.0; meanx=0.0; meanxx=0.0; meany=0.0; meanxy=0.0;
				for (i=-smooth_width; i<=smooth_width; i++) if ( (i+k>=0) && (i+k<n_freq) ) {
					if (results[j][i+k] != null) {
						if (triangular_smooth==true) weight=1.0-fabs(i/(FTYPE) smooth_width);  //triangular smooth... there should be scaling constants in here but they're irrelevant because we normalize by sumwt anyhow (so it's only relative weight that matters)
							else weight=exp(-sqr(i/(FTYPE) half_width));  //gaussian smooth... there should be scaling constants in here but they're irrelevant because we normalize by sumwt anyhow (so it's only relative weight that matters)
						sumwt=sumwt+weight;
						wtd_sum=wtd_sum+weight*results[j][i+k];
						meanx=meanx+log10(freq[k+i]/freq[k])*weight;
						meanxx=meanxx+log10(freq[k+i]/freq[k])*log10(freq[k+i]/freq[k])*weight;
						meany=meany+weight*log10(results[j][i+k]);
						meanxy=meanxy+weight*log10(freq[k+i]/freq[k])*log10(results[j][i+k]);
					}
				} // loop i's
				if ((sumwt==0.0) || (results[j][k]==null)) sm_pwr[j][k]=null; 
					else { if (line_smooth==true) {  //here we calculate the average by fitting a line in log-log space, then picking the predicted value from the line
						meany=meany/sumwt;
						meanx=meanx/sumwt;
						meanxy=meanxy/sumwt;
						meanxx=meanxx/sumwt;
						sm_pwr[j][k]=meany - meanx * (meanxy-meanx*meany)/(meanxx-meanx*meanx);
						sm_pwr[j][k]=pow(10.0, (double) sm_pwr[j][k]); } 
					else sm_pwr[j][k]=wtd_sum/sumwt; } //store in temp array
			} //loop over wavelengths
		} //loop over columns
	} //if (smooth_over_bins==true)



	
	if (smooth_over_logs==true) {  // here we smooth over a given log width rather than a given number of bins.  
							// This allows us to sample evenly in frequency space (so each unit of f gets equal weight, as it should), 
							// but average over more noise at high frequencies
		if ( ! (logf = calloc(n_freq , sizeof(FTYPE)) )) {fprintf(stderr, "out of memory allocating logf!\n"); exit(0);}
		for (k=0; k<n_freq; k=k+1) logf[k]=log10(freq[k]);
		
		for (j=0; j<n_columns; j++) {
			for (i=0; i<n_freq; i++) 
				{ if (smooth_over_bins==true) xxsignal[j][i]=sm_pwr[j][i]; else xxsignal[j][i]=results[j][i];}}
		//we're just re-using xxsignal as a way to hold the bin-smoothed power (if it's been smoothed that way already) and then smooth it again over a fixed interval of log-freq.
		//and otherwise just smooth the raw results

		for (j=0; j<n_columns; j++) {	//step through read_columns
			kmin=0; kmax=0;	//initialize limiting k's
			
			for (k=0; k<n_freq; k=k+1) {	//step through frequencies
				i=kmin-1;		//find new first point within range
				do { i=i+1; } while ( (logf[i]-logf[k]<-2*log_smooth_width) && (i<n_freq-2) );
				kmin=i;
				i=kmax-1;	//find new last point within range
				do { i=i+1; } while ( (logf[i]-logf[k]<2*log_smooth_width) && (i<n_freq-2) );
				kmax=i;

				sumwt=0.0;	//step through range and do weighted sum
				wtd_sum=0.0;  meanx=0.0; meanxx=0.0; meany=0.0; meanxy=0.0;
				for (i=kmin; i<=kmax; i++) if ( (i>=0) && (i<n_freq) ) {
					if (xxsignal[j][i] != null) {
	//					weight=exp(-sqr((logf[i]-logf[k])/(0.5*log_smooth_width)));  //gaussian smooth... there should be scaling constants in here but they're irrelevant because we normalize by sumwt anyhow (so it's only relative weight that matters)
						if (triangular_smooth==true) 
								{weight=1.0-fabs((logf[i]-logf[k])/(FTYPE) log_smooth_width);  //triangular smooth... there should be scaling constants in here but they're irrelevant because we normalize by sumwt anyhow (so it's only relative weight that matters)
									if (weight<0.0) weight=0.0;} //this is a kludge to deal with the fact that the range of i will be wider than log_smooth_width... hence the line above would give negative values
								else weight=exp(-sqr((logf[i]-logf[k])/(0.5*log_smooth_width)));  //gaussian smooth... there should be scaling constants in here but they're irrelevant because we normalize by sumwt anyhow (so it's only relative weight that matters)
						sumwt=sumwt+weight;
						wtd_sum=wtd_sum+weight*xxsignal[j][i];
						meanx=meanx+(logf[i]-logf[k])*weight;
						meanxx=meanxx+(logf[i]-logf[k])*(logf[i]-logf[k])*weight;
						meany=meany+weight*log10(xxsignal[j][i]);
						meanxy=meanxy+weight*(logf[i]-logf[k])*log10(xxsignal[j][i]);
					}
				} // loop i's
				if ((sumwt==0.0) || (xxsignal[j][k]==null)) sm_pwr[j][k]=null; 
					else { if (line_smooth==true) {  //here we calculate the average by fitting a line in log-log space, then picking the predicted value from the line
						meany=meany/sumwt;
						meanx=meanx/sumwt;
						meanxy=meanxy/sumwt;
						meanxx=meanxx/sumwt;
						sm_pwr[j][k]=meany - meanx * (meanxy-meanx*meany)/(meanxx-meanx*meanx);
						sm_pwr[j][k]=pow(10.0, (double) sm_pwr[j][k]); } 
					else sm_pwr[j][k]=wtd_sum/sumwt; } //store in temp array
			} //loop over wavelengths
		} //loop over columns
		
		free(logf);
		
	
	}


if (do_logfile==true) fclose(outfile);

if (do_alias_filtering==true) {
	//here we do the alias filtering (on the smoothed power spectrum)
	for (j=0; j<n_columns; j++) {
//		if (c==0.0) for (i=0; i<n_freq; i++) raw[i]=sm_pwr[j][i]; //use the smoothed spectrum for FT's
//			else for (i=0; i<n_freq; i++) raw[i]=results[j][i]; //but use the unsmoothed spectrum with wavelets
		for (i=0; i<n_freq; i++) raw[i]=sm_pwr[j][i]; //use smoothed spectrum for all alias-filtering
		alias_filter(freq, raw, n_freq, fs, fc, fcutoff, avgs, filtered, model, aliased, &alpha, &alpha_se, &regr_slope, &regr_slope_se);  //do alias filtering here
		for (i=0; i<n_freq; i++) {
			filtered_pwr[j][i]=filtered[i];
			save_model[j][i]=model[i];
			save_aliased[j][i]=aliased[i];
			save_alpha[j]=alpha; 
			save_alpha_se[j]=alpha_se;
			save_regr_alpha[j]=-regr_slope; 
			save_regr_alpha_se[j]=regr_slope_se;
		}

	fprintf(stderr, "alias filtered column %i  alpha=%g  s.e=%g   %s\n", j+1, alpha, alpha_se, header[j]);

	} //next j
} //if


	fprintf(stderr, "analysis done... elapsed time %g seconds\n", difftime( time(NULL) , start_time ) );

	print_results( );
	
	
//	fclose(locfile);
//	fclose(pointfile);

	
	if (save_benchmarks==true) {
		free(original_t);
		for (j=0; j<n_columns; j++) free(benchmark[j]);
	}

	return 0;
}

